/********************************************************************************
** Form generated from reading UI file 'ChangeDewarpingDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHANGEDEWARPINGDIALOG_H
#define UI_CHANGEDEWARPINGDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace output {

class Ui_ChangeDewarpingDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *dpiGroup;
    QVBoxLayout *verticalLayout_4;
    QRadioButton *offRB;
    QRadioButton *autoRB;
    QRadioButton *marginalRB;
    QRadioButton *manualRB;
    QGroupBox *dewarpingOptionsGroup;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *dewarpingPostDeskewCB;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *thisPageRB;
    QRadioButton *allPagesRB;
    QRadioButton *thisPageAndFollowersRB;
    QWidget *selectedPagesWidget;
    QVBoxLayout *verticalLayout_3;
    QRadioButton *selectedPagesRB;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_5;
    QLabel *selectedPagesHint;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *output__ChangeDewarpingDialog)
    {
        if (output__ChangeDewarpingDialog->objectName().isEmpty())
            output__ChangeDewarpingDialog->setObjectName("output__ChangeDewarpingDialog");
        output__ChangeDewarpingDialog->setWindowModality(Qt::WindowModal);
        output__ChangeDewarpingDialog->resize(320, 371);
        verticalLayout = new QVBoxLayout(output__ChangeDewarpingDialog);
        verticalLayout->setObjectName("verticalLayout");
        dpiGroup = new QGroupBox(output__ChangeDewarpingDialog);
        dpiGroup->setObjectName("dpiGroup");
        verticalLayout_4 = new QVBoxLayout(dpiGroup);
        verticalLayout_4->setObjectName("verticalLayout_4");
        offRB = new QRadioButton(dpiGroup);
        offRB->setObjectName("offRB");
        offRB->setChecked(true);

        verticalLayout_4->addWidget(offRB);

        autoRB = new QRadioButton(dpiGroup);
        autoRB->setObjectName("autoRB");
        autoRB->setEnabled(true);

        verticalLayout_4->addWidget(autoRB);

        marginalRB = new QRadioButton(dpiGroup);
        marginalRB->setObjectName("marginalRB");
        marginalRB->setEnabled(true);

        verticalLayout_4->addWidget(marginalRB);

        manualRB = new QRadioButton(dpiGroup);
        manualRB->setObjectName("manualRB");

        verticalLayout_4->addWidget(manualRB);


        verticalLayout->addWidget(dpiGroup);

        dewarpingOptionsGroup = new QGroupBox(output__ChangeDewarpingDialog);
        dewarpingOptionsGroup->setObjectName("dewarpingOptionsGroup");
        verticalLayout_5 = new QVBoxLayout(dewarpingOptionsGroup);
        verticalLayout_5->setObjectName("verticalLayout_5");
        dewarpingPostDeskewCB = new QCheckBox(dewarpingOptionsGroup);
        dewarpingPostDeskewCB->setObjectName("dewarpingPostDeskewCB");

        verticalLayout_5->addWidget(dewarpingPostDeskewCB);


        verticalLayout->addWidget(dewarpingOptionsGroup);

        groupBox_3 = new QGroupBox(output__ChangeDewarpingDialog);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_2 = new QVBoxLayout(groupBox_3);
        verticalLayout_2->setObjectName("verticalLayout_2");
        thisPageRB = new QRadioButton(groupBox_3);
        thisPageRB->setObjectName("thisPageRB");
        thisPageRB->setChecked(true);

        verticalLayout_2->addWidget(thisPageRB);

        allPagesRB = new QRadioButton(groupBox_3);
        allPagesRB->setObjectName("allPagesRB");

        verticalLayout_2->addWidget(allPagesRB);

        thisPageAndFollowersRB = new QRadioButton(groupBox_3);
        thisPageAndFollowersRB->setObjectName("thisPageAndFollowersRB");

        verticalLayout_2->addWidget(thisPageAndFollowersRB);

        selectedPagesWidget = new QWidget(groupBox_3);
        selectedPagesWidget->setObjectName("selectedPagesWidget");
        verticalLayout_3 = new QVBoxLayout(selectedPagesWidget);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        selectedPagesRB = new QRadioButton(selectedPagesWidget);
        selectedPagesRB->setObjectName("selectedPagesRB");

        verticalLayout_3->addWidget(selectedPagesRB);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_5 = new QSpacerItem(30, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_5);

        selectedPagesHint = new QLabel(selectedPagesWidget);
        selectedPagesHint->setObjectName("selectedPagesHint");
        QFont font;
        font.setPointSize(7);
        selectedPagesHint->setFont(font);

        horizontalLayout_2->addWidget(selectedPagesHint);


        verticalLayout_3->addLayout(horizontalLayout_2);


        verticalLayout_2->addWidget(selectedPagesWidget);


        verticalLayout->addWidget(groupBox_3);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(output__ChangeDewarpingDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(output__ChangeDewarpingDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, output__ChangeDewarpingDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(output__ChangeDewarpingDialog);
    } // setupUi

    void retranslateUi(QDialog *output__ChangeDewarpingDialog)
    {
        output__ChangeDewarpingDialog->setWindowTitle(QCoreApplication::translate("output::ChangeDewarpingDialog", "Apply Dewarping Mode", nullptr));
        dpiGroup->setTitle(QCoreApplication::translate("output::ChangeDewarpingDialog", "Mode", nullptr));
        offRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "Off", nullptr));
        autoRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "Auto (experimental)", nullptr));
        marginalRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "Marginal (experimental)", nullptr));
        manualRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "Manual", nullptr));
        dewarpingOptionsGroup->setTitle(QCoreApplication::translate("output::ChangeDewarpingDialog", "Options", nullptr));
        dewarpingPostDeskewCB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "Post deskew", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("output::ChangeDewarpingDialog", "Apply to", nullptr));
        thisPageRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "This page only", nullptr));
        allPagesRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "All pages", nullptr));
        thisPageAndFollowersRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "This page and the following ones", nullptr));
        selectedPagesRB->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "Selected pages", nullptr));
        selectedPagesHint->setText(QCoreApplication::translate("output::ChangeDewarpingDialog", "Use Ctrl+Click / Shift+Click to select multiple pages.", nullptr));
    } // retranslateUi

};

} // namespace output

namespace output {
namespace Ui {
    class ChangeDewarpingDialog: public Ui_ChangeDewarpingDialog {};
} // namespace Ui
} // namespace output

#endif // UI_CHANGEDEWARPINGDIALOG_H
