/****************************************************************************
** Meta object code from reading C++ file 'OptionsWidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../scantailor-advanced-1.1.1/src/core/filters/select_content/OptionsWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'OptionsWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14select_content13OptionsWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto select_content::OptionsWidget::qt_create_metaobjectdata<qt_meta_tag_ZN14select_content13OptionsWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "select_content::OptionsWidget",
        "pageRectChangedLocally",
        "",
        "QRectF",
        "pageRect",
        "pageRectStateChanged",
        "state",
        "manualContentRectSet",
        "contentRect",
        "manualPageRectSet",
        "updatePageRectSize",
        "QSizeF",
        "size",
        "showApplyToDialog",
        "applySelection",
        "std::set<PageId>",
        "pages",
        "applyContentBox",
        "applyPageBox",
        "contentDetectToggled",
        "AutoManualMode",
        "mode",
        "pageDetectToggled",
        "fineTuningChanged",
        "checked",
        "dimensionsChangedLocally"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'pageRectChangedLocally'
        QtMocHelpers::SignalData<void(const QRectF &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'pageRectStateChanged'
        QtMocHelpers::SignalData<void(bool)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 6 },
        }}),
        // Slot 'manualContentRectSet'
        QtMocHelpers::SlotData<void(const QRectF &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 8 },
        }}),
        // Slot 'manualPageRectSet'
        QtMocHelpers::SlotData<void(const QRectF &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'updatePageRectSize'
        QtMocHelpers::SlotData<void(const QSizeF &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'showApplyToDialog'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applySelection'
        QtMocHelpers::SlotData<void(const std::set<PageId> &, bool, bool)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { QMetaType::Bool, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'contentDetectToggled'
        QtMocHelpers::SlotData<void(AutoManualMode)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'pageDetectToggled'
        QtMocHelpers::SlotData<void(AutoManualMode)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'fineTuningChanged'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 24 },
        }}),
        // Slot 'dimensionsChangedLocally'
        QtMocHelpers::SlotData<void(double)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 2 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OptionsWidget, qt_meta_tag_ZN14select_content13OptionsWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject select_content::OptionsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<FilterOptionsWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14select_content13OptionsWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14select_content13OptionsWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14select_content13OptionsWidgetE_t>.metaTypes,
    nullptr
} };

void select_content::OptionsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OptionsWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->pageRectChangedLocally((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 1: _t->pageRectStateChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->manualContentRectSet((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 3: _t->manualPageRectSet((*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[1]))); break;
        case 4: _t->updatePageRectSize((*reinterpret_cast<std::add_pointer_t<QSizeF>>(_a[1]))); break;
        case 5: _t->showApplyToDialog(); break;
        case 6: _t->applySelection((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 7: _t->contentDetectToggled((*reinterpret_cast<std::add_pointer_t<AutoManualMode>>(_a[1]))); break;
        case 8: _t->pageDetectToggled((*reinterpret_cast<std::add_pointer_t<AutoManualMode>>(_a[1]))); break;
        case 9: _t->fineTuningChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->dimensionsChangedLocally((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(const QRectF & )>(_a, &OptionsWidget::pageRectChangedLocally, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(bool )>(_a, &OptionsWidget::pageRectStateChanged, 1))
            return;
    }
}

const QMetaObject *select_content::OptionsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *select_content::OptionsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14select_content13OptionsWidgetE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "UnitsListener"))
        return static_cast< UnitsListener*>(this);
    return FilterOptionsWidget::qt_metacast(_clname);
}

int select_content::OptionsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FilterOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void select_content::OptionsWidget::pageRectChangedLocally(const QRectF & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void select_content::OptionsWidget::pageRectStateChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
