// Copyright (C) 2019  Joseph Artsimovich <joseph.artsimovich@gmail.com>, 4lex4 <4lex49@zoho.com>
// Use of this source code is governed by the GNU GPLv3 license that can be found in the LICENSE file.

#include "Utils.h"

namespace foundation {
QString Utils::doubleToString(double val) {
  return QString::number(val, 'g', 16);
}
}  // namespace foundation