// Copyright (C) 2019  Joseph Artsimovich <joseph.artsimovich@gmail.com>, 4lex4 <4lex49@zoho.com>
// Use of this source code is governed by the GNU GPLv3 license that can be found in the LICENSE file.

#ifndef SCANTAILOR_OUTPUT_PICTUREZONEPROPFACTORY_H_
#define SCANTAILOR_OUTPUT_PICTUREZONEPROPFACTORY_H_

#include "PropertyFactory.h"

namespace output {
class PictureZonePropFactory : public PropertyFactory {
 public:
  PictureZonePropFactory();
};
}  // namespace output
#endif
