/********************************************************************************
** Form generated from reading UI file 'StatusBarPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STATUSBARPANEL_H
#define UI_STATUSBARPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_StatusBarPanel
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *zoneModeLabel;
    QFrame *zoneModeLine;
    QLabel *mousePosLabel;
    QFrame *mousePosLine;
    QLabel *physSizeLabel;
    QFrame *physSizeLine;
    QLabel *pageNoLabel;
    QFrame *pageInfoLine;
    QLabel *pageInfoLabel;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *StatusBarPanel)
    {
        if (StatusBarPanel->objectName().isEmpty())
            StatusBarPanel->setObjectName("StatusBarPanel");
        StatusBarPanel->resize(300, 16);
        horizontalLayout = new QHBoxLayout(StatusBarPanel);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        zoneModeLabel = new QLabel(StatusBarPanel);
        zoneModeLabel->setObjectName("zoneModeLabel");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(zoneModeLabel->sizePolicy().hasHeightForWidth());
        zoneModeLabel->setSizePolicy(sizePolicy);
        zoneModeLabel->setMinimumSize(QSize(16, 16));
        zoneModeLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(zoneModeLabel);

        zoneModeLine = new QFrame(StatusBarPanel);
        zoneModeLine->setObjectName("zoneModeLine");
        zoneModeLine->setFrameShape(QFrame::Shape::VLine);
        zoneModeLine->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(zoneModeLine);

        mousePosLabel = new QLabel(StatusBarPanel);
        mousePosLabel->setObjectName("mousePosLabel");
        sizePolicy.setHeightForWidth(mousePosLabel->sizePolicy().hasHeightForWidth());
        mousePosLabel->setSizePolicy(sizePolicy);
        mousePosLabel->setMinimumSize(QSize(0, 0));
        mousePosLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(mousePosLabel);

        mousePosLine = new QFrame(StatusBarPanel);
        mousePosLine->setObjectName("mousePosLine");
        mousePosLine->setFrameShape(QFrame::Shape::VLine);
        mousePosLine->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(mousePosLine);

        physSizeLabel = new QLabel(StatusBarPanel);
        physSizeLabel->setObjectName("physSizeLabel");
        sizePolicy.setHeightForWidth(physSizeLabel->sizePolicy().hasHeightForWidth());
        physSizeLabel->setSizePolicy(sizePolicy);
        physSizeLabel->setMinimumSize(QSize(0, 0));
        physSizeLabel->setAlignment(Qt::AlignCenter);
        physSizeLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        horizontalLayout->addWidget(physSizeLabel);

        physSizeLine = new QFrame(StatusBarPanel);
        physSizeLine->setObjectName("physSizeLine");
        physSizeLine->setFrameShape(QFrame::Shape::VLine);
        physSizeLine->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(physSizeLine);

        pageNoLabel = new QLabel(StatusBarPanel);
        pageNoLabel->setObjectName("pageNoLabel");
        sizePolicy.setHeightForWidth(pageNoLabel->sizePolicy().hasHeightForWidth());
        pageNoLabel->setSizePolicy(sizePolicy);
        pageNoLabel->setMinimumSize(QSize(0, 0));
        pageNoLabel->setAlignment(Qt::AlignCenter);
        pageNoLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse);

        horizontalLayout->addWidget(pageNoLabel);

        pageInfoLine = new QFrame(StatusBarPanel);
        pageInfoLine->setObjectName("pageInfoLine");
        pageInfoLine->setFrameShape(QFrame::Shape::VLine);
        pageInfoLine->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(pageInfoLine);

        pageInfoLabel = new QLabel(StatusBarPanel);
        pageInfoLabel->setObjectName("pageInfoLabel");
        sizePolicy.setHeightForWidth(pageInfoLabel->sizePolicy().hasHeightForWidth());
        pageInfoLabel->setSizePolicy(sizePolicy);
        pageInfoLabel->setMinimumSize(QSize(0, 0));
        pageInfoLabel->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(pageInfoLabel);

        horizontalSpacer_2 = new QSpacerItem(10, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        retranslateUi(StatusBarPanel);

        QMetaObject::connectSlotsByName(StatusBarPanel);
    } // setupUi

    void retranslateUi(QWidget *StatusBarPanel)
    {
        StatusBarPanel->setWindowTitle(QCoreApplication::translate("StatusBarPanel", "Form", nullptr));
#if QT_CONFIG(tooltip)
        zoneModeLabel->setToolTip(QCoreApplication::translate("StatusBarPanel", "Zone creation mode.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        zoneModeLabel->setStatusTip(QCoreApplication::translate("StatusBarPanel", "The type of a new zone created.", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(tooltip)
        mousePosLabel->setToolTip(QCoreApplication::translate("StatusBarPanel", "Mouse position.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        mousePosLabel->setStatusTip(QCoreApplication::translate("StatusBarPanel", "Mouse position relative to page.", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(tooltip)
        physSizeLabel->setToolTip(QCoreApplication::translate("StatusBarPanel", "Image size.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        physSizeLabel->setStatusTip(QCoreApplication::translate("StatusBarPanel", "Image size in current units.", nullptr));
#endif // QT_CONFIG(statustip)
#if QT_CONFIG(tooltip)
        pageNoLabel->setToolTip(QCoreApplication::translate("StatusBarPanel", "Page number.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        pageNoLabel->setStatusTip(QCoreApplication::translate("StatusBarPanel", "Position of the selected page in current order.", nullptr));
#endif // QT_CONFIG(statustip)
        pageNoLabel->setText(QString());
#if QT_CONFIG(tooltip)
        pageInfoLabel->setToolTip(QCoreApplication::translate("StatusBarPanel", "Page information.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        pageInfoLabel->setStatusTip(QCoreApplication::translate("StatusBarPanel", "Page name and type.", nullptr));
#endif // QT_CONFIG(statustip)
        pageInfoLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class StatusBarPanel: public Ui_StatusBarPanel {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STATUSBARPANEL_H
