/********************************************************************************
** Form generated from reading UI file 'AboutDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QGridLayout *gridLayout;
    QLabel *label_4;
    QSpacerItem *verticalSpacer;
    QLabel *label_3;
    QLabel *label;
    QLabel *version;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QLabel *label_2;
    QLabel *label_5;
    QLabel *label_8;
    QLabel *label_9;
    QLabel *label_6;
    QVBoxLayout *verticalLayout_5;
    QLabel *label_12;
    QLabel *label_13;
    QLabel *label_14;
    QLabel *label_15;
    QLabel *label_16;
    QLabel *label_29;
    QLabel *label_7;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_4;
    QTextBrowser *licenseViewer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName("AboutDialog");
        AboutDialog->resize(482, 349);
        verticalLayout = new QVBoxLayout(AboutDialog);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(AboutDialog);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setDocumentMode(true);
        tab = new QWidget();
        tab->setObjectName("tab");
        gridLayout = new QGridLayout(tab);
        gridLayout->setObjectName("gridLayout");
        label_4 = new QLabel(tab);
        label_4->setObjectName("label_4");
        label_4->setWordWrap(true);

        gridLayout->addWidget(label_4, 3, 0, 1, 2);

        verticalSpacer = new QSpacerItem(440, 127, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 4, 0, 1, 2);

        label_3 = new QLabel(tab);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setStyleSheet(QString::fromUtf8("font-size: 16pt;\n"
"font-weight: bold;"));
        label_3->setText(QString::fromUtf8("ScanTailor Advanced"));
        label_3->setAlignment(Qt::AlignBottom|Qt::AlignHCenter);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        label = new QLabel(tab);
        label->setObjectName("label");
        label->setMaximumSize(QSize(100, 100));
        label->setText(QString::fromUtf8(""));
        label->setPixmap(QPixmap(QString::fromUtf8(":/appicon.svg")));
        label->setScaledContents(true);

        gridLayout->addWidget(label, 0, 1, 2, 1);

        version = new QLabel(tab);
        version->setObjectName("version");
        version->setStyleSheet(QString::fromUtf8("font-size:10pt;\n"
"font-weight:bold;"));
        version->setText(QString::fromUtf8("version"));
        version->setAlignment(Qt::AlignHCenter|Qt::AlignTop);

        gridLayout->addWidget(version, 1, 0, 1, 1);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_3 = new QVBoxLayout(tab_2);
        verticalLayout_3->setObjectName("verticalLayout_3");
        scrollArea = new QScrollArea(tab_2);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 444, 261));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName("verticalLayout_2");
        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName("label_2");
        label_2->setStyleSheet(QString::fromUtf8("font-weight: bold;"));

        verticalLayout_2->addWidget(label_2);

        label_5 = new QLabel(scrollAreaWidgetContents);
        label_5->setObjectName("label_5");

        verticalLayout_2->addWidget(label_5);

        label_8 = new QLabel(scrollAreaWidgetContents);
        label_8->setObjectName("label_8");
        label_8->setStyleSheet(QString::fromUtf8("font-weight: bold;"));

        verticalLayout_2->addWidget(label_8);

        label_9 = new QLabel(scrollAreaWidgetContents);
        label_9->setObjectName("label_9");

        verticalLayout_2->addWidget(label_9);

        label_6 = new QLabel(scrollAreaWidgetContents);
        label_6->setObjectName("label_6");
        label_6->setStyleSheet(QString::fromUtf8("font-weight: bold;"));

        verticalLayout_2->addWidget(label_6);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setSpacing(0);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(-1, 0, -1, -1);
        label_12 = new QLabel(scrollAreaWidgetContents);
        label_12->setObjectName("label_12");

        verticalLayout_5->addWidget(label_12);

        label_13 = new QLabel(scrollAreaWidgetContents);
        label_13->setObjectName("label_13");

        verticalLayout_5->addWidget(label_13);

        label_14 = new QLabel(scrollAreaWidgetContents);
        label_14->setObjectName("label_14");

        verticalLayout_5->addWidget(label_14);

        label_15 = new QLabel(scrollAreaWidgetContents);
        label_15->setObjectName("label_15");

        verticalLayout_5->addWidget(label_15);

        label_16 = new QLabel(scrollAreaWidgetContents);
        label_16->setObjectName("label_16");

        verticalLayout_5->addWidget(label_16);

        label_29 = new QLabel(scrollAreaWidgetContents);
        label_29->setObjectName("label_29");

        verticalLayout_5->addWidget(label_29);

        label_7 = new QLabel(scrollAreaWidgetContents);
        label_7->setObjectName("label_7");

        verticalLayout_5->addWidget(label_7);


        verticalLayout_2->addLayout(verticalLayout_5);

        verticalSpacer_2 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_4 = new QVBoxLayout(tab_3);
        verticalLayout_4->setObjectName("verticalLayout_4");
        licenseViewer = new QTextBrowser(tab_3);
        licenseViewer->setObjectName("licenseViewer");
        licenseViewer->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:'Ubuntu'; font-size:11pt;\"><br /></p></body></html>"));
        licenseViewer->setOpenExternalLinks(true);

        verticalLayout_4->addWidget(licenseViewer);

        tabWidget->addTab(tab_3, QString());

        verticalLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(AboutDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(AboutDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::clicked, AboutDialog, qOverload<>(&QDialog::accept));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "ScanTailor Advanced", nullptr));
        label_4->setText(QCoreApplication::translate("AboutDialog", "Scan Tailor is an interactive post-processing tool for scanned pages. It performs operations such as page splitting, skew correction, adding/removing margins, and others. You give it raw scans, and you get pages ready to be printed or assembled into a PDF or DJVU file.  Scanning and optical character recognition is out of scope of this project.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("AboutDialog", "About", nullptr));
        label_2->setText(QCoreApplication::translate("AboutDialog", "Lead Developer of original version", nullptr));
        label_5->setText(QCoreApplication::translate("AboutDialog", "Joseph Artsimovich", nullptr));
        label_8->setText(QCoreApplication::translate("AboutDialog", "Lead Developer of Advanced version", nullptr));
        label_9->setText(QCoreApplication::translate("AboutDialog", "4lex4", nullptr));
        label_6->setText(QCoreApplication::translate("AboutDialog", "Contributors", nullptr));
        label_12->setText(QCoreApplication::translate("AboutDialog", "U235 - Picture auto-detection algorithm.", nullptr));
        label_13->setText(QCoreApplication::translate("AboutDialog", "Robert B. - First generation dewarping algorithm.", nullptr));
        label_14->setText(QCoreApplication::translate("AboutDialog", "Andrey Bergman - System load adjustment.", nullptr));
        label_15->setText(QCoreApplication::translate("AboutDialog", "Petr Kov\303\241\305\231 - ver. Enhanced", nullptr));
        label_16->setText(QCoreApplication::translate("AboutDialog", "Vadim Kuznetsov - ver. Plus", nullptr));
        label_29->setText(QCoreApplication::translate("AboutDialog", "monday2000 - ver. Featured", nullptr));
        label_7->setText(QCoreApplication::translate("AboutDialog", "Alexander Trufanov - ver. Universal", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("AboutDialog", "Authors", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("AboutDialog", "License", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
