/********************************************************************************
** Form generated from reading UI file 'ErrorWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ERRORWIDGET_H
#define UI_ERRORWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ErrorWidget
{
public:
    QGridLayout *gridLayout;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;
    QGroupBox *groupBox;
    QHBoxLayout *hboxLayout;
    QLabel *imageLabel;
    QLabel *textLabel;
    QSpacerItem *spacerItem2;
    QSpacerItem *spacerItem3;

    void setupUi(QWidget *ErrorWidget)
    {
        if (ErrorWidget->objectName().isEmpty())
            ErrorWidget->setObjectName("ErrorWidget");
        ErrorWidget->resize(368, 208);
        gridLayout = new QGridLayout(ErrorWidget);
        gridLayout->setObjectName("gridLayout");
        spacerItem = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 0, 1, 1, 1);

        spacerItem1 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem1, 1, 0, 1, 1);

        groupBox = new QGroupBox(ErrorWidget);
        groupBox->setObjectName("groupBox");
        groupBox->setAlignment(Qt::AlignHCenter);
        hboxLayout = new QHBoxLayout(groupBox);
        hboxLayout->setObjectName("hboxLayout");
        imageLabel = new QLabel(groupBox);
        imageLabel->setObjectName("imageLabel");
        imageLabel->setText(QString::fromUtf8("ImageLabel"));

        hboxLayout->addWidget(imageLabel);

        textLabel = new QLabel(groupBox);
        textLabel->setObjectName("textLabel");
        textLabel->setText(QString::fromUtf8("TextLabel"));
        textLabel->setTextFormat(Qt::AutoText);
        textLabel->setWordWrap(true);
        textLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse);

        hboxLayout->addWidget(textLabel);


        gridLayout->addWidget(groupBox, 1, 1, 1, 1);

        spacerItem2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(spacerItem2, 1, 2, 1, 1);

        spacerItem3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem3, 2, 1, 1, 1);


        retranslateUi(ErrorWidget);

        QMetaObject::connectSlotsByName(ErrorWidget);
    } // setupUi

    void retranslateUi(QWidget *ErrorWidget)
    {
        ErrorWidget->setWindowTitle(QCoreApplication::translate("ErrorWidget", "Form", nullptr));
        groupBox->setTitle(QString());
    } // retranslateUi

};

namespace Ui {
    class ErrorWidget: public Ui_ErrorWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ERRORWIDGET_H
