/****************************************************************************
** Meta object code from reading C++ file 'OptionsWidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../scantailor-advanced-1.1.1/src/core/filters/page_layout/OptionsWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'OptionsWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11page_layout13OptionsWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto page_layout::OptionsWidget::qt_create_metaobjectdata<qt_meta_tag_ZN11page_layout13OptionsWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "page_layout::OptionsWidget",
        "leftRightLinkToggled",
        "",
        "linked",
        "topBottomLinkToggled",
        "alignmentChanged",
        "Alignment",
        "alignment",
        "marginsSetLocally",
        "Margins",
        "marginsMm",
        "aggregateHardSizeChanged",
        "marginsSetExternally",
        "horMarginsChanged",
        "val",
        "vertMarginsChanged",
        "autoMarginsToggled",
        "checked",
        "horizontalAlignmentModeChanged",
        "idx",
        "verticalAlignmentModeChanged",
        "topBottomLinkClicked",
        "leftRightLinkClicked",
        "alignWithOthersToggled",
        "alignmentButtonClicked",
        "showApplyMarginsDialog",
        "showApplyAlignmentDialog",
        "matchSizeToAllPages",
        "onFixDpiClicked",
        "applyMargins",
        "std::set<PageId>",
        "pages",
        "applyLeft",
        "applyRight",
        "applyTop",
        "applyBottom",
        "applyAlignment"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'leftRightLinkToggled'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'topBottomLinkToggled'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'alignmentChanged'
        QtMocHelpers::SignalData<void(const Alignment &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'marginsSetLocally'
        QtMocHelpers::SignalData<void(const Margins &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Signal 'aggregateHardSizeChanged'
        QtMocHelpers::SignalData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'marginsSetExternally'
        QtMocHelpers::SlotData<void(const Margins &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'horMarginsChanged'
        QtMocHelpers::SlotData<void(double)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'vertMarginsChanged'
        QtMocHelpers::SlotData<void(double)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 14 },
        }}),
        // Slot 'autoMarginsToggled'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'horizontalAlignmentModeChanged'
        QtMocHelpers::SlotData<void(int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'verticalAlignmentModeChanged'
        QtMocHelpers::SlotData<void(int)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 19 },
        }}),
        // Slot 'topBottomLinkClicked'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'leftRightLinkClicked'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'alignWithOthersToggled'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'alignmentButtonClicked'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showApplyMarginsDialog'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showApplyAlignmentDialog'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'matchSizeToAllPages'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onFixDpiClicked'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyMargins'
        QtMocHelpers::SlotData<void(const std::set<PageId> &, bool, bool, bool, bool)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { QMetaType::Bool, 32 }, { QMetaType::Bool, 33 }, { QMetaType::Bool, 34 },
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'applyMargins'
        QtMocHelpers::SlotData<void(const std::set<PageId> &, bool, bool, bool)>(29, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { QMetaType::Bool, 32 }, { QMetaType::Bool, 33 }, { QMetaType::Bool, 34 },
        }}),
        // Slot 'applyMargins'
        QtMocHelpers::SlotData<void(const std::set<PageId> &, bool, bool)>(29, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { QMetaType::Bool, 32 }, { QMetaType::Bool, 33 },
        }}),
        // Slot 'applyMargins'
        QtMocHelpers::SlotData<void(const std::set<PageId> &, bool)>(29, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 30, 31 }, { QMetaType::Bool, 32 },
        }}),
        // Slot 'applyMargins'
        QtMocHelpers::SlotData<void(const std::set<PageId> &)>(29, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
        // Slot 'applyAlignment'
        QtMocHelpers::SlotData<void(const std::set<PageId> &)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 30, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OptionsWidget, qt_meta_tag_ZN11page_layout13OptionsWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject page_layout::OptionsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<FilterOptionsWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11page_layout13OptionsWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11page_layout13OptionsWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11page_layout13OptionsWidgetE_t>.metaTypes,
    nullptr
} };

void page_layout::OptionsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OptionsWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->leftRightLinkToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->topBottomLinkToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->alignmentChanged((*reinterpret_cast<std::add_pointer_t<Alignment>>(_a[1]))); break;
        case 3: _t->marginsSetLocally((*reinterpret_cast<std::add_pointer_t<Margins>>(_a[1]))); break;
        case 4: _t->aggregateHardSizeChanged(); break;
        case 5: _t->marginsSetExternally((*reinterpret_cast<std::add_pointer_t<Margins>>(_a[1]))); break;
        case 6: _t->horMarginsChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 7: _t->vertMarginsChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 8: _t->autoMarginsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->horizontalAlignmentModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->verticalAlignmentModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->topBottomLinkClicked(); break;
        case 12: _t->leftRightLinkClicked(); break;
        case 13: _t->alignWithOthersToggled(); break;
        case 14: _t->alignmentButtonClicked(); break;
        case 15: _t->showApplyMarginsDialog(); break;
        case 16: _t->showApplyAlignmentDialog(); break;
        case 17: _t->matchSizeToAllPages(); break;
        case 18: _t->onFixDpiClicked(); break;
        case 19: _t->applyMargins((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 20: _t->applyMargins((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 21: _t->applyMargins((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 22: _t->applyMargins((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 23: _t->applyMargins((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1]))); break;
        case 24: _t->applyAlignment((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(bool )>(_a, &OptionsWidget::leftRightLinkToggled, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(bool )>(_a, &OptionsWidget::topBottomLinkToggled, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(const Alignment & )>(_a, &OptionsWidget::alignmentChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(const Margins & )>(_a, &OptionsWidget::marginsSetLocally, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)()>(_a, &OptionsWidget::aggregateHardSizeChanged, 4))
            return;
    }
}

const QMetaObject *page_layout::OptionsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *page_layout::OptionsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11page_layout13OptionsWidgetE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "UnitsListener"))
        return static_cast< UnitsListener*>(this);
    return FilterOptionsWidget::qt_metacast(_clname);
}

int page_layout::OptionsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FilterOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void page_layout::OptionsWidget::leftRightLinkToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void page_layout::OptionsWidget::topBottomLinkToggled(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void page_layout::OptionsWidget::alignmentChanged(const Alignment & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void page_layout::OptionsWidget::marginsSetLocally(const Margins & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void page_layout::OptionsWidget::aggregateHardSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
