/********************************************************************************
** Form generated from reading UI file 'SplitModeDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SPLITMODEDIALOG_H
#define UI_SPLITMODEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

namespace page_split {

class Ui_SplitModeDialog
{
public:
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *spacerItem;
    QLabel *layoutTypeLabel;
    QSpacerItem *spacerItem1;
    QHBoxLayout *horizontalLayout_3;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout;
    QRadioButton *modeAuto;
    QRadioButton *modeManual;
    QGroupBox *optionsBox;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *applyCutOption;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout;
    QRadioButton *thisPageRB;
    QRadioButton *allPagesRB;
    QRadioButton *thisPageAndFollowersRB;
    QRadioButton *thisEveryOtherRB;
    QRadioButton *everyOtherRB;
    QRadioButton *selectedPagesRB;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_5;
    QLabel *selectedPagesHint;
    QRadioButton *everyOtherSelectedRB;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_6;
    QLabel *everyOtherSelectedHint;
    QSpacerItem *spacerItem2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *page_split__SplitModeDialog)
    {
        if (page_split__SplitModeDialog->objectName().isEmpty())
            page_split__SplitModeDialog->setObjectName("page_split__SplitModeDialog");
        page_split__SplitModeDialog->setWindowModality(Qt::WindowModal);
        page_split__SplitModeDialog->resize(330, 383);
        verticalLayout_4 = new QVBoxLayout(page_split__SplitModeDialog);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        spacerItem = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem);

        layoutTypeLabel = new QLabel(page_split__SplitModeDialog);
        layoutTypeLabel->setObjectName("layoutTypeLabel");
        layoutTypeLabel->setFrameShape(QFrame::StyledPanel);
        layoutTypeLabel->setPixmap(QPixmap(QString::fromUtf8(":/icons/layout_type_auto.svg")));

        horizontalLayout->addWidget(layoutTypeLabel);

        spacerItem1 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(spacerItem1);


        verticalLayout_4->addLayout(horizontalLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        groupBox = new QGroupBox(page_split__SplitModeDialog);
        groupBox->setObjectName("groupBox");
        vboxLayout = new QVBoxLayout(groupBox);
        vboxLayout->setObjectName("vboxLayout");
        modeAuto = new QRadioButton(groupBox);
        modeAuto->setObjectName("modeAuto");
        modeAuto->setChecked(true);

        vboxLayout->addWidget(modeAuto);

        modeManual = new QRadioButton(groupBox);
        modeManual->setObjectName("modeManual");

        vboxLayout->addWidget(modeManual);


        horizontalLayout_3->addWidget(groupBox);

        optionsBox = new QGroupBox(page_split__SplitModeDialog);
        optionsBox->setObjectName("optionsBox");
        verticalLayout_3 = new QVBoxLayout(optionsBox);
        verticalLayout_3->setObjectName("verticalLayout_3");
        applyCutOption = new QCheckBox(optionsBox);
        applyCutOption->setObjectName("applyCutOption");

        verticalLayout_3->addWidget(applyCutOption);

        verticalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        horizontalLayout_3->addWidget(optionsBox);


        verticalLayout_4->addLayout(horizontalLayout_3);

        groupBox_2 = new QGroupBox(page_split__SplitModeDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout = new QVBoxLayout(groupBox_2);
        verticalLayout->setObjectName("verticalLayout");
        thisPageRB = new QRadioButton(groupBox_2);
        thisPageRB->setObjectName("thisPageRB");
        thisPageRB->setChecked(true);

        verticalLayout->addWidget(thisPageRB);

        allPagesRB = new QRadioButton(groupBox_2);
        allPagesRB->setObjectName("allPagesRB");

        verticalLayout->addWidget(allPagesRB);

        thisPageAndFollowersRB = new QRadioButton(groupBox_2);
        thisPageAndFollowersRB->setObjectName("thisPageAndFollowersRB");

        verticalLayout->addWidget(thisPageAndFollowersRB);

        thisEveryOtherRB = new QRadioButton(groupBox_2);
        thisEveryOtherRB->setObjectName("thisEveryOtherRB");

        verticalLayout->addWidget(thisEveryOtherRB);

        everyOtherRB = new QRadioButton(groupBox_2);
        everyOtherRB->setObjectName("everyOtherRB");

        verticalLayout->addWidget(everyOtherRB);

        selectedPagesRB = new QRadioButton(groupBox_2);
        selectedPagesRB->setObjectName("selectedPagesRB");

        verticalLayout->addWidget(selectedPagesRB);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_5 = new QSpacerItem(30, 1, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_5);

        selectedPagesHint = new QLabel(groupBox_2);
        selectedPagesHint->setObjectName("selectedPagesHint");
        QFont font;
        font.setPointSize(7);
        selectedPagesHint->setFont(font);

        horizontalLayout_2->addWidget(selectedPagesHint);


        verticalLayout->addLayout(horizontalLayout_2);

        everyOtherSelectedRB = new QRadioButton(groupBox_2);
        everyOtherSelectedRB->setObjectName("everyOtherSelectedRB");

        verticalLayout->addWidget(everyOtherSelectedRB);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_6 = new QSpacerItem(30, 1, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_6);

        everyOtherSelectedHint = new QLabel(groupBox_2);
        everyOtherSelectedHint->setObjectName("everyOtherSelectedHint");
        everyOtherSelectedHint->setFont(font);

        horizontalLayout_4->addWidget(everyOtherSelectedHint);


        verticalLayout->addLayout(horizontalLayout_4);


        verticalLayout_4->addWidget(groupBox_2);

        spacerItem2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(spacerItem2);

        buttonBox = new QDialogButtonBox(page_split__SplitModeDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_4->addWidget(buttonBox);


        retranslateUi(page_split__SplitModeDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, page_split__SplitModeDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(page_split__SplitModeDialog);
    } // setupUi

    void retranslateUi(QDialog *page_split__SplitModeDialog)
    {
        page_split__SplitModeDialog->setWindowTitle(QCoreApplication::translate("page_split::SplitModeDialog", "Split Pages", nullptr));
        groupBox->setTitle(QCoreApplication::translate("page_split::SplitModeDialog", "Mode", nullptr));
        modeAuto->setText(QCoreApplication::translate("page_split::SplitModeDialog", "Auto", nullptr));
        modeManual->setText(QCoreApplication::translate("page_split::SplitModeDialog", "Manual", nullptr));
        optionsBox->setTitle(QCoreApplication::translate("page_split::SplitModeDialog", "Options", nullptr));
        applyCutOption->setText(QCoreApplication::translate("page_split::SplitModeDialog", "Apply cut", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("page_split::SplitModeDialog", "Apply to", nullptr));
        thisPageRB->setText(QCoreApplication::translate("page_split::SplitModeDialog", "This page only", nullptr));
        allPagesRB->setText(QCoreApplication::translate("page_split::SplitModeDialog", "All pages", nullptr));
        thisPageAndFollowersRB->setText(QCoreApplication::translate("page_split::SplitModeDialog", "This page and the following ones", nullptr));
        thisEveryOtherRB->setText(QCoreApplication::translate("page_split::SplitModeDialog", "This page and the following every other page", nullptr));
        everyOtherRB->setText(QCoreApplication::translate("page_split::SplitModeDialog", "Every other page", nullptr));
        selectedPagesRB->setText(QCoreApplication::translate("page_split::SplitModeDialog", "Selected pages", nullptr));
        selectedPagesHint->setText(QCoreApplication::translate("page_split::SplitModeDialog", "Use Ctrl+Click / Shift+Click to select multiple pages.", nullptr));
        everyOtherSelectedRB->setText(QCoreApplication::translate("page_split::SplitModeDialog", "Every other selected page", nullptr));
        everyOtherSelectedHint->setText(QCoreApplication::translate("page_split::SplitModeDialog", "The current page will be included.", nullptr));
    } // retranslateUi

};

} // namespace page_split

namespace page_split {
namespace Ui {
    class SplitModeDialog: public Ui_SplitModeDialog {};
} // namespace Ui
} // namespace page_split

#endif // UI_SPLITMODEDIALOG_H
