/********************************************************************************
** Form generated from reading UI file 'ProjectFilesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROJECTFILESDIALOG_H
#define UI_PROJECTFILESDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListView>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ProjectFilesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *groupBox_2;
    QHBoxLayout *hboxLayout;
    QLineEdit *inpDirLine;
    QPushButton *inpDirBrowseBtn;
    QGroupBox *groupBox;
    QHBoxLayout *hboxLayout1;
    QLineEdit *outDirLine;
    QPushButton *outDirBrowseBtn;
    QHBoxLayout *hboxLayout2;
    QGroupBox *groupBox_3;
    QVBoxLayout *vboxLayout;
    QListView *offProjectList;
    QPushButton *offProjectSelectAllBtn;
    QVBoxLayout *vboxLayout1;
    QSpacerItem *spacerItem;
    QToolButton *addToProjectBtn;
    QToolButton *removeFromProjectBtn;
    QSpacerItem *spacerItem1;
    QGroupBox *groupBox_4;
    QVBoxLayout *vboxLayout2;
    QListView *inProjectList;
    QPushButton *inProjectSelectAllBtn;
    QCheckBox *rtlLayoutCB;
    QCheckBox *forceFixDpi;
    QProgressBar *progressBar;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ProjectFilesDialog)
    {
        if (ProjectFilesDialog->objectName().isEmpty())
            ProjectFilesDialog->setObjectName("ProjectFilesDialog");
        ProjectFilesDialog->resize(482, 506);
        verticalLayout = new QVBoxLayout(ProjectFilesDialog);
        verticalLayout->setObjectName("verticalLayout");
        groupBox_2 = new QGroupBox(ProjectFilesDialog);
        groupBox_2->setObjectName("groupBox_2");
        hboxLayout = new QHBoxLayout(groupBox_2);
        hboxLayout->setObjectName("hboxLayout");
        inpDirLine = new QLineEdit(groupBox_2);
        inpDirLine->setObjectName("inpDirLine");
        inpDirLine->setEnabled(true);

        hboxLayout->addWidget(inpDirLine);

        inpDirBrowseBtn = new QPushButton(groupBox_2);
        inpDirBrowseBtn->setObjectName("inpDirBrowseBtn");

        hboxLayout->addWidget(inpDirBrowseBtn);


        verticalLayout->addWidget(groupBox_2);

        groupBox = new QGroupBox(ProjectFilesDialog);
        groupBox->setObjectName("groupBox");
        hboxLayout1 = new QHBoxLayout(groupBox);
        hboxLayout1->setObjectName("hboxLayout1");
        outDirLine = new QLineEdit(groupBox);
        outDirLine->setObjectName("outDirLine");

        hboxLayout1->addWidget(outDirLine);

        outDirBrowseBtn = new QPushButton(groupBox);
        outDirBrowseBtn->setObjectName("outDirBrowseBtn");

        hboxLayout1->addWidget(outDirBrowseBtn);


        verticalLayout->addWidget(groupBox);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        groupBox_3 = new QGroupBox(ProjectFilesDialog);
        groupBox_3->setObjectName("groupBox_3");
        vboxLayout = new QVBoxLayout(groupBox_3);
        vboxLayout->setObjectName("vboxLayout");
        offProjectList = new QListView(groupBox_3);
        offProjectList->setObjectName("offProjectList");
        offProjectList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout->addWidget(offProjectList);

        offProjectSelectAllBtn = new QPushButton(groupBox_3);
        offProjectSelectAllBtn->setObjectName("offProjectSelectAllBtn");

        vboxLayout->addWidget(offProjectSelectAllBtn);


        hboxLayout2->addWidget(groupBox_3);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName("vboxLayout1");
        spacerItem = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacerItem);

        addToProjectBtn = new QToolButton(ProjectFilesDialog);
        addToProjectBtn->setObjectName("addToProjectBtn");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(addToProjectBtn->sizePolicy().hasHeightForWidth());
        addToProjectBtn->setSizePolicy(sizePolicy);
        addToProjectBtn->setText(QString::fromUtf8(""));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/right-arrow-inscribed.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        addToProjectBtn->setIcon(icon);
        addToProjectBtn->setIconSize(QSize(20, 20));

        vboxLayout1->addWidget(addToProjectBtn);

        removeFromProjectBtn = new QToolButton(ProjectFilesDialog);
        removeFromProjectBtn->setObjectName("removeFromProjectBtn");
        sizePolicy.setHeightForWidth(removeFromProjectBtn->sizePolicy().hasHeightForWidth());
        removeFromProjectBtn->setSizePolicy(sizePolicy);
        removeFromProjectBtn->setText(QString::fromUtf8(""));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/left-arrow-inscribed.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        removeFromProjectBtn->setIcon(icon1);
        removeFromProjectBtn->setIconSize(QSize(20, 20));

        vboxLayout1->addWidget(removeFromProjectBtn);

        spacerItem1 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacerItem1);


        hboxLayout2->addLayout(vboxLayout1);

        groupBox_4 = new QGroupBox(ProjectFilesDialog);
        groupBox_4->setObjectName("groupBox_4");
        vboxLayout2 = new QVBoxLayout(groupBox_4);
        vboxLayout2->setObjectName("vboxLayout2");
        inProjectList = new QListView(groupBox_4);
        inProjectList->setObjectName("inProjectList");
        inProjectList->setEnabled(true);
        inProjectList->setSelectionMode(QAbstractItemView::ExtendedSelection);

        vboxLayout2->addWidget(inProjectList);

        inProjectSelectAllBtn = new QPushButton(groupBox_4);
        inProjectSelectAllBtn->setObjectName("inProjectSelectAllBtn");

        vboxLayout2->addWidget(inProjectSelectAllBtn);


        hboxLayout2->addWidget(groupBox_4);


        verticalLayout->addLayout(hboxLayout2);

        rtlLayoutCB = new QCheckBox(ProjectFilesDialog);
        rtlLayoutCB->setObjectName("rtlLayoutCB");

        verticalLayout->addWidget(rtlLayoutCB);

        forceFixDpi = new QCheckBox(ProjectFilesDialog);
        forceFixDpi->setObjectName("forceFixDpi");

        verticalLayout->addWidget(forceFixDpi);

        progressBar = new QProgressBar(ProjectFilesDialog);
        progressBar->setObjectName("progressBar");
        progressBar->setValue(0);
        progressBar->setTextVisible(false);

        verticalLayout->addWidget(progressBar);

        buttonBox = new QDialogButtonBox(ProjectFilesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ProjectFilesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ProjectFilesDialog, qOverload<>(&QDialog::reject));
        QObject::connect(offProjectSelectAllBtn, &QPushButton::clicked, offProjectList, qOverload<>(&QListView::selectAll));
        QObject::connect(inProjectSelectAllBtn, &QPushButton::clicked, inProjectList, qOverload<>(&QListView::selectAll));

        QMetaObject::connectSlotsByName(ProjectFilesDialog);
    } // setupUi

    void retranslateUi(QDialog *ProjectFilesDialog)
    {
        ProjectFilesDialog->setWindowTitle(QCoreApplication::translate("ProjectFilesDialog", "Project Files", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("ProjectFilesDialog", "Input Directory", nullptr));
        inpDirBrowseBtn->setText(QCoreApplication::translate("ProjectFilesDialog", "Browse", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ProjectFilesDialog", "Output Directory", nullptr));
        outDirBrowseBtn->setText(QCoreApplication::translate("ProjectFilesDialog", "Browse", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("ProjectFilesDialog", "Files Not In Project", nullptr));
        offProjectSelectAllBtn->setText(QCoreApplication::translate("ProjectFilesDialog", "Select All", nullptr));
#if QT_CONFIG(tooltip)
        addToProjectBtn->setToolTip(QCoreApplication::translate("ProjectFilesDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Add selected files to project.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        removeFromProjectBtn->setToolTip(QCoreApplication::translate("ProjectFilesDialog", "<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Remove selected files from project.</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_4->setTitle(QCoreApplication::translate("ProjectFilesDialog", "Files In Project", nullptr));
        inProjectSelectAllBtn->setText(QCoreApplication::translate("ProjectFilesDialog", "Select All", nullptr));
        rtlLayoutCB->setText(QCoreApplication::translate("ProjectFilesDialog", "Right to left layout (for Hebrew and Arabic)", nullptr));
        forceFixDpi->setText(QCoreApplication::translate("ProjectFilesDialog", "Fix DPIs, even if they look OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ProjectFilesDialog: public Ui_ProjectFilesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROJECTFILESDIALOG_H
