/********************************************************************************
** Form generated from reading UI file 'RelinkingDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RELINKINGDIALOG_H
#define UI_RELINKINGDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <RelinkablePathVisualization.h>
#include <RelinkingListView.h>

QT_BEGIN_NAMESPACE

class Ui_RelinkingDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_3;
    RelinkablePathVisualization *pathVisualization;
    QLabel *errorLabel;
    QToolButton *undoButton;
    RelinkingListView *listView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *RelinkingDialog)
    {
        if (RelinkingDialog->objectName().isEmpty())
            RelinkingDialog->setObjectName("RelinkingDialog");
        RelinkingDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(RelinkingDialog);
        verticalLayout->setSpacing(6);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalLayout_3->setContentsMargins(-1, -1, 0, -1);
        pathVisualization = new RelinkablePathVisualization(RelinkingDialog);
        pathVisualization->setObjectName("pathVisualization");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pathVisualization->sizePolicy().hasHeightForWidth());
        pathVisualization->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(pathVisualization);

        errorLabel = new QLabel(RelinkingDialog);
        errorLabel->setObjectName("errorLabel");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(errorLabel->sizePolicy().hasHeightForWidth());
        errorLabel->setSizePolicy(sizePolicy1);
        QPalette palette;
        QBrush brush(QColor(255, 0, 0, 255));
        brush.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::WindowText, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::WindowText, brush);
        QBrush brush1(QColor(120, 120, 120, 255));
        brush1.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::WindowText, brush1);
        errorLabel->setPalette(palette);
        errorLabel->setText(QString::fromUtf8("error"));
        errorLabel->setWordWrap(true);

        horizontalLayout_3->addWidget(errorLabel);

        undoButton = new QToolButton(RelinkingDialog);
        undoButton->setObjectName("undoButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(undoButton->sizePolicy().hasHeightForWidth());
        undoButton->setSizePolicy(sizePolicy2);
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/undo.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        undoButton->setIcon(icon);

        horizontalLayout_3->addWidget(undoButton);


        verticalLayout->addLayout(horizontalLayout_3);

        listView = new RelinkingListView(RelinkingDialog);
        listView->setObjectName("listView");
        listView->setStyleSheet(QString::fromUtf8("show-decoration-selected: 0;"));
        listView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        listView->setEditTriggers(QAbstractItemView::NoEditTriggers);

        verticalLayout->addWidget(listView);

        buttonBox = new QDialogButtonBox(RelinkingDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(RelinkingDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, RelinkingDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(RelinkingDialog);
    } // setupUi

    void retranslateUi(QDialog *RelinkingDialog)
    {
        RelinkingDialog->setWindowTitle(QCoreApplication::translate("RelinkingDialog", "Relinking", nullptr));
#if QT_CONFIG(tooltip)
        undoButton->setToolTip(QCoreApplication::translate("RelinkingDialog", "Undo", nullptr));
#endif // QT_CONFIG(tooltip)
        undoButton->setText(QCoreApplication::translate("RelinkingDialog", "...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RelinkingDialog: public Ui_RelinkingDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RELINKINGDIALOG_H
