/********************************************************************************
** Form generated from reading UI file 'OptionsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGET_H
#define UI_OPTIONSWIDGET_H

#include <CollapsibleGroupBox.h>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace page_split {

class Ui_OptionsWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    CollapsibleGroupBox *pageLayoutGroup;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QToolButton *singlePageUncutBtn;
    QToolButton *pagePlusOffcutBtn;
    QToolButton *twoPagesBtn;
    QSpacerItem *horizontalSpacer_2;
    QLabel *scopeLabel;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *changeBtn;
    QSpacerItem *spacerItem1;
    CollapsibleGroupBox *splitLineGroup;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *autoBtn;
    QPushButton *manualBtn;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *page_split__OptionsWidget)
    {
        if (page_split__OptionsWidget->objectName().isEmpty())
            page_split__OptionsWidget->setObjectName("page_split__OptionsWidget");
        page_split__OptionsWidget->resize(199, 219);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Ignored, QSizePolicy::Policy::Ignored);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(page_split__OptionsWidget->sizePolicy().hasHeightForWidth());
        page_split__OptionsWidget->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(page_split__OptionsWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        pageLayoutGroup = new CollapsibleGroupBox(page_split__OptionsWidget);
        pageLayoutGroup->setObjectName("pageLayoutGroup");
        pageLayoutGroup->setAlignment(Qt::AlignCenter);
        verticalLayout = new QVBoxLayout(pageLayoutGroup);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        singlePageUncutBtn = new QToolButton(pageLayoutGroup);
        singlePageUncutBtn->setObjectName("singlePageUncutBtn");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/single_page_uncut.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        singlePageUncutBtn->setIcon(icon);
        singlePageUncutBtn->setIconSize(QSize(32, 32));
        singlePageUncutBtn->setCheckable(true);
        singlePageUncutBtn->setChecked(false);
        singlePageUncutBtn->setAutoExclusive(true);

        horizontalLayout->addWidget(singlePageUncutBtn);

        pagePlusOffcutBtn = new QToolButton(pageLayoutGroup);
        pagePlusOffcutBtn->setObjectName("pagePlusOffcutBtn");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/right_page_plus_offcut.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pagePlusOffcutBtn->setIcon(icon1);
        pagePlusOffcutBtn->setIconSize(QSize(32, 32));
        pagePlusOffcutBtn->setCheckable(true);
        pagePlusOffcutBtn->setChecked(false);
        pagePlusOffcutBtn->setAutoExclusive(true);

        horizontalLayout->addWidget(pagePlusOffcutBtn);

        twoPagesBtn = new QToolButton(pageLayoutGroup);
        twoPagesBtn->setObjectName("twoPagesBtn");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/two_pages.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        twoPagesBtn->setIcon(icon2);
        twoPagesBtn->setIconSize(QSize(32, 32));
        twoPagesBtn->setCheckable(true);
        twoPagesBtn->setAutoExclusive(true);

        horizontalLayout->addWidget(twoPagesBtn);

        horizontalSpacer_2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout);

        scopeLabel = new QLabel(pageLayoutGroup);
        scopeLabel->setObjectName("scopeLabel");
        scopeLabel->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(scopeLabel);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        spacerItem = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        changeBtn = new QPushButton(pageLayoutGroup);
        changeBtn->setObjectName("changeBtn");

        hboxLayout->addWidget(changeBtn);

        spacerItem1 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem1);


        verticalLayout->addLayout(hboxLayout);


        verticalLayout_3->addWidget(pageLayoutGroup);

        splitLineGroup = new CollapsibleGroupBox(page_split__OptionsWidget);
        splitLineGroup->setObjectName("splitLineGroup");
        splitLineGroup->setAlignment(Qt::AlignCenter);
        verticalLayout_2 = new QVBoxLayout(splitLineGroup);
        verticalLayout_2->setSpacing(9);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(9, 9, -1, -1);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        autoBtn = new QPushButton(splitLineGroup);
        autoBtn->setObjectName("autoBtn");
        autoBtn->setCheckable(true);
        autoBtn->setChecked(true);
        autoBtn->setAutoExclusive(true);

        horizontalLayout_3->addWidget(autoBtn);

        manualBtn = new QPushButton(splitLineGroup);
        manualBtn->setObjectName("manualBtn");
        manualBtn->setCheckable(true);
        manualBtn->setAutoExclusive(true);

        horizontalLayout_3->addWidget(manualBtn);


        verticalLayout_2->addLayout(horizontalLayout_3);


        verticalLayout_3->addWidget(splitLineGroup);

        verticalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        retranslateUi(page_split__OptionsWidget);

        QMetaObject::connectSlotsByName(page_split__OptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *page_split__OptionsWidget)
    {
        page_split__OptionsWidget->setWindowTitle(QCoreApplication::translate("page_split::OptionsWidget", "Form", nullptr));
        pageLayoutGroup->setTitle(QCoreApplication::translate("page_split::OptionsWidget", "Page Layout", nullptr));
        scopeLabel->setText(QCoreApplication::translate("page_split::OptionsWidget", "?", nullptr));
        changeBtn->setText(QCoreApplication::translate("page_split::OptionsWidget", "Change ...", nullptr));
        splitLineGroup->setTitle(QCoreApplication::translate("page_split::OptionsWidget", "Split Line", nullptr));
        autoBtn->setText(QCoreApplication::translate("page_split::OptionsWidget", "Auto", nullptr));
        manualBtn->setText(QCoreApplication::translate("page_split::OptionsWidget", "Manual", nullptr));
    } // retranslateUi

};

} // namespace page_split

namespace page_split {
namespace Ui {
    class OptionsWidget: public Ui_OptionsWidget {};
} // namespace Ui
} // namespace page_split

#endif // UI_OPTIONSWIDGET_H
