/********************************************************************************
** Form generated from reading UI file 'OptionsWidgetBinarizationSauvola.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGETBINARIZATIONSAUVOLA_H
#define UI_OPTIONSWIDGETBINARIZATIONSAUVOLA_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

namespace output {

class Ui_OptionsWidgetBinarizationSauvola
{
public:
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout;
    QLabel *sauvolaDeltaLabel;
    QDoubleSpinBox *sauvolaDelta;
    QLabel *windowSizeLabel;
    QSpinBox *windowSize;
    QLabel *sauvolaCoefLabel;
    QDoubleSpinBox *sauvolaCoef;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *output__OptionsWidgetBinarizationSauvola)
    {
        if (output__OptionsWidgetBinarizationSauvola->objectName().isEmpty())
            output__OptionsWidgetBinarizationSauvola->setObjectName("output__OptionsWidgetBinarizationSauvola");
        output__OptionsWidgetBinarizationSauvola->resize(138, 48);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(output__OptionsWidgetBinarizationSauvola->sizePolicy().hasHeightForWidth());
        output__OptionsWidgetBinarizationSauvola->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(output__OptionsWidgetBinarizationSauvola);
        horizontalLayout->setSpacing(4);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(0, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        sauvolaDeltaLabel = new QLabel(output__OptionsWidgetBinarizationSauvola);
        sauvolaDeltaLabel->setObjectName("sauvolaDeltaLabel");

        gridLayout->addWidget(sauvolaDeltaLabel, 0, 0, 1, 1);

        sauvolaDelta = new QDoubleSpinBox(output__OptionsWidgetBinarizationSauvola);
        sauvolaDelta->setObjectName("sauvolaDelta");
        sauvolaDelta->setMinimum(-100.000000000000000);
        sauvolaDelta->setMaximum(100.000000000000000);
        sauvolaDelta->setSingleStep(1.000000000000000);

        gridLayout->addWidget(sauvolaDelta, 0, 1, 1, 1);

        windowSizeLabel = new QLabel(output__OptionsWidgetBinarizationSauvola);
        windowSizeLabel->setObjectName("windowSizeLabel");

        gridLayout->addWidget(windowSizeLabel, 1, 0, 1, 1);

        windowSize = new QSpinBox(output__OptionsWidgetBinarizationSauvola);
        windowSize->setObjectName("windowSize");
        windowSize->setMinimum(3);
        windowSize->setMaximum(9999);

        gridLayout->addWidget(windowSize, 1, 1, 1, 1);

        sauvolaCoefLabel = new QLabel(output__OptionsWidgetBinarizationSauvola);
        sauvolaCoefLabel->setObjectName("sauvolaCoefLabel");

        gridLayout->addWidget(sauvolaCoefLabel, 2, 0, 1, 1);

        sauvolaCoef = new QDoubleSpinBox(output__OptionsWidgetBinarizationSauvola);
        sauvolaCoef->setObjectName("sauvolaCoef");
        sauvolaCoef->setMinimum(0.010000000000000);
        sauvolaCoef->setMaximum(9.990000000000000);
        sauvolaCoef->setSingleStep(0.010000000000000);

        gridLayout->addWidget(sauvolaCoef, 2, 1, 1, 1);


        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(0, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        retranslateUi(output__OptionsWidgetBinarizationSauvola);

        QMetaObject::connectSlotsByName(output__OptionsWidgetBinarizationSauvola);
    } // setupUi

    void retranslateUi(QWidget *output__OptionsWidgetBinarizationSauvola)
    {
        output__OptionsWidgetBinarizationSauvola->setWindowTitle(QCoreApplication::translate("output::OptionsWidgetBinarizationSauvola", "Form", nullptr));
        sauvolaDeltaLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationSauvola", "Delta:", nullptr));
#if QT_CONFIG(tooltip)
        sauvolaDelta->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationSauvola", "Default value is 0.", nullptr));
#endif // QT_CONFIG(tooltip)
        windowSizeLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationSauvola", "Window size:", nullptr));
#if QT_CONFIG(tooltip)
        windowSize->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationSauvola", "The dimensions of a pixel neighborhood to consider.", nullptr));
#endif // QT_CONFIG(tooltip)
        sauvolaCoefLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationSauvola", "Coef:", nullptr));
#if QT_CONFIG(tooltip)
        sauvolaCoef->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationSauvola", "Default value is 0.34.", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

} // namespace output

namespace output {
namespace Ui {
    class OptionsWidgetBinarizationSauvola: public Ui_OptionsWidgetBinarizationSauvola {};
} // namespace Ui
} // namespace output

#endif // UI_OPTIONSWIDGETBINARIZATIONSAUVOLA_H
