/********************************************************************************
** Form generated from reading UI file 'LoadFilesStatusDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LOADFILESSTATUSDIALOG_H
#define UI_LOADFILESSTATUSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QPlainTextEdit>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_LoadFilesStatusDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget;
    QWidget *loadedTab;
    QVBoxLayout *verticalLayout;
    QPlainTextEdit *loadedFiles;
    QWidget *failedTab;
    QVBoxLayout *verticalLayout_2;
    QPlainTextEdit *failedFiles;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *LoadFilesStatusDialog)
    {
        if (LoadFilesStatusDialog->objectName().isEmpty())
            LoadFilesStatusDialog->setObjectName("LoadFilesStatusDialog");
        LoadFilesStatusDialog->resize(461, 312);
        verticalLayout_3 = new QVBoxLayout(LoadFilesStatusDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        tabWidget = new QTabWidget(LoadFilesStatusDialog);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setDocumentMode(true);
        loadedTab = new QWidget();
        loadedTab->setObjectName("loadedTab");
        verticalLayout = new QVBoxLayout(loadedTab);
        verticalLayout->setObjectName("verticalLayout");
        loadedFiles = new QPlainTextEdit(loadedTab);
        loadedFiles->setObjectName("loadedFiles");
        QPalette palette;
        QBrush brush(QColor(0, 161, 0, 255));
        brush.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Text, brush);
        palette.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Text, brush);
        QBrush brush1(QColor(120, 120, 120, 255));
        brush1.setStyle(Qt::BrushStyle::SolidPattern);
        palette.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Text, brush1);
        loadedFiles->setPalette(palette);
        QFont font;
        font.setBold(true);
        loadedFiles->setFont(font);
        loadedFiles->setLineWrapMode(QPlainTextEdit::NoWrap);
        loadedFiles->setReadOnly(true);

        verticalLayout->addWidget(loadedFiles);

        tabWidget->addTab(loadedTab, QString());
        failedTab = new QWidget();
        failedTab->setObjectName("failedTab");
        verticalLayout_2 = new QVBoxLayout(failedTab);
        verticalLayout_2->setObjectName("verticalLayout_2");
        failedFiles = new QPlainTextEdit(failedTab);
        failedFiles->setObjectName("failedFiles");
        QPalette palette1;
        QBrush brush2(QColor(198, 18, 18, 255));
        brush2.setStyle(Qt::BrushStyle::SolidPattern);
        palette1.setBrush(QPalette::ColorGroup::Active, QPalette::ColorRole::Text, brush2);
        palette1.setBrush(QPalette::ColorGroup::Inactive, QPalette::ColorRole::Text, brush2);
        palette1.setBrush(QPalette::ColorGroup::Disabled, QPalette::ColorRole::Text, brush1);
        failedFiles->setPalette(palette1);
        failedFiles->setFont(font);
        failedFiles->setLineWrapMode(QPlainTextEdit::NoWrap);
        failedFiles->setReadOnly(true);

        verticalLayout_2->addWidget(failedFiles);

        tabWidget->addTab(failedTab, QString());

        verticalLayout_3->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(LoadFilesStatusDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(buttonBox);


        retranslateUi(LoadFilesStatusDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, LoadFilesStatusDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, LoadFilesStatusDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(LoadFilesStatusDialog);
    } // setupUi

    void retranslateUi(QDialog *LoadFilesStatusDialog)
    {
        LoadFilesStatusDialog->setWindowTitle(QCoreApplication::translate("LoadFilesStatusDialog", "Some files failed to load", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(loadedTab), QCoreApplication::translate("LoadFilesStatusDialog", "Loaded successfully: %1", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(failedTab), QCoreApplication::translate("LoadFilesStatusDialog", "Failed to load: %1", nullptr));
    } // retranslateUi

};

namespace Ui {
    class LoadFilesStatusDialog: public Ui_LoadFilesStatusDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LOADFILESSTATUSDIALOG_H
