/********************************************************************************
** Form generated from reading UI file 'NewOpenProjectPanel.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWOPENPROJECTPANEL_H
#define UI_NEWOPENPROJECTPANEL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_NewOpenProjectPanel
{
public:
    QVBoxLayout *verticalLayout_2;
    QWidget *widget;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_2;
    QLabel *newProjectLabel;
    QSpacerItem *horizontalSpacer;
    QLabel *openProjectLabel;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *recentProjectsGroup;
    QVBoxLayout *verticalLayout_3;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *NewOpenProjectPanel)
    {
        if (NewOpenProjectPanel->objectName().isEmpty())
            NewOpenProjectPanel->setObjectName("NewOpenProjectPanel");
        NewOpenProjectPanel->resize(425, 136);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(NewOpenProjectPanel->sizePolicy().hasHeightForWidth());
        NewOpenProjectPanel->setSizePolicy(sizePolicy);
        NewOpenProjectPanel->setAutoFillBackground(false);
        verticalLayout_2 = new QVBoxLayout(NewOpenProjectPanel);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(7, 7, 7, 7);
        widget = new QWidget(NewOpenProjectPanel);
        widget->setObjectName("widget");
        verticalLayout = new QVBoxLayout(widget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_2 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        newProjectLabel = new QLabel(widget);
        newProjectLabel->setObjectName("newProjectLabel");
        QFont font;
        font.setPointSize(16);
        newProjectLabel->setFont(font);
        newProjectLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(newProjectLabel);

        horizontalSpacer = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        openProjectLabel = new QLabel(widget);
        openProjectLabel->setObjectName("openProjectLabel");
        openProjectLabel->setFont(font);

        horizontalLayout->addWidget(openProjectLabel);

        horizontalSpacer_3 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer_2 = new QSpacerItem(0, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout->addItem(verticalSpacer_2);

        recentProjectsGroup = new QGroupBox(widget);
        recentProjectsGroup->setObjectName("recentProjectsGroup");
        QFont font1;
        font1.setPointSize(16);
        font1.setKerning(true);
        recentProjectsGroup->setFont(font1);
        recentProjectsGroup->setStyleSheet(QString::fromUtf8(""));
        verticalLayout_3 = new QVBoxLayout(recentProjectsGroup);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalSpacer = new QSpacerItem(0, 10, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout_3->addItem(verticalSpacer);


        verticalLayout->addWidget(recentProjectsGroup);


        verticalLayout_2->addWidget(widget);


        retranslateUi(NewOpenProjectPanel);

        QMetaObject::connectSlotsByName(NewOpenProjectPanel);
    } // setupUi

    void retranslateUi(QWidget *NewOpenProjectPanel)
    {
        NewOpenProjectPanel->setWindowTitle(QCoreApplication::translate("NewOpenProjectPanel", "Form", nullptr));
        newProjectLabel->setText(QCoreApplication::translate("NewOpenProjectPanel", "New Project ...", nullptr));
        openProjectLabel->setText(QCoreApplication::translate("NewOpenProjectPanel", "Open Project ...", nullptr));
        recentProjectsGroup->setTitle(QCoreApplication::translate("NewOpenProjectPanel", "Recent Projects", nullptr));
    } // retranslateUi

};

namespace Ui {
    class NewOpenProjectPanel: public Ui_NewOpenProjectPanel {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NEWOPENPROJECTPANEL_H
