/********************************************************************************
** Form generated from reading UI file 'OptionsWidgetBinarizationOtsu.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGETBINARIZATIONOTSU_H
#define UI_OPTIONSWIDGETBINARIZATIONOTSU_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace output {

class Ui_OptionsWidgetBinarizationOtsu
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_45;
    QLabel *thresholLabel;
    QSpacerItem *horizontalSpacer_46;
    QSlider *thresholdSlider;
    QHBoxLayout *horizontalLayout_2;
    QLabel *lighterThresholdLink;
    QToolButton *neutralThresholdBtn;
    QLabel *darkerThresholdLink;

    void setupUi(QWidget *output__OptionsWidgetBinarizationOtsu)
    {
        if (output__OptionsWidgetBinarizationOtsu->objectName().isEmpty())
            output__OptionsWidgetBinarizationOtsu->setObjectName("output__OptionsWidgetBinarizationOtsu");
        output__OptionsWidgetBinarizationOtsu->resize(180, 70);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(output__OptionsWidgetBinarizationOtsu->sizePolicy().hasHeightForWidth());
        output__OptionsWidgetBinarizationOtsu->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(output__OptionsWidgetBinarizationOtsu);
        verticalLayout->setSpacing(4);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalSpacer_45 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_45);

        thresholLabel = new QLabel(output__OptionsWidgetBinarizationOtsu);
        thresholLabel->setObjectName("thresholLabel");
        thresholLabel->setFrameShape(QFrame::NoFrame);

        horizontalLayout->addWidget(thresholLabel);

        horizontalSpacer_46 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_46);


        verticalLayout->addLayout(horizontalLayout);

        thresholdSlider = new QSlider(output__OptionsWidgetBinarizationOtsu);
        thresholdSlider->setObjectName("thresholdSlider");
        thresholdSlider->setMinimum(-30);
        thresholdSlider->setMaximum(30);
        thresholdSlider->setPageStep(5);
        thresholdSlider->setTracking(true);
        thresholdSlider->setOrientation(Qt::Horizontal);
        thresholdSlider->setInvertedAppearance(false);
        thresholdSlider->setInvertedControls(false);
        thresholdSlider->setTickPosition(QSlider::TicksBelow);

        verticalLayout->addWidget(thresholdSlider);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        lighterThresholdLink = new QLabel(output__OptionsWidgetBinarizationOtsu);
        lighterThresholdLink->setObjectName("lighterThresholdLink");

        horizontalLayout_2->addWidget(lighterThresholdLink);

        neutralThresholdBtn = new QToolButton(output__OptionsWidgetBinarizationOtsu);
        neutralThresholdBtn->setObjectName("neutralThresholdBtn");
        neutralThresholdBtn->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        neutralThresholdBtn->setIconSize(QSize(12, 12));
        neutralThresholdBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
        neutralThresholdBtn->setAutoRaise(true);
        neutralThresholdBtn->setArrowType(Qt::UpArrow);

        horizontalLayout_2->addWidget(neutralThresholdBtn);

        darkerThresholdLink = new QLabel(output__OptionsWidgetBinarizationOtsu);
        darkerThresholdLink->setObjectName("darkerThresholdLink");
        darkerThresholdLink->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_2->addWidget(darkerThresholdLink);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(output__OptionsWidgetBinarizationOtsu);

        QMetaObject::connectSlotsByName(output__OptionsWidgetBinarizationOtsu);
    } // setupUi

    void retranslateUi(QWidget *output__OptionsWidgetBinarizationOtsu)
    {
        output__OptionsWidgetBinarizationOtsu->setWindowTitle(QCoreApplication::translate("output::OptionsWidgetBinarizationOtsu", "Form", nullptr));
        thresholLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationOtsu", "0", nullptr));
        lighterThresholdLink->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationOtsu", "Thinner", nullptr));
        neutralThresholdBtn->setText(QString());
        darkerThresholdLink->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationOtsu", "Thicker", nullptr));
    } // retranslateUi

};

} // namespace output

namespace output {
namespace Ui {
    class OptionsWidgetBinarizationOtsu: public Ui_OptionsWidgetBinarizationOtsu {};
} // namespace Ui
} // namespace output

#endif // UI_OPTIONSWIDGETBINARIZATIONOTSU_H
