/********************************************************************************
** Form generated from reading UI file 'FixDpiDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FIXDPIDIALOG_H
#define UI_FIXDPIDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_FixDpiDialog
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *vboxLayout1;
    QTreeView *undefinedDpiView;
    QWidget *tab_2;
    QVBoxLayout *vboxLayout2;
    QTreeView *allPagesView;
    QGroupBox *groupBox;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QComboBox *dpiCombo;
    QLineEdit *xDpi;
    QLabel *label_3;
    QLineEdit *yDpi;
    QPushButton *applyBtn;
    QSpacerItem *spacerItem1;
    QSpacerItem *spacerItem2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *FixDpiDialog)
    {
        if (FixDpiDialog->objectName().isEmpty())
            FixDpiDialog->setObjectName("FixDpiDialog");
        FixDpiDialog->resize(339, 339);
        vboxLayout = new QVBoxLayout(FixDpiDialog);
        vboxLayout->setObjectName("vboxLayout");
        tabWidget = new QTabWidget(FixDpiDialog);
        tabWidget->setObjectName("tabWidget");
        tab = new QWidget();
        tab->setObjectName("tab");
        vboxLayout1 = new QVBoxLayout(tab);
        vboxLayout1->setObjectName("vboxLayout1");
        undefinedDpiView = new QTreeView(tab);
        undefinedDpiView->setObjectName("undefinedDpiView");

        vboxLayout1->addWidget(undefinedDpiView);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        vboxLayout2 = new QVBoxLayout(tab_2);
        vboxLayout2->setObjectName("vboxLayout2");
        allPagesView = new QTreeView(tab_2);
        allPagesView->setObjectName("allPagesView");

        vboxLayout2->addWidget(allPagesView);

        tabWidget->addTab(tab_2, QString());

        vboxLayout->addWidget(tabWidget);

        groupBox = new QGroupBox(FixDpiDialog);
        groupBox->setObjectName("groupBox");
        hboxLayout = new QHBoxLayout(groupBox);
        hboxLayout->setObjectName("hboxLayout");
        spacerItem = new QSpacerItem(0, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        dpiCombo = new QComboBox(groupBox);
        dpiCombo->addItem(QString());
        dpiCombo->setObjectName("dpiCombo");
        dpiCombo->setEnabled(false);

        hboxLayout->addWidget(dpiCombo);

        xDpi = new QLineEdit(groupBox);
        xDpi->setObjectName("xDpi");
        xDpi->setEnabled(false);
        xDpi->setMaximumSize(QSize(45, 16777215));

        hboxLayout->addWidget(xDpi);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName("label_3");

        hboxLayout->addWidget(label_3);

        yDpi = new QLineEdit(groupBox);
        yDpi->setObjectName("yDpi");
        yDpi->setEnabled(false);
        yDpi->setMaximumSize(QSize(45, 16777215));

        hboxLayout->addWidget(yDpi);

        applyBtn = new QPushButton(groupBox);
        applyBtn->setObjectName("applyBtn");
        applyBtn->setEnabled(false);

        hboxLayout->addWidget(applyBtn);

        spacerItem1 = new QSpacerItem(0, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem1);


        vboxLayout->addWidget(groupBox);

        spacerItem2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem2);

        buttonBox = new QDialogButtonBox(FixDpiDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(FixDpiDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, FixDpiDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, FixDpiDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(FixDpiDialog);
    } // setupUi

    void retranslateUi(QDialog *FixDpiDialog)
    {
        FixDpiDialog->setWindowTitle(QCoreApplication::translate("FixDpiDialog", "Fix DPI", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("FixDpiDialog", "Tab 1", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("FixDpiDialog", "Tab 2", nullptr));
        groupBox->setTitle(QCoreApplication::translate("FixDpiDialog", "DPI", nullptr));
        dpiCombo->setItemText(0, QCoreApplication::translate("FixDpiDialog", "Custom", nullptr));

        label_3->setText(QCoreApplication::translate("FixDpiDialog", "x", nullptr));
        applyBtn->setText(QCoreApplication::translate("FixDpiDialog", "Apply", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FixDpiDialog: public Ui_FixDpiDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FIXDPIDIALOG_H
