/********************************************************************************
** Form generated from reading UI file 'OptionsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGET_H
#define UI_OPTIONSWIDGET_H

#include <CollapsibleGroupBox.h>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace output {

class Ui_OptionsWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    CollapsibleGroupBox *dpiPanel;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_5;
    QLabel *dpiLabel;
    QSpacerItem *horizontalSpacer_6;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_7;
    QPushButton *changeDpiButton;
    QSpacerItem *horizontalSpacer_8;
    CollapsibleGroupBox *modePanel;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QComboBox *colorModeSelector;
    QSpacerItem *horizontalSpacer_2;
    CollapsibleGroupBox *commonOptions;
    QHBoxLayout *horizontalLayout_6;
    QSpacerItem *horizontalSpacer_13;
    QVBoxLayout *verticalLayout_12;
    QCheckBox *fillOffcutCB;
    QCheckBox *fillMarginsCB;
    QCheckBox *equalizeIlluminationCB;
    QCheckBox *equalizeIlluminationColorCB;
    QCheckBox *savitzkyGolaySmoothingCB;
    QCheckBox *morphologicalSmoothingCB;
    QSpacerItem *horizontalSpacer_14;
    CollapsibleGroupBox *fillingOptions;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_23;
    QSpacerItem *horizontalSpacer_43;
    QLabel *fillingColorLabel;
    QComboBox *fillingColorBox;
    QSpacerItem *horizontalSpacer_44;
    CollapsibleGroupBox *thresholdOptions;
    QVBoxLayout *verticalLayout_11;
    QHBoxLayout *horizontalLayout_22;
    QSpacerItem *horizontalSpacer_42;
    QLabel *thresholMethodLabel;
    QComboBox *thresholdMethodBox;
    QSpacerItem *horizontalSpacer_41;
    QStackedWidget *binarizationOptions;
    QWidget *page;
    QWidget *page_2;
    CollapsibleGroupBox *colorOperationsOptions;
    QHBoxLayout *horizontalLayout_15;
    QSpacerItem *horizontalSpacer_30;
    QVBoxLayout *verticalLayout_14;
    QGridLayout *filterColorOptions;
    QLabel *wienerLabel;
    QDoubleSpinBox *wienerCoef;
    QSpinBox *wienerWindowSize;
    QCheckBox *colorSegmentationCB;
    QWidget *segmenterOptionsWidget;
    QVBoxLayout *verticalLayout_19;
    QHBoxLayout *horizontalLayout_27;
    QSpacerItem *horizontalSpacer_56;
    QLabel *rLabel;
    QSpinBox *redAdjustmentSB;
    QLabel *gLabel;
    QSpinBox *greenAdjustmentSB;
    QLabel *bLabel;
    QSpinBox *blueAdjustmentSB;
    QSpacerItem *horizontalSpacer_57;
    QHBoxLayout *horizontalLayout_28;
    QSpacerItem *horizontalSpacer_58;
    QLabel *reduceNoiseLabel;
    QSpinBox *reduceNoiseSB;
    QSpacerItem *horizontalSpacer_59;
    QCheckBox *posterizeCB;
    QWidget *posterizeOptionsWidget;
    QVBoxLayout *verticalLayout_15;
    QHBoxLayout *horizontalLayout_17;
    QSpacerItem *horizontalSpacer_32;
    QLabel *posterizeLevelLabel;
    QSpinBox *posterizeLevelSB;
    QSpacerItem *horizontalSpacer_33;
    QHBoxLayout *horizontalLayout_19;
    QSpacerItem *horizontalSpacer_36;
    QCheckBox *posterizeNormalizationCB;
    QSpacerItem *horizontalSpacer_37;
    QHBoxLayout *horizontalLayout_18;
    QSpacerItem *horizontalSpacer_34;
    QCheckBox *posterizeForceBwCB;
    QSpacerItem *horizontalSpacer_35;
    QSpacerItem *horizontalSpacer_31;
    CollapsibleGroupBox *pictureShapeOptions;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_13;
    QSpacerItem *horizontalSpacer_23;
    QComboBox *pictureShapeSelector;
    QSpacerItem *horizontalSpacer_24;
    QWidget *pictureShapeSensitivityOptions;
    QHBoxLayout *horizontalLayout_14;
    QSpacerItem *horizontalSpacer_27;
    QLabel *pictureShapeSensitivityLabel;
    QSpinBox *pictureShapeSensitivitySB;
    QSpacerItem *horizontalSpacer_28;
    QHBoxLayout *horizontalLayout_20;
    QSpacerItem *horizontalSpacer_38;
    QCheckBox *higherSearchSensitivityCB;
    QSpacerItem *horizontalSpacer_39;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_9;
    QPushButton *applyColorsButton;
    QSpacerItem *horizontalSpacer_10;
    CollapsibleGroupBox *splittingOptions;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *horizontalLayout_12;
    QSpacerItem *horizontalSpacer_21;
    QVBoxLayout *verticalLayout;
    QCheckBox *splittingCB;
    QRadioButton *bwForegroundRB;
    QHBoxLayout *horizontalLayout_16;
    QSpacerItem *horizontalSpacer_29;
    QCheckBox *originalBackgroundCB;
    QRadioButton *colorForegroundRB;
    QSpacerItem *horizontalSpacer_22;
    QHBoxLayout *horizontalLayout_7;
    QSpacerItem *horizontalSpacer_25;
    QPushButton *applySplittingButton;
    QSpacerItem *horizontalSpacer_26;
    CollapsibleGroupBox *despecklePanel;
    QVBoxLayout *verticalLayout_5;
    QVBoxLayout *verticalLayout_18;
    QHBoxLayout *horizontalLayout_25;
    QSpacerItem *horizontalSpacer_48;
    QVBoxLayout *verticalLayout_20;
    QCheckBox *despeckleCB;
    QSpacerItem *horizontalSpacer_49;
    QSlider *despeckleSlider;
    QHBoxLayout *horizontalLayout_9;
    QSpacerItem *horizontalSpacer_15;
    QPushButton *applyDespeckleButton;
    QSpacerItem *horizontalSpacer_16;
    CollapsibleGroupBox *processingOptionsPanel;
    QVBoxLayout *verticalLayout_17;
    QHBoxLayout *horizontalLayout_24;
    QSpacerItem *horizontalSpacer_47;
    QVBoxLayout *verticalLayout_16;
    QCheckBox *blackOnWhiteCB;
    QSpacerItem *horizontalSpacer_46;
    QHBoxLayout *horizontalLayout_21;
    QSpacerItem *horizontalSpacer_40;
    QPushButton *applyProcessingOptionsButton;
    QSpacerItem *horizontalSpacer_45;
    CollapsibleGroupBox *depthPerceptionPanel;
    QVBoxLayout *verticalLayout_8;
    QSlider *depthPerceptionSlider;
    QHBoxLayout *horizontalLayout_10;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *applyDepthPerceptionButton;
    QSpacerItem *horizontalSpacer_4;
    CollapsibleGroupBox *dewarpingPanel;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout_11;
    QSpacerItem *horizontalSpacer_19;
    QLabel *dewarpingStatusLabel;
    QSpacerItem *horizontalSpacer_20;
    QHBoxLayout *horizontalLayout_8;
    QSpacerItem *horizontalSpacer_17;
    QPushButton *changeDewarpingButton;
    QSpacerItem *horizontalSpacer_18;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *output__OptionsWidget)
    {
        if (output__OptionsWidget->objectName().isEmpty())
            output__OptionsWidget->setObjectName("output__OptionsWidget");
        output__OptionsWidget->resize(228, 1336);
        verticalLayout_3 = new QVBoxLayout(output__OptionsWidget);
        verticalLayout_3->setObjectName("verticalLayout_3");
        dpiPanel = new CollapsibleGroupBox(output__OptionsWidget);
        dpiPanel->setObjectName("dpiPanel");
        dpiPanel->setStyleSheet(QString::fromUtf8(""));
        dpiPanel->setAlignment(Qt::AlignCenter);
        dpiPanel->setFlat(false);
        verticalLayout_2 = new QVBoxLayout(dpiPanel);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_5 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_5);

        dpiLabel = new QLabel(dpiPanel);
        dpiLabel->setObjectName("dpiLabel");
        dpiLabel->setFrameShape(QFrame::NoFrame);
        dpiLabel->setFrameShadow(QFrame::Plain);

        horizontalLayout_3->addWidget(dpiLabel);

        horizontalSpacer_6 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_6);


        verticalLayout_2->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_7 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_7);

        changeDpiButton = new QPushButton(dpiPanel);
        changeDpiButton->setObjectName("changeDpiButton");

        horizontalLayout_4->addWidget(changeDpiButton);

        horizontalSpacer_8 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_8);


        verticalLayout_2->addLayout(horizontalLayout_4);


        verticalLayout_3->addWidget(dpiPanel);

        modePanel = new CollapsibleGroupBox(output__OptionsWidget);
        modePanel->setObjectName("modePanel");
        modePanel->setAlignment(Qt::AlignCenter);
        modePanel->setFlat(false);
        modePanel->setCheckable(false);
        verticalLayout_6 = new QVBoxLayout(modePanel);
        verticalLayout_6->setObjectName("verticalLayout_6");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        colorModeSelector = new QComboBox(modePanel);
        colorModeSelector->setObjectName("colorModeSelector");

        horizontalLayout_2->addWidget(colorModeSelector);

        horizontalSpacer_2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_6->addLayout(horizontalLayout_2);

        commonOptions = new CollapsibleGroupBox(modePanel);
        commonOptions->setObjectName("commonOptions");
        commonOptions->setStyleSheet(QString::fromUtf8(""));
        commonOptions->setAlignment(Qt::AlignCenter);
        horizontalLayout_6 = new QHBoxLayout(commonOptions);
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalLayout_6->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_13 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_13);

        verticalLayout_12 = new QVBoxLayout();
        verticalLayout_12->setSpacing(2);
        verticalLayout_12->setObjectName("verticalLayout_12");
        verticalLayout_12->setContentsMargins(5, 5, 5, 5);
        fillOffcutCB = new QCheckBox(commonOptions);
        fillOffcutCB->setObjectName("fillOffcutCB");

        verticalLayout_12->addWidget(fillOffcutCB);

        fillMarginsCB = new QCheckBox(commonOptions);
        fillMarginsCB->setObjectName("fillMarginsCB");

        verticalLayout_12->addWidget(fillMarginsCB);

        equalizeIlluminationCB = new QCheckBox(commonOptions);
        equalizeIlluminationCB->setObjectName("equalizeIlluminationCB");

        verticalLayout_12->addWidget(equalizeIlluminationCB);

        equalizeIlluminationColorCB = new QCheckBox(commonOptions);
        equalizeIlluminationColorCB->setObjectName("equalizeIlluminationColorCB");

        verticalLayout_12->addWidget(equalizeIlluminationColorCB);

        savitzkyGolaySmoothingCB = new QCheckBox(commonOptions);
        savitzkyGolaySmoothingCB->setObjectName("savitzkyGolaySmoothingCB");

        verticalLayout_12->addWidget(savitzkyGolaySmoothingCB);

        morphologicalSmoothingCB = new QCheckBox(commonOptions);
        morphologicalSmoothingCB->setObjectName("morphologicalSmoothingCB");

        verticalLayout_12->addWidget(morphologicalSmoothingCB);


        horizontalLayout_6->addLayout(verticalLayout_12);

        horizontalSpacer_14 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_14);


        verticalLayout_6->addWidget(commonOptions);

        fillingOptions = new CollapsibleGroupBox(modePanel);
        fillingOptions->setObjectName("fillingOptions");
        fillingOptions->setAlignment(Qt::AlignCenter);
        fillingOptions->setFlat(false);
        verticalLayout_10 = new QVBoxLayout(fillingOptions);
        verticalLayout_10->setObjectName("verticalLayout_10");
        horizontalLayout_23 = new QHBoxLayout();
        horizontalLayout_23->setObjectName("horizontalLayout_23");
        horizontalSpacer_43 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_23->addItem(horizontalSpacer_43);

        fillingColorLabel = new QLabel(fillingOptions);
        fillingColorLabel->setObjectName("fillingColorLabel");

        horizontalLayout_23->addWidget(fillingColorLabel);

        fillingColorBox = new QComboBox(fillingOptions);
        fillingColorBox->setObjectName("fillingColorBox");

        horizontalLayout_23->addWidget(fillingColorBox);

        horizontalSpacer_44 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_23->addItem(horizontalSpacer_44);


        verticalLayout_10->addLayout(horizontalLayout_23);


        verticalLayout_6->addWidget(fillingOptions);

        thresholdOptions = new CollapsibleGroupBox(modePanel);
        thresholdOptions->setObjectName("thresholdOptions");
        thresholdOptions->setAlignment(Qt::AlignCenter);
        thresholdOptions->setFlat(false);
        verticalLayout_11 = new QVBoxLayout(thresholdOptions);
        verticalLayout_11->setObjectName("verticalLayout_11");
        horizontalLayout_22 = new QHBoxLayout();
        horizontalLayout_22->setObjectName("horizontalLayout_22");
        horizontalSpacer_42 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_22->addItem(horizontalSpacer_42);

        thresholMethodLabel = new QLabel(thresholdOptions);
        thresholMethodLabel->setObjectName("thresholMethodLabel");

        horizontalLayout_22->addWidget(thresholMethodLabel);

        thresholdMethodBox = new QComboBox(thresholdOptions);
        thresholdMethodBox->setObjectName("thresholdMethodBox");

        horizontalLayout_22->addWidget(thresholdMethodBox);

        horizontalSpacer_41 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_22->addItem(horizontalSpacer_41);


        verticalLayout_11->addLayout(horizontalLayout_22);

        binarizationOptions = new QStackedWidget(thresholdOptions);
        binarizationOptions->setObjectName("binarizationOptions");
        page = new QWidget();
        page->setObjectName("page");
        binarizationOptions->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        binarizationOptions->addWidget(page_2);

        verticalLayout_11->addWidget(binarizationOptions);


        verticalLayout_6->addWidget(thresholdOptions);

        colorOperationsOptions = new CollapsibleGroupBox(modePanel);
        colorOperationsOptions->setObjectName("colorOperationsOptions");
        colorOperationsOptions->setStyleSheet(QString::fromUtf8(""));
        colorOperationsOptions->setAlignment(Qt::AlignCenter);
        horizontalLayout_15 = new QHBoxLayout(colorOperationsOptions);
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        horizontalLayout_15->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_30 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_30);

        verticalLayout_14 = new QVBoxLayout();
        verticalLayout_14->setSpacing(2);
        verticalLayout_14->setObjectName("verticalLayout_14");
        verticalLayout_14->setContentsMargins(5, 5, 5, 5);
        filterColorOptions = new QGridLayout();
        filterColorOptions->setObjectName("filterColorOptions");
        wienerLabel = new QLabel(colorOperationsOptions);
        wienerLabel->setObjectName("wienerLabel");

        filterColorOptions->addWidget(wienerLabel, 0, 0, 1, 1);

        wienerCoef = new QDoubleSpinBox(colorOperationsOptions);
        wienerCoef->setObjectName("wienerCoef");
        wienerCoef->setMinimum(0.000000000000000);
        wienerCoef->setMaximum(1.000000000000000);
        wienerCoef->setSingleStep(0.010000000000000);

        filterColorOptions->addWidget(wienerCoef, 0, 1, 1, 1);

        wienerWindowSize = new QSpinBox(colorOperationsOptions);
        wienerWindowSize->setObjectName("wienerWindowSize");
        wienerWindowSize->setMinimum(3);
        wienerWindowSize->setMaximum(9999);

        filterColorOptions->addWidget(wienerWindowSize, 0, 2, 1, 1);


        verticalLayout_14->addLayout(filterColorOptions);

        colorSegmentationCB = new QCheckBox(colorOperationsOptions);
        colorSegmentationCB->setObjectName("colorSegmentationCB");

        verticalLayout_14->addWidget(colorSegmentationCB);

        segmenterOptionsWidget = new QWidget(colorOperationsOptions);
        segmenterOptionsWidget->setObjectName("segmenterOptionsWidget");
        verticalLayout_19 = new QVBoxLayout(segmenterOptionsWidget);
        verticalLayout_19->setObjectName("verticalLayout_19");
        verticalLayout_19->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_27 = new QHBoxLayout();
        horizontalLayout_27->setSpacing(0);
        horizontalLayout_27->setObjectName("horizontalLayout_27");
        horizontalSpacer_56 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_27->addItem(horizontalSpacer_56);

        rLabel = new QLabel(segmenterOptionsWidget);
        rLabel->setObjectName("rLabel");
        rLabel->setMargin(1);
        rLabel->setIndent(0);

        horizontalLayout_27->addWidget(rLabel);

        redAdjustmentSB = new QSpinBox(segmenterOptionsWidget);
        redAdjustmentSB->setObjectName("redAdjustmentSB");
        redAdjustmentSB->setFrame(false);
        redAdjustmentSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        redAdjustmentSB->setMinimum(-99);
        redAdjustmentSB->setMaximum(99);
        redAdjustmentSB->setValue(0);

        horizontalLayout_27->addWidget(redAdjustmentSB);

        gLabel = new QLabel(segmenterOptionsWidget);
        gLabel->setObjectName("gLabel");
        gLabel->setMargin(1);
        gLabel->setIndent(3);

        horizontalLayout_27->addWidget(gLabel);

        greenAdjustmentSB = new QSpinBox(segmenterOptionsWidget);
        greenAdjustmentSB->setObjectName("greenAdjustmentSB");
        greenAdjustmentSB->setFrame(false);
        greenAdjustmentSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        greenAdjustmentSB->setMinimum(-99);
        greenAdjustmentSB->setMaximum(99);
        greenAdjustmentSB->setValue(0);

        horizontalLayout_27->addWidget(greenAdjustmentSB);

        bLabel = new QLabel(segmenterOptionsWidget);
        bLabel->setObjectName("bLabel");
        bLabel->setMargin(1);
        bLabel->setIndent(3);

        horizontalLayout_27->addWidget(bLabel);

        blueAdjustmentSB = new QSpinBox(segmenterOptionsWidget);
        blueAdjustmentSB->setObjectName("blueAdjustmentSB");
        blueAdjustmentSB->setFrame(false);
        blueAdjustmentSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        blueAdjustmentSB->setMinimum(-99);
        blueAdjustmentSB->setMaximum(99);
        blueAdjustmentSB->setValue(0);

        horizontalLayout_27->addWidget(blueAdjustmentSB);

        horizontalSpacer_57 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_27->addItem(horizontalSpacer_57);


        verticalLayout_19->addLayout(horizontalLayout_27);

        horizontalLayout_28 = new QHBoxLayout();
        horizontalLayout_28->setObjectName("horizontalLayout_28");
        horizontalSpacer_58 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_28->addItem(horizontalSpacer_58);

        reduceNoiseLabel = new QLabel(segmenterOptionsWidget);
        reduceNoiseLabel->setObjectName("reduceNoiseLabel");

        horizontalLayout_28->addWidget(reduceNoiseLabel);

        reduceNoiseSB = new QSpinBox(segmenterOptionsWidget);
        reduceNoiseSB->setObjectName("reduceNoiseSB");
        reduceNoiseSB->setFrame(true);
        reduceNoiseSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        reduceNoiseSB->setMinimum(0);
        reduceNoiseSB->setMaximum(999);
        reduceNoiseSB->setValue(0);

        horizontalLayout_28->addWidget(reduceNoiseSB);

        horizontalSpacer_59 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_28->addItem(horizontalSpacer_59);


        verticalLayout_19->addLayout(horizontalLayout_28);


        verticalLayout_14->addWidget(segmenterOptionsWidget);

        posterizeCB = new QCheckBox(colorOperationsOptions);
        posterizeCB->setObjectName("posterizeCB");

        verticalLayout_14->addWidget(posterizeCB);

        posterizeOptionsWidget = new QWidget(colorOperationsOptions);
        posterizeOptionsWidget->setObjectName("posterizeOptionsWidget");
        verticalLayout_15 = new QVBoxLayout(posterizeOptionsWidget);
        verticalLayout_15->setObjectName("verticalLayout_15");
        verticalLayout_15->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        horizontalLayout_17->setContentsMargins(-1, 0, -1, -1);
        horizontalSpacer_32 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_32);

        posterizeLevelLabel = new QLabel(posterizeOptionsWidget);
        posterizeLevelLabel->setObjectName("posterizeLevelLabel");

        horizontalLayout_17->addWidget(posterizeLevelLabel);

        posterizeLevelSB = new QSpinBox(posterizeOptionsWidget);
        posterizeLevelSB->setObjectName("posterizeLevelSB");
        posterizeLevelSB->setFrame(true);
        posterizeLevelSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        posterizeLevelSB->setMinimum(2);
        posterizeLevelSB->setMaximum(255);

        horizontalLayout_17->addWidget(posterizeLevelSB);

        horizontalSpacer_33 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_33);


        verticalLayout_15->addLayout(horizontalLayout_17);

        horizontalLayout_19 = new QHBoxLayout();
        horizontalLayout_19->setObjectName("horizontalLayout_19");
        horizontalLayout_19->setContentsMargins(-1, 0, -1, -1);
        horizontalSpacer_36 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_19->addItem(horizontalSpacer_36);

        posterizeNormalizationCB = new QCheckBox(posterizeOptionsWidget);
        posterizeNormalizationCB->setObjectName("posterizeNormalizationCB");

        horizontalLayout_19->addWidget(posterizeNormalizationCB);

        horizontalSpacer_37 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_19->addItem(horizontalSpacer_37);


        verticalLayout_15->addLayout(horizontalLayout_19);

        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setObjectName("horizontalLayout_18");
        horizontalLayout_18->setContentsMargins(-1, 0, -1, -1);
        horizontalSpacer_34 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_18->addItem(horizontalSpacer_34);

        posterizeForceBwCB = new QCheckBox(posterizeOptionsWidget);
        posterizeForceBwCB->setObjectName("posterizeForceBwCB");

        horizontalLayout_18->addWidget(posterizeForceBwCB);

        horizontalSpacer_35 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_18->addItem(horizontalSpacer_35);


        verticalLayout_15->addLayout(horizontalLayout_18);


        verticalLayout_14->addWidget(posterizeOptionsWidget);


        horizontalLayout_15->addLayout(verticalLayout_14);

        horizontalSpacer_31 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_31);


        verticalLayout_6->addWidget(colorOperationsOptions);

        pictureShapeOptions = new CollapsibleGroupBox(modePanel);
        pictureShapeOptions->setObjectName("pictureShapeOptions");
        pictureShapeOptions->setAlignment(Qt::AlignCenter);
        verticalLayout_4 = new QVBoxLayout(pictureShapeOptions);
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        horizontalSpacer_23 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_23);

        pictureShapeSelector = new QComboBox(pictureShapeOptions);
        pictureShapeSelector->setObjectName("pictureShapeSelector");

        horizontalLayout_13->addWidget(pictureShapeSelector);

        horizontalSpacer_24 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_24);


        verticalLayout_4->addLayout(horizontalLayout_13);

        pictureShapeSensitivityOptions = new QWidget(pictureShapeOptions);
        pictureShapeSensitivityOptions->setObjectName("pictureShapeSensitivityOptions");
        horizontalLayout_14 = new QHBoxLayout(pictureShapeSensitivityOptions);
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        horizontalLayout_14->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_27 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_27);

        pictureShapeSensitivityLabel = new QLabel(pictureShapeSensitivityOptions);
        pictureShapeSensitivityLabel->setObjectName("pictureShapeSensitivityLabel");

        horizontalLayout_14->addWidget(pictureShapeSensitivityLabel);

        pictureShapeSensitivitySB = new QSpinBox(pictureShapeSensitivityOptions);
        pictureShapeSensitivitySB->setObjectName("pictureShapeSensitivitySB");
        pictureShapeSensitivitySB->setFrame(true);
        pictureShapeSensitivitySB->setMinimum(0);
        pictureShapeSensitivitySB->setMaximum(100);

        horizontalLayout_14->addWidget(pictureShapeSensitivitySB);

        horizontalSpacer_28 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_28);


        verticalLayout_4->addWidget(pictureShapeSensitivityOptions);

        horizontalLayout_20 = new QHBoxLayout();
        horizontalLayout_20->setObjectName("horizontalLayout_20");
        horizontalSpacer_38 = new QSpacerItem(1, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_38);

        higherSearchSensitivityCB = new QCheckBox(pictureShapeOptions);
        higherSearchSensitivityCB->setObjectName("higherSearchSensitivityCB");

        horizontalLayout_20->addWidget(higherSearchSensitivityCB);

        horizontalSpacer_39 = new QSpacerItem(1, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_39);


        verticalLayout_4->addLayout(horizontalLayout_20);


        verticalLayout_6->addWidget(pictureShapeOptions);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_9 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_9);

        applyColorsButton = new QPushButton(modePanel);
        applyColorsButton->setObjectName("applyColorsButton");

        horizontalLayout_5->addWidget(applyColorsButton);

        horizontalSpacer_10 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_10);


        verticalLayout_6->addLayout(horizontalLayout_5);


        verticalLayout_3->addWidget(modePanel);

        splittingOptions = new CollapsibleGroupBox(output__OptionsWidget);
        splittingOptions->setObjectName("splittingOptions");
        splittingOptions->setStyleSheet(QString::fromUtf8(""));
        splittingOptions->setAlignment(Qt::AlignCenter);
        verticalLayout_9 = new QVBoxLayout(splittingOptions);
        verticalLayout_9->setObjectName("verticalLayout_9");
        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        horizontalSpacer_21 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_21);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        splittingCB = new QCheckBox(splittingOptions);
        splittingCB->setObjectName("splittingCB");

        verticalLayout->addWidget(splittingCB);

        bwForegroundRB = new QRadioButton(splittingOptions);
        bwForegroundRB->setObjectName("bwForegroundRB");
        bwForegroundRB->setChecked(true);

        verticalLayout->addWidget(bwForegroundRB);

        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        horizontalSpacer_29 = new QSpacerItem(15, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_29);

        originalBackgroundCB = new QCheckBox(splittingOptions);
        originalBackgroundCB->setObjectName("originalBackgroundCB");

        horizontalLayout_16->addWidget(originalBackgroundCB);


        verticalLayout->addLayout(horizontalLayout_16);

        colorForegroundRB = new QRadioButton(splittingOptions);
        colorForegroundRB->setObjectName("colorForegroundRB");

        verticalLayout->addWidget(colorForegroundRB);


        horizontalLayout_12->addLayout(verticalLayout);

        horizontalSpacer_22 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_22);


        verticalLayout_9->addLayout(horizontalLayout_12);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalSpacer_25 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_25);

        applySplittingButton = new QPushButton(splittingOptions);
        applySplittingButton->setObjectName("applySplittingButton");

        horizontalLayout_7->addWidget(applySplittingButton);

        horizontalSpacer_26 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_26);


        verticalLayout_9->addLayout(horizontalLayout_7);


        verticalLayout_3->addWidget(splittingOptions);

        despecklePanel = new CollapsibleGroupBox(output__OptionsWidget);
        despecklePanel->setObjectName("despecklePanel");
        despecklePanel->setAlignment(Qt::AlignCenter);
        despecklePanel->setFlat(false);
        verticalLayout_5 = new QVBoxLayout(despecklePanel);
        verticalLayout_5->setSpacing(6);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(15, 9, 15, -1);
        verticalLayout_18 = new QVBoxLayout();
        verticalLayout_18->setSpacing(4);
        verticalLayout_18->setObjectName("verticalLayout_18");
        verticalLayout_18->setContentsMargins(0, 0, 0, -1);
        horizontalLayout_25 = new QHBoxLayout();
        horizontalLayout_25->setObjectName("horizontalLayout_25");
        horizontalSpacer_48 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_25->addItem(horizontalSpacer_48);

        verticalLayout_20 = new QVBoxLayout();
        verticalLayout_20->setObjectName("verticalLayout_20");
        despeckleCB = new QCheckBox(despecklePanel);
        despeckleCB->setObjectName("despeckleCB");

        verticalLayout_20->addWidget(despeckleCB);


        horizontalLayout_25->addLayout(verticalLayout_20);

        horizontalSpacer_49 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_25->addItem(horizontalSpacer_49);


        verticalLayout_18->addLayout(horizontalLayout_25);

        despeckleSlider = new QSlider(despecklePanel);
        despeckleSlider->setObjectName("despeckleSlider");
        despeckleSlider->setMinimum(5);
        despeckleSlider->setMaximum(35);
        despeckleSlider->setSingleStep(2);
        despeckleSlider->setPageStep(5);
        despeckleSlider->setValue(10);
        despeckleSlider->setOrientation(Qt::Horizontal);

        verticalLayout_18->addWidget(despeckleSlider);


        verticalLayout_5->addLayout(verticalLayout_18);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        horizontalSpacer_15 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_15);

        applyDespeckleButton = new QPushButton(despecklePanel);
        applyDespeckleButton->setObjectName("applyDespeckleButton");

        horizontalLayout_9->addWidget(applyDespeckleButton);

        horizontalSpacer_16 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_16);


        verticalLayout_5->addLayout(horizontalLayout_9);


        verticalLayout_3->addWidget(despecklePanel);

        processingOptionsPanel = new CollapsibleGroupBox(output__OptionsWidget);
        processingOptionsPanel->setObjectName("processingOptionsPanel");
        processingOptionsPanel->setAlignment(Qt::AlignCenter);
        verticalLayout_17 = new QVBoxLayout(processingOptionsPanel);
        verticalLayout_17->setObjectName("verticalLayout_17");
        horizontalLayout_24 = new QHBoxLayout();
        horizontalLayout_24->setObjectName("horizontalLayout_24");
        horizontalSpacer_47 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_24->addItem(horizontalSpacer_47);

        verticalLayout_16 = new QVBoxLayout();
        verticalLayout_16->setObjectName("verticalLayout_16");
        blackOnWhiteCB = new QCheckBox(processingOptionsPanel);
        blackOnWhiteCB->setObjectName("blackOnWhiteCB");

        verticalLayout_16->addWidget(blackOnWhiteCB);


        horizontalLayout_24->addLayout(verticalLayout_16);

        horizontalSpacer_46 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_24->addItem(horizontalSpacer_46);


        verticalLayout_17->addLayout(horizontalLayout_24);

        horizontalLayout_21 = new QHBoxLayout();
        horizontalLayout_21->setObjectName("horizontalLayout_21");
        horizontalSpacer_40 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_21->addItem(horizontalSpacer_40);

        applyProcessingOptionsButton = new QPushButton(processingOptionsPanel);
        applyProcessingOptionsButton->setObjectName("applyProcessingOptionsButton");

        horizontalLayout_21->addWidget(applyProcessingOptionsButton);

        horizontalSpacer_45 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_21->addItem(horizontalSpacer_45);


        verticalLayout_17->addLayout(horizontalLayout_21);


        verticalLayout_3->addWidget(processingOptionsPanel);

        depthPerceptionPanel = new CollapsibleGroupBox(output__OptionsWidget);
        depthPerceptionPanel->setObjectName("depthPerceptionPanel");
        depthPerceptionPanel->setAlignment(Qt::AlignCenter);
        depthPerceptionPanel->setFlat(false);
        verticalLayout_8 = new QVBoxLayout(depthPerceptionPanel);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(15, -1, 15, -1);
        depthPerceptionSlider = new QSlider(depthPerceptionPanel);
        depthPerceptionSlider->setObjectName("depthPerceptionSlider");
        depthPerceptionSlider->setMinimum(10);
        depthPerceptionSlider->setMaximum(30);
        depthPerceptionSlider->setPageStep(5);
        depthPerceptionSlider->setValue(10);
        depthPerceptionSlider->setOrientation(Qt::Horizontal);

        verticalLayout_8->addWidget(depthPerceptionSlider);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        horizontalSpacer_3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_3);

        applyDepthPerceptionButton = new QPushButton(depthPerceptionPanel);
        applyDepthPerceptionButton->setObjectName("applyDepthPerceptionButton");

        horizontalLayout_10->addWidget(applyDepthPerceptionButton);

        horizontalSpacer_4 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_4);


        verticalLayout_8->addLayout(horizontalLayout_10);


        verticalLayout_3->addWidget(depthPerceptionPanel);

        dewarpingPanel = new CollapsibleGroupBox(output__OptionsWidget);
        dewarpingPanel->setObjectName("dewarpingPanel");
        dewarpingPanel->setAlignment(Qt::AlignCenter);
        dewarpingPanel->setFlat(false);
        verticalLayout_7 = new QVBoxLayout(dewarpingPanel);
        verticalLayout_7->setObjectName("verticalLayout_7");
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        horizontalSpacer_19 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_19);

        dewarpingStatusLabel = new QLabel(dewarpingPanel);
        dewarpingStatusLabel->setObjectName("dewarpingStatusLabel");

        horizontalLayout_11->addWidget(dewarpingStatusLabel);

        horizontalSpacer_20 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_20);


        verticalLayout_7->addLayout(horizontalLayout_11);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalSpacer_17 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_17);

        changeDewarpingButton = new QPushButton(dewarpingPanel);
        changeDewarpingButton->setObjectName("changeDewarpingButton");

        horizontalLayout_8->addWidget(changeDewarpingButton);

        horizontalSpacer_18 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_18);


        verticalLayout_7->addLayout(horizontalLayout_8);


        verticalLayout_3->addWidget(dewarpingPanel);

        verticalSpacer = new QSpacerItem(0, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        retranslateUi(output__OptionsWidget);

        binarizationOptions->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(output__OptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *output__OptionsWidget)
    {
        output__OptionsWidget->setWindowTitle(QCoreApplication::translate("output::OptionsWidget", "Form", nullptr));
#if QT_CONFIG(tooltip)
        dpiPanel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        dpiPanel->setTitle(QCoreApplication::translate("output::OptionsWidget", "Output Resolution (DPI)", nullptr));
        dpiLabel->setText(QCoreApplication::translate("output::OptionsWidget", "0", nullptr));
        changeDpiButton->setText(QCoreApplication::translate("output::OptionsWidget", "Change ...", nullptr));
        modePanel->setTitle(QCoreApplication::translate("output::OptionsWidget", "Mode", nullptr));
        commonOptions->setTitle(QCoreApplication::translate("output::OptionsWidget", "Options", nullptr));
        fillOffcutCB->setText(QCoreApplication::translate("output::OptionsWidget", "Fill offcut", nullptr));
        fillMarginsCB->setText(QCoreApplication::translate("output::OptionsWidget", "Fill margins", nullptr));
#if QT_CONFIG(tooltip)
        equalizeIlluminationCB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Normalize illumination before binarization.", nullptr));
#endif // QT_CONFIG(tooltip)
        equalizeIlluminationCB->setText(QCoreApplication::translate("output::OptionsWidget", "Equalize illumination (B&&W)", nullptr));
#if QT_CONFIG(tooltip)
        equalizeIlluminationColorCB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Normalize illumination in color mode / in picture zones in mixed mode.", nullptr));
#endif // QT_CONFIG(tooltip)
        equalizeIlluminationColorCB->setText(QCoreApplication::translate("output::OptionsWidget", "Equalize illumination (Color)", nullptr));
        savitzkyGolaySmoothingCB->setText(QCoreApplication::translate("output::OptionsWidget", "Savitzky-Golay smoothing", nullptr));
        morphologicalSmoothingCB->setText(QCoreApplication::translate("output::OptionsWidget", "Morphological smoothing", nullptr));
        fillingOptions->setTitle(QCoreApplication::translate("output::OptionsWidget", "Filling", nullptr));
        fillingColorLabel->setText(QCoreApplication::translate("output::OptionsWidget", "Color: ", nullptr));
        thresholdOptions->setTitle(QCoreApplication::translate("output::OptionsWidget", "Threshold", nullptr));
        thresholMethodLabel->setText(QCoreApplication::translate("output::OptionsWidget", "Method:", nullptr));
        colorOperationsOptions->setTitle(QCoreApplication::translate("output::OptionsWidget", "Color operations", nullptr));
        wienerLabel->setText(QCoreApplication::translate("output::OptionsWidget", "Wiener denoiser", nullptr));
#if QT_CONFIG(tooltip)
        wienerCoef->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Value is 0.0 .. 1.0..", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        wienerWindowSize->setToolTip(QCoreApplication::translate("output::OptionsWidget", "The dimensions of a pixel neighborhood to consider.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        colorSegmentationCB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Split the image into color segments and colorize b&w mask.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        colorSegmentationCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        colorSegmentationCB->setText(QCoreApplication::translate("output::OptionsWidget", "Color segmentation", nullptr));
        rLabel->setText(QCoreApplication::translate("output::OptionsWidget", "R", nullptr));
#if QT_CONFIG(tooltip)
        redAdjustmentSB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Red component adjustment. A negative value means the segmenter will be more sensitive to red and vice versa for a positive one.", nullptr));
#endif // QT_CONFIG(tooltip)
        gLabel->setText(QCoreApplication::translate("output::OptionsWidget", "G", nullptr));
#if QT_CONFIG(tooltip)
        greenAdjustmentSB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Green component adjustment. A negative value means the segmenter will be more sensitive to green and vice versa for a positive one.", nullptr));
#endif // QT_CONFIG(tooltip)
        bLabel->setText(QCoreApplication::translate("output::OptionsWidget", "B", nullptr));
#if QT_CONFIG(tooltip)
        blueAdjustmentSB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Blue component adjustment. A negative value means the segmenter will be more sensitive to blue and vice versa for a positive one.", nullptr));
#endif // QT_CONFIG(tooltip)
        reduceNoiseLabel->setText(QCoreApplication::translate("output::OptionsWidget", "Reduce noise:", nullptr));
#if QT_CONFIG(tooltip)
        reduceNoiseSB->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        posterizeCB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Reduce the number of colors of the output image by grouping similar colors.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        posterizeCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        posterizeCB->setText(QCoreApplication::translate("output::OptionsWidget", "Posterize", nullptr));
        posterizeLevelLabel->setText(QCoreApplication::translate("output::OptionsWidget", "Level:", nullptr));
#if QT_CONFIG(tooltip)
        posterizeLevelSB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Lower value means lower count of colors in the output image, values between 2 and 6 inclusive guarantee an indexed image.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        posterizeNormalizationCB->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        posterizeNormalizationCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        posterizeNormalizationCB->setText(QCoreApplication::translate("output::OptionsWidget", "Normalize", nullptr));
#if QT_CONFIG(tooltip)
        posterizeForceBwCB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Make dark and light gray gradients black and white respectively.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        posterizeForceBwCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        posterizeForceBwCB->setText(QCoreApplication::translate("output::OptionsWidget", "Force b&&w", nullptr));
        pictureShapeOptions->setTitle(QCoreApplication::translate("output::OptionsWidget", "Picture Shape", nullptr));
        pictureShapeSensitivityLabel->setText(QCoreApplication::translate("output::OptionsWidget", "Sensitivity (%):", nullptr));
        higherSearchSensitivityCB->setText(QCoreApplication::translate("output::OptionsWidget", "Higher search sensitivity", nullptr));
        applyColorsButton->setText(QCoreApplication::translate("output::OptionsWidget", "Apply To ...", nullptr));
        splittingOptions->setTitle(QCoreApplication::translate("output::OptionsWidget", "Splitting", nullptr));
        splittingCB->setText(QCoreApplication::translate("output::OptionsWidget", "Split output", nullptr));
        bwForegroundRB->setText(QCoreApplication::translate("output::OptionsWidget", "B&&W foreground", nullptr));
#if QT_CONFIG(tooltip)
        originalBackgroundCB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "Save the original background of the foreground layer.", nullptr));
#endif // QT_CONFIG(tooltip)
        originalBackgroundCB->setText(QCoreApplication::translate("output::OptionsWidget", "Original background", nullptr));
        colorForegroundRB->setText(QCoreApplication::translate("output::OptionsWidget", "Color foreground", nullptr));
        applySplittingButton->setText(QCoreApplication::translate("output::OptionsWidget", "Apply To ...", nullptr));
        despecklePanel->setTitle(QCoreApplication::translate("output::OptionsWidget", "Despeckling", nullptr));
        despeckleCB->setText(QCoreApplication::translate("output::OptionsWidget", "Despeckle", nullptr));
        applyDespeckleButton->setText(QCoreApplication::translate("output::OptionsWidget", "Apply To ...", nullptr));
        processingOptionsPanel->setTitle(QCoreApplication::translate("output::OptionsWidget", "Processing", nullptr));
#if QT_CONFIG(tooltip)
        blackOnWhiteCB->setToolTip(QCoreApplication::translate("output::OptionsWidget", "This option should be enabled when the page has dark content on light background and disabled if vice versa in order to correct processing algorithms.", nullptr));
#endif // QT_CONFIG(tooltip)
        blackOnWhiteCB->setText(QCoreApplication::translate("output::OptionsWidget", "Black on white mode", nullptr));
        applyProcessingOptionsButton->setText(QCoreApplication::translate("output::OptionsWidget", "Apply To ...", nullptr));
        depthPerceptionPanel->setTitle(QCoreApplication::translate("output::OptionsWidget", "Depth perception", nullptr));
        applyDepthPerceptionButton->setText(QCoreApplication::translate("output::OptionsWidget", "Apply To ...", nullptr));
        dewarpingPanel->setTitle(QCoreApplication::translate("output::OptionsWidget", "Dewarping", nullptr));
        dewarpingStatusLabel->setText(QString());
        changeDewarpingButton->setText(QCoreApplication::translate("output::OptionsWidget", "Change ...", nullptr));
    } // retranslateUi

};

} // namespace output

namespace output {
namespace Ui {
    class OptionsWidget: public Ui_OptionsWidget {};
} // namespace Ui
} // namespace output

#endif // UI_OPTIONSWIDGET_H
