// Copyright (C) 2019  Joseph Artsimovich <joseph.artsimovich@gmail.com>, 4lex4 <4lex49@zoho.com>
// Use of this source code is governed by the GNU GPLv3 license that can be found in the LICENSE file.

#ifndef SCANTAILOR_OUTPUT_IMAGEVIEWTAB_H_
#define SCANTAILOR_OUTPUT_IMAGEVIEWTAB_H_

namespace output {
enum ImageViewTab { TAB_OUTPUT, TAB_PICTURE_ZONES, TAB_FILL_ZONES, TAB_DEWARPING, TAB_DESPECKLING };
}  // namespace output
#endif
