/********************************************************************************
** Form generated from reading UI file 'ApplyDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_APPLYDIALOG_H
#define UI_APPLYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

namespace page_layout {

class Ui_ApplyDialog
{
public:
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_2;
    QRadioButton *thisPageRB;
    QRadioButton *allPagesRB;
    QRadioButton *thisPageAndFollowersRB;
    QRadioButton *thisEveryOtherRB;
    QRadioButton *everyOtherRB;
    QRadioButton *selectedPagesRB;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_5;
    QLabel *selectedPagesHint;
    QRadioButton *everyOtherSelectedRB;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_6;
    QLabel *everyOtherSelectedHint;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *page_layout__ApplyDialog)
    {
        if (page_layout__ApplyDialog->objectName().isEmpty())
            page_layout__ApplyDialog->setObjectName("page_layout__ApplyDialog");
        page_layout__ApplyDialog->setWindowModality(Qt::WindowModal);
        page_layout__ApplyDialog->resize(361, 290);
        verticalLayout_5 = new QVBoxLayout(page_layout__ApplyDialog);
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox_2 = new QGroupBox(page_layout__ApplyDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_2 = new QVBoxLayout(groupBox_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        thisPageRB = new QRadioButton(groupBox_2);
        thisPageRB->setObjectName("thisPageRB");
        thisPageRB->setChecked(true);

        verticalLayout_2->addWidget(thisPageRB);

        allPagesRB = new QRadioButton(groupBox_2);
        allPagesRB->setObjectName("allPagesRB");

        verticalLayout_2->addWidget(allPagesRB);

        thisPageAndFollowersRB = new QRadioButton(groupBox_2);
        thisPageAndFollowersRB->setObjectName("thisPageAndFollowersRB");

        verticalLayout_2->addWidget(thisPageAndFollowersRB);

        thisEveryOtherRB = new QRadioButton(groupBox_2);
        thisEveryOtherRB->setObjectName("thisEveryOtherRB");

        verticalLayout_2->addWidget(thisEveryOtherRB);

        everyOtherRB = new QRadioButton(groupBox_2);
        everyOtherRB->setObjectName("everyOtherRB");

        verticalLayout_2->addWidget(everyOtherRB);

        selectedPagesRB = new QRadioButton(groupBox_2);
        selectedPagesRB->setObjectName("selectedPagesRB");

        verticalLayout_2->addWidget(selectedPagesRB);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(6);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_5 = new QSpacerItem(30, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_5);

        selectedPagesHint = new QLabel(groupBox_2);
        selectedPagesHint->setObjectName("selectedPagesHint");
        QFont font;
        font.setPointSize(7);
        selectedPagesHint->setFont(font);

        horizontalLayout_2->addWidget(selectedPagesHint);


        verticalLayout_2->addLayout(horizontalLayout_2);

        everyOtherSelectedRB = new QRadioButton(groupBox_2);
        everyOtherSelectedRB->setObjectName("everyOtherSelectedRB");

        verticalLayout_2->addWidget(everyOtherSelectedRB);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_6 = new QSpacerItem(30, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_6);

        everyOtherSelectedHint = new QLabel(groupBox_2);
        everyOtherSelectedHint->setObjectName("everyOtherSelectedHint");
        everyOtherSelectedHint->setFont(font);

        horizontalLayout_5->addWidget(everyOtherSelectedHint);


        verticalLayout_2->addLayout(horizontalLayout_5);


        verticalLayout_5->addWidget(groupBox_2);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(page_layout__ApplyDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_5->addWidget(buttonBox);


        retranslateUi(page_layout__ApplyDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, page_layout__ApplyDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(page_layout__ApplyDialog);
    } // setupUi

    void retranslateUi(QDialog *page_layout__ApplyDialog)
    {
        groupBox_2->setTitle(QCoreApplication::translate("page_layout::ApplyDialog", "Apply to", nullptr));
        thisPageRB->setText(QCoreApplication::translate("page_layout::ApplyDialog", "This page only (already applied)", nullptr));
        allPagesRB->setText(QCoreApplication::translate("page_layout::ApplyDialog", "All pages", nullptr));
        thisPageAndFollowersRB->setText(QCoreApplication::translate("page_layout::ApplyDialog", "This page and the following ones", nullptr));
        thisEveryOtherRB->setText(QCoreApplication::translate("page_layout::ApplyDialog", "This page and the following every other page", nullptr));
        everyOtherRB->setText(QCoreApplication::translate("page_layout::ApplyDialog", "Every other page", nullptr));
        selectedPagesRB->setText(QCoreApplication::translate("page_layout::ApplyDialog", "Selected pages", nullptr));
        selectedPagesHint->setText(QCoreApplication::translate("page_layout::ApplyDialog", "Use Ctrl+Click / Shift+Click to select multiple pages.", nullptr));
        everyOtherSelectedRB->setText(QCoreApplication::translate("page_layout::ApplyDialog", "Every other selected page", nullptr));
        everyOtherSelectedHint->setText(QCoreApplication::translate("page_layout::ApplyDialog", "The current page will be included.", nullptr));
        (void)page_layout__ApplyDialog;
    } // retranslateUi

};

} // namespace page_layout

namespace page_layout {
namespace Ui {
    class ApplyDialog: public Ui_ApplyDialog {};
} // namespace Ui
} // namespace page_layout

#endif // UI_APPLYDIALOG_H
