/********************************************************************************
** Form generated from reading UI file 'OutOfMemoryDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OUTOFMEMORYDIALOG_H
#define UI_OUTOFMEMORYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_OutOfMemoryDialog
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *topLevelStack;
    QWidget *mainPage;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QSpacerItem *verticalSpacer_4;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_5;
    QLabel *label_4;
    QFrame *line;
    QLabel *label_6;
    QFrame *line_2;
    QLabel *label_8;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_4;
    QScrollArea *scrollArea_2;
    QWidget *scrollAreaWidgetContents_2;
    QVBoxLayout *verticalLayout_6;
    QLabel *label_10;
    QFrame *line_3;
    QLabel *only_32bit_1;
    QFrame *only_32bit_2;
    QLabel *label_12;
    QFrame *line_5;
    QLabel *label_14;
    QSpacerItem *verticalSpacer;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_8;
    QScrollArea *scrollArea_3;
    QWidget *scrollAreaWidgetContents_3;
    QVBoxLayout *verticalLayout_7;
    QLabel *label_5;
    QSpacerItem *verticalSpacer_3;
    QHBoxLayout *horizontalLayout_9;
    QPushButton *saveProjectBtn;
    QSpacerItem *horizontalSpacer;
    QPushButton *saveProjectAsBtn;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *dontSaveBtn;
    QWidget *saveSuccessPage;
    QVBoxLayout *verticalLayout_9;
    QLabel *label_2;
    QSpacerItem *verticalSpacer_5;
    QLabel *label_3;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *OutOfMemoryDialog)
    {
        if (OutOfMemoryDialog->objectName().isEmpty())
            OutOfMemoryDialog->setObjectName("OutOfMemoryDialog");
        OutOfMemoryDialog->resize(470, 342);
        verticalLayout = new QVBoxLayout(OutOfMemoryDialog);
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        topLevelStack = new QStackedWidget(OutOfMemoryDialog);
        topLevelStack->setObjectName("topLevelStack");
        mainPage = new QWidget();
        mainPage->setObjectName("mainPage");
        verticalLayout_2 = new QVBoxLayout(mainPage);
        verticalLayout_2->setObjectName("verticalLayout_2");
        label = new QLabel(mainPage);
        label->setObjectName("label");
        label->setStyleSheet(QString::fromUtf8("font-size: 17px;\n"
"font-weight: bold;"));
        label->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(label);

        verticalSpacer_4 = new QSpacerItem(1, 7, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        verticalLayout_2->addItem(verticalSpacer_4);

        tabWidget = new QTabWidget(mainPage);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setDocumentMode(false);
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_3 = new QVBoxLayout(tab);
        verticalLayout_3->setSpacing(0);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(tab);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 446, 227));
        verticalLayout_5 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_5->setObjectName("verticalLayout_5");
        label_4 = new QLabel(scrollAreaWidgetContents);
        label_4->setObjectName("label_4");
        label_4->setWordWrap(true);
        label_4->setOpenExternalLinks(true);

        verticalLayout_5->addWidget(label_4);

        line = new QFrame(scrollAreaWidgetContents);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_5->addWidget(line);

        label_6 = new QLabel(scrollAreaWidgetContents);
        label_6->setObjectName("label_6");
        label_6->setWordWrap(true);
        label_6->setOpenExternalLinks(true);

        verticalLayout_5->addWidget(label_6);

        line_2 = new QFrame(scrollAreaWidgetContents);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_5->addWidget(line_2);

        label_8 = new QLabel(scrollAreaWidgetContents);
        label_8->setObjectName("label_8");
        label_8->setWordWrap(true);
        label_8->setOpenExternalLinks(true);

        verticalLayout_5->addWidget(label_8);

        verticalSpacer_2 = new QSpacerItem(1, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_2);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_4 = new QVBoxLayout(tab_2);
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        scrollArea_2 = new QScrollArea(tab_2);
        scrollArea_2->setObjectName("scrollArea_2");
        scrollArea_2->setFrameShape(QFrame::NoFrame);
        scrollArea_2->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName("scrollAreaWidgetContents_2");
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 98, 715));
        verticalLayout_6 = new QVBoxLayout(scrollAreaWidgetContents_2);
        verticalLayout_6->setObjectName("verticalLayout_6");
        label_10 = new QLabel(scrollAreaWidgetContents_2);
        label_10->setObjectName("label_10");
        label_10->setWordWrap(true);
        label_10->setOpenExternalLinks(true);

        verticalLayout_6->addWidget(label_10);

        line_3 = new QFrame(scrollAreaWidgetContents_2);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_6->addWidget(line_3);

        only_32bit_1 = new QLabel(scrollAreaWidgetContents_2);
        only_32bit_1->setObjectName("only_32bit_1");
        only_32bit_1->setWordWrap(true);
        only_32bit_1->setOpenExternalLinks(true);

        verticalLayout_6->addWidget(only_32bit_1);

        only_32bit_2 = new QFrame(scrollAreaWidgetContents_2);
        only_32bit_2->setObjectName("only_32bit_2");
        only_32bit_2->setFrameShape(QFrame::Shape::HLine);
        only_32bit_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_6->addWidget(only_32bit_2);

        label_12 = new QLabel(scrollAreaWidgetContents_2);
        label_12->setObjectName("label_12");
        label_12->setWordWrap(true);
        label_12->setOpenExternalLinks(true);

        verticalLayout_6->addWidget(label_12);

        line_5 = new QFrame(scrollAreaWidgetContents_2);
        line_5->setObjectName("line_5");
        line_5->setFrameShape(QFrame::Shape::HLine);
        line_5->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_6->addWidget(line_5);

        label_14 = new QLabel(scrollAreaWidgetContents_2);
        label_14->setObjectName("label_14");
        label_14->setWordWrap(true);
        label_14->setOpenExternalLinks(true);

        verticalLayout_6->addWidget(label_14);

        verticalSpacer = new QSpacerItem(20, 9, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_6->addItem(verticalSpacer);

        scrollArea_2->setWidget(scrollAreaWidgetContents_2);

        verticalLayout_4->addWidget(scrollArea_2);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_8 = new QVBoxLayout(tab_3);
        verticalLayout_8->setObjectName("verticalLayout_8");
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        scrollArea_3 = new QScrollArea(tab_3);
        scrollArea_3->setObjectName("scrollArea_3");
        scrollArea_3->setFrameShape(QFrame::NoFrame);
        scrollArea_3->setFrameShadow(QFrame::Plain);
        scrollArea_3->setWidgetResizable(true);
        scrollAreaWidgetContents_3 = new QWidget();
        scrollAreaWidgetContents_3->setObjectName("scrollAreaWidgetContents_3");
        scrollAreaWidgetContents_3->setGeometry(QRect(0, 0, 98, 414));
        verticalLayout_7 = new QVBoxLayout(scrollAreaWidgetContents_3);
        verticalLayout_7->setObjectName("verticalLayout_7");
        label_5 = new QLabel(scrollAreaWidgetContents_3);
        label_5->setObjectName("label_5");
        label_5->setWordWrap(true);
        label_5->setOpenExternalLinks(true);

        verticalLayout_7->addWidget(label_5);

        verticalSpacer_3 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_7->addItem(verticalSpacer_3);

        scrollArea_3->setWidget(scrollAreaWidgetContents_3);

        verticalLayout_8->addWidget(scrollArea_3);

        tabWidget->addTab(tab_3, QString());

        verticalLayout_2->addWidget(tabWidget);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        saveProjectBtn = new QPushButton(mainPage);
        saveProjectBtn->setObjectName("saveProjectBtn");

        horizontalLayout_9->addWidget(saveProjectBtn);

        horizontalSpacer = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer);

        saveProjectAsBtn = new QPushButton(mainPage);
        saveProjectAsBtn->setObjectName("saveProjectAsBtn");

        horizontalLayout_9->addWidget(saveProjectAsBtn);

        horizontalSpacer_2 = new QSpacerItem(40, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_2);

        dontSaveBtn = new QPushButton(mainPage);
        dontSaveBtn->setObjectName("dontSaveBtn");

        horizontalLayout_9->addWidget(dontSaveBtn);


        verticalLayout_2->addLayout(horizontalLayout_9);

        topLevelStack->addWidget(mainPage);
        saveSuccessPage = new QWidget();
        saveSuccessPage->setObjectName("saveSuccessPage");
        verticalLayout_9 = new QVBoxLayout(saveSuccessPage);
        verticalLayout_9->setObjectName("verticalLayout_9");
        label_2 = new QLabel(saveSuccessPage);
        label_2->setObjectName("label_2");
        label_2->setStyleSheet(QString::fromUtf8("font-size: 17px;\n"
"font-weight: bold;"));
        label_2->setAlignment(Qt::AlignCenter);

        verticalLayout_9->addWidget(label_2);

        verticalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Preferred);

        verticalLayout_9->addItem(verticalSpacer_5);

        label_3 = new QLabel(saveSuccessPage);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_3->setWordWrap(true);

        verticalLayout_9->addWidget(label_3);

        buttonBox = new QDialogButtonBox(saveSuccessPage);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        verticalLayout_9->addWidget(buttonBox);

        topLevelStack->addWidget(saveSuccessPage);

        verticalLayout->addWidget(topLevelStack);


        retranslateUi(OutOfMemoryDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, OutOfMemoryDialog, qOverload<>(&QDialog::reject));

        topLevelStack->setCurrentIndex(0);
        tabWidget->setCurrentIndex(0);
        saveProjectBtn->setDefault(true);


        QMetaObject::connectSlotsByName(OutOfMemoryDialog);
    } // setupUi

    void retranslateUi(QDialog *OutOfMemoryDialog)
    {
        OutOfMemoryDialog->setWindowTitle(QCoreApplication::translate("OutOfMemoryDialog", "Out of memory", nullptr));
        label->setText(QCoreApplication::translate("OutOfMemoryDialog", "Out of Memory Situation in Scan Tailor", nullptr));
        label_4->setText(QCoreApplication::translate("OutOfMemoryDialog", "Did you have to fix the DPI of your source images? Are you sure the values you entered were correct?", nullptr));
        label_6->setText(QCoreApplication::translate("OutOfMemoryDialog", "Sometimes your source images may have wrong DPI embedded into them. Scan Tailor tries to detect those, but it's not always easy to tell. You may need to check \"Fix DPI even if they look normal\" when creating a project and look into \"All pages\" tab in the \"Fix DPI\" dialog, which is also accessible from the Tools menu.", nullptr));
        label_8->setText(QCoreApplication::translate("OutOfMemoryDialog", "Is your output DPI set too high? Usually you don't need it higher than 600.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("OutOfMemoryDialog", "Possible reasons", nullptr));
        label_10->setText(QCoreApplication::translate("OutOfMemoryDialog", "Fix your DPIs. Learn how to <a href=\"http://vimeo.com/12524529\">estimate unknown DPIs</a>.", nullptr));
        only_32bit_1->setText(QCoreApplication::translate("OutOfMemoryDialog", "If your hardware and operating system are 64-bit capable, consider switching to a 64-bit version of Scan Tailor.", nullptr));
        label_12->setText(QCoreApplication::translate("OutOfMemoryDialog", "When working with grayscale images, make sure they are really grayscale. If they are actually color images that just happen to look grayscale, convert them to grayscale using some kind of batch image converter. This will both save memory and increase performance.", nullptr));
        label_14->setText(QCoreApplication::translate("OutOfMemoryDialog", "As a last resort, you can save some memory by making sure thumbnails are pre-created rather than created on demand. This can be done by slowly scrolling the thumbnail list all the way from top to bottom before starting any real work.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("OutOfMemoryDialog", "What can help", nullptr));
        label_5->setText(QCoreApplication::translate("OutOfMemoryDialog", "Surprisingly, upgrading your RAM won't help here. The lack of RAM is compensated by the swap mechanism, which makes things slow, but keeps programs running. An out of memory situation means we ran out of memory address space, which has nothing to do with the amount of RAM you have. The only way to increase the memory address space is to go 64-bit hardware, 64-bit operating system and 64-bit Scan Tailor.", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("OutOfMemoryDialog", "What won't help", nullptr));
        saveProjectBtn->setText(QCoreApplication::translate("OutOfMemoryDialog", "Save Project", nullptr));
        saveProjectAsBtn->setText(QCoreApplication::translate("OutOfMemoryDialog", "Save Project As ...", nullptr));
        dontSaveBtn->setText(QCoreApplication::translate("OutOfMemoryDialog", "Don't Save", nullptr));
        label_2->setText(QCoreApplication::translate("OutOfMemoryDialog", "Project Saved Successfully", nullptr));
        label_3->setText(QCoreApplication::translate("OutOfMemoryDialog", "Please note that while Scan Tailor tries to catch out-of-memory situations and give you the opportunity to save your project, it's not always possible. This time it succeeded, but the next time it might just crash.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class OutOfMemoryDialog: public Ui_OutOfMemoryDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OUTOFMEMORYDIALOG_H
