/********************************************************************************
** Form generated from reading UI file 'MainWindow.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MAINWINDOW_H
#define UI_MAINWINDOW_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDockWidget>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "NonOwningWidget.h"
#include "StageListView.h"

QT_BEGIN_NAMESPACE

class Ui_MainWindow
{
public:
    QAction *actionDebug;
    QAction *actionSaveProject;
    QAction *actionSaveProjectAs;
    QAction *actionNextPage;
    QAction *actionPrevPage;
    QAction *actionNewProject;
    QAction *actionOpenProject;
    QAction *actionPrevPageQ;
    QAction *actionNextPageW;
    QAction *actionCloseProject;
    QAction *actionQuit;
    QAction *actionSettings;
    QAction *actionFirstPage;
    QAction *actionLastPage;
    QAction *actionAbout;
    QAction *actionFixDpi;
    QAction *actionRelinking;
    QAction *actionSwitchFilter1;
    QAction *actionSwitchFilter2;
    QAction *actionSwitchFilter3;
    QAction *actionSwitchFilter4;
    QAction *actionSwitchFilter5;
    QAction *actionSwitchFilter6;
    QAction *actionPixels;
    QAction *actionMilimeters;
    QAction *actionInches;
    QAction *actionCentimetres;
    QAction *actionDefaults;
    QAction *actionPrevSelectedPageQ;
    QAction *actionNextSelectedPageW;
    QAction *actionNextSelectedPage;
    QAction *actionPrevSelectedPage;
    QAction *actionGotoPage;
    QAction *actionMagnifyThumbnails;
    QAction *actionDiminishThumbnails;
    QAction *actionReloadPage;
    QWidget *centralwidget;
    QHBoxLayout *horizontalLayout;
    QFrame *imageViewFrame;
    QMenuBar *menubar;
    QMenu *menuDebug;
    QMenu *menuUnits;
    QMenu *menuFile;
    QMenu *menuHelp;
    QStatusBar *statusBar;
    QDockWidget *thumbnailsDockWidget;
    QWidget *dockWidgetContents_2;
    QHBoxLayout *horizontalLayout_4;
    QVBoxLayout *verticalLayout_2;
    QToolButton *focusButton;
    QFrame *line;
    QToolButton *prevPageBtn;
    QToolButton *nextPageBtn;
    QToolButton *filterSelectedBtn;
    QFrame *line_2;
    QToolButton *magnifyThumbnailsBtn;
    QToolButton *diminishThumbnailsBtn;
    QFrame *line_3;
    QToolButton *gotoPageBtn;
    QToolButton *selectionModeBtn;
    QToolButton *thumbColumnViewBtn;
    QToolButton *sortingOrderBtn;
    QToolButton *deviationHighlightingBtn;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout;
    QGraphicsView *thumbView;
    QComboBox *sortOptions;
    QDockWidget *filterDockWidget;
    QWidget *dockWidgetContents_5;
    QVBoxLayout *verticalLayout_3;
    StageListView *filterList;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_4;
    NonOwningWidget *filterOptions;

    void setupUi(QMainWindow *MainWindow)
    {
        if (MainWindow->objectName().isEmpty())
            MainWindow->setObjectName("MainWindow");
        MainWindow->resize(783, 464);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MainWindow->sizePolicy().hasHeightForWidth());
        MainWindow->setSizePolicy(sizePolicy);
        MainWindow->setStyleSheet(QString::fromUtf8(""));
        actionDebug = new QAction(MainWindow);
        actionDebug->setObjectName("actionDebug");
        actionDebug->setCheckable(true);
        actionDebug->setChecked(false);
#if QT_CONFIG(shortcut)
        actionDebug->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionDebug->setAutoRepeat(false);
        actionSaveProject = new QAction(MainWindow);
        actionSaveProject->setObjectName("actionSaveProject");
#if QT_CONFIG(shortcut)
        actionSaveProject->setShortcut(QString::fromUtf8("Ctrl+S"));
#endif // QT_CONFIG(shortcut)
        actionSaveProject->setAutoRepeat(false);
        actionSaveProjectAs = new QAction(MainWindow);
        actionSaveProjectAs->setObjectName("actionSaveProjectAs");
#if QT_CONFIG(shortcut)
        actionSaveProjectAs->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionSaveProjectAs->setAutoRepeat(false);
        actionNextPage = new QAction(MainWindow);
        actionNextPage->setObjectName("actionNextPage");
#if QT_CONFIG(shortcut)
        actionNextPage->setShortcut(QString::fromUtf8("PgDown"));
#endif // QT_CONFIG(shortcut)
        actionNextPage->setAutoRepeat(false);
        actionPrevPage = new QAction(MainWindow);
        actionPrevPage->setObjectName("actionPrevPage");
#if QT_CONFIG(shortcut)
        actionPrevPage->setShortcut(QString::fromUtf8("PgUp"));
#endif // QT_CONFIG(shortcut)
        actionPrevPage->setAutoRepeat(false);
        actionNewProject = new QAction(MainWindow);
        actionNewProject->setObjectName("actionNewProject");
#if QT_CONFIG(shortcut)
        actionNewProject->setShortcut(QString::fromUtf8("Ctrl+N"));
#endif // QT_CONFIG(shortcut)
        actionNewProject->setAutoRepeat(false);
        actionOpenProject = new QAction(MainWindow);
        actionOpenProject->setObjectName("actionOpenProject");
#if QT_CONFIG(shortcut)
        actionOpenProject->setShortcut(QString::fromUtf8("Ctrl+O"));
#endif // QT_CONFIG(shortcut)
        actionOpenProject->setAutoRepeat(false);
        actionPrevPageQ = new QAction(MainWindow);
        actionPrevPageQ->setObjectName("actionPrevPageQ");
#if QT_CONFIG(shortcut)
        actionPrevPageQ->setShortcut(QString::fromUtf8("Q"));
#endif // QT_CONFIG(shortcut)
        actionPrevPageQ->setAutoRepeat(false);
        actionNextPageW = new QAction(MainWindow);
        actionNextPageW->setObjectName("actionNextPageW");
#if QT_CONFIG(shortcut)
        actionNextPageW->setShortcut(QString::fromUtf8("W"));
#endif // QT_CONFIG(shortcut)
        actionNextPageW->setAutoRepeat(false);
        actionCloseProject = new QAction(MainWindow);
        actionCloseProject->setObjectName("actionCloseProject");
#if QT_CONFIG(shortcut)
        actionCloseProject->setShortcut(QString::fromUtf8("Ctrl+W"));
#endif // QT_CONFIG(shortcut)
        actionCloseProject->setAutoRepeat(false);
        actionQuit = new QAction(MainWindow);
        actionQuit->setObjectName("actionQuit");
#if QT_CONFIG(shortcut)
        actionQuit->setShortcut(QString::fromUtf8("Ctrl+Q"));
#endif // QT_CONFIG(shortcut)
        actionQuit->setAutoRepeat(false);
        actionSettings = new QAction(MainWindow);
        actionSettings->setObjectName("actionSettings");
#if QT_CONFIG(shortcut)
        actionSettings->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionSettings->setAutoRepeat(false);
        actionFirstPage = new QAction(MainWindow);
        actionFirstPage->setObjectName("actionFirstPage");
#if QT_CONFIG(shortcut)
        actionFirstPage->setShortcut(QString::fromUtf8("Home"));
#endif // QT_CONFIG(shortcut)
        actionFirstPage->setAutoRepeat(false);
        actionLastPage = new QAction(MainWindow);
        actionLastPage->setObjectName("actionLastPage");
#if QT_CONFIG(shortcut)
        actionLastPage->setShortcut(QString::fromUtf8("End"));
#endif // QT_CONFIG(shortcut)
        actionLastPage->setAutoRepeat(false);
        actionAbout = new QAction(MainWindow);
        actionAbout->setObjectName("actionAbout");
#if QT_CONFIG(shortcut)
        actionAbout->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionAbout->setAutoRepeat(false);
        actionFixDpi = new QAction(MainWindow);
        actionFixDpi->setObjectName("actionFixDpi");
#if QT_CONFIG(shortcut)
        actionFixDpi->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionFixDpi->setAutoRepeat(false);
        actionRelinking = new QAction(MainWindow);
        actionRelinking->setObjectName("actionRelinking");
#if QT_CONFIG(shortcut)
        actionRelinking->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionRelinking->setAutoRepeat(false);
        actionSwitchFilter1 = new QAction(MainWindow);
        actionSwitchFilter1->setObjectName("actionSwitchFilter1");
#if QT_CONFIG(shortcut)
        actionSwitchFilter1->setShortcut(QString::fromUtf8("O"));
#endif // QT_CONFIG(shortcut)
        actionSwitchFilter1->setAutoRepeat(false);
        actionSwitchFilter2 = new QAction(MainWindow);
        actionSwitchFilter2->setObjectName("actionSwitchFilter2");
#if QT_CONFIG(shortcut)
        actionSwitchFilter2->setShortcut(QString::fromUtf8("E"));
#endif // QT_CONFIG(shortcut)
        actionSwitchFilter2->setAutoRepeat(false);
        actionSwitchFilter3 = new QAction(MainWindow);
        actionSwitchFilter3->setObjectName("actionSwitchFilter3");
#if QT_CONFIG(shortcut)
        actionSwitchFilter3->setShortcut(QString::fromUtf8("N"));
#endif // QT_CONFIG(shortcut)
        actionSwitchFilter3->setAutoRepeat(false);
        actionSwitchFilter4 = new QAction(MainWindow);
        actionSwitchFilter4->setObjectName("actionSwitchFilter4");
#if QT_CONFIG(shortcut)
        actionSwitchFilter4->setShortcut(QString::fromUtf8("V"));
#endif // QT_CONFIG(shortcut)
        actionSwitchFilter4->setAutoRepeat(false);
        actionSwitchFilter5 = new QAction(MainWindow);
        actionSwitchFilter5->setObjectName("actionSwitchFilter5");
#if QT_CONFIG(shortcut)
        actionSwitchFilter5->setShortcut(QString::fromUtf8("K"));
#endif // QT_CONFIG(shortcut)
        actionSwitchFilter5->setAutoRepeat(false);
        actionSwitchFilter6 = new QAction(MainWindow);
        actionSwitchFilter6->setObjectName("actionSwitchFilter6");
#if QT_CONFIG(shortcut)
        actionSwitchFilter6->setShortcut(QString::fromUtf8("P"));
#endif // QT_CONFIG(shortcut)
        actionSwitchFilter6->setAutoRepeat(false);
        actionPixels = new QAction(MainWindow);
        actionPixels->setObjectName("actionPixels");
        actionPixels->setCheckable(true);
#if QT_CONFIG(shortcut)
        actionPixels->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionPixels->setAutoRepeat(false);
        actionMilimeters = new QAction(MainWindow);
        actionMilimeters->setObjectName("actionMilimeters");
        actionMilimeters->setCheckable(true);
        actionMilimeters->setChecked(false);
#if QT_CONFIG(shortcut)
        actionMilimeters->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionMilimeters->setAutoRepeat(false);
        actionInches = new QAction(MainWindow);
        actionInches->setObjectName("actionInches");
        actionInches->setCheckable(true);
#if QT_CONFIG(shortcut)
        actionInches->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionInches->setAutoRepeat(false);
        actionCentimetres = new QAction(MainWindow);
        actionCentimetres->setObjectName("actionCentimetres");
        actionCentimetres->setCheckable(true);
#if QT_CONFIG(shortcut)
        actionCentimetres->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionCentimetres->setAutoRepeat(false);
        actionDefaults = new QAction(MainWindow);
        actionDefaults->setObjectName("actionDefaults");
#if QT_CONFIG(shortcut)
        actionDefaults->setShortcut(QString::fromUtf8(""));
#endif // QT_CONFIG(shortcut)
        actionDefaults->setAutoRepeat(false);
        actionPrevSelectedPageQ = new QAction(MainWindow);
        actionPrevSelectedPageQ->setObjectName("actionPrevSelectedPageQ");
#if QT_CONFIG(shortcut)
        actionPrevSelectedPageQ->setShortcut(QString::fromUtf8("Shift+Q"));
#endif // QT_CONFIG(shortcut)
        actionPrevSelectedPageQ->setAutoRepeat(false);
        actionNextSelectedPageW = new QAction(MainWindow);
        actionNextSelectedPageW->setObjectName("actionNextSelectedPageW");
#if QT_CONFIG(shortcut)
        actionNextSelectedPageW->setShortcut(QString::fromUtf8("Shift+W"));
#endif // QT_CONFIG(shortcut)
        actionNextSelectedPageW->setAutoRepeat(false);
        actionNextSelectedPage = new QAction(MainWindow);
        actionNextSelectedPage->setObjectName("actionNextSelectedPage");
#if QT_CONFIG(shortcut)
        actionNextSelectedPage->setShortcut(QString::fromUtf8("Shift+PgDown"));
#endif // QT_CONFIG(shortcut)
        actionNextSelectedPage->setAutoRepeat(false);
        actionPrevSelectedPage = new QAction(MainWindow);
        actionPrevSelectedPage->setObjectName("actionPrevSelectedPage");
#if QT_CONFIG(shortcut)
        actionPrevSelectedPage->setShortcut(QString::fromUtf8("Shift+PgUp"));
#endif // QT_CONFIG(shortcut)
        actionPrevSelectedPage->setAutoRepeat(false);
        actionGotoPage = new QAction(MainWindow);
        actionGotoPage->setObjectName("actionGotoPage");
#if QT_CONFIG(shortcut)
        actionGotoPage->setShortcut(QString::fromUtf8("Ctrl+G"));
#endif // QT_CONFIG(shortcut)
        actionGotoPage->setAutoRepeat(false);
        actionMagnifyThumbnails = new QAction(MainWindow);
        actionMagnifyThumbnails->setObjectName("actionMagnifyThumbnails");
#if QT_CONFIG(shortcut)
        actionMagnifyThumbnails->setShortcut(QString::fromUtf8("Ctrl++"));
#endif // QT_CONFIG(shortcut)
        actionDiminishThumbnails = new QAction(MainWindow);
        actionDiminishThumbnails->setObjectName("actionDiminishThumbnails");
#if QT_CONFIG(shortcut)
        actionDiminishThumbnails->setShortcut(QString::fromUtf8("Ctrl+-"));
#endif // QT_CONFIG(shortcut)
        actionReloadPage = new QAction(MainWindow);
        actionReloadPage->setObjectName("actionReloadPage");
#if QT_CONFIG(shortcut)
        actionReloadPage->setShortcut(QString::fromUtf8("F5"));
#endif // QT_CONFIG(shortcut)
        actionReloadPage->setAutoRepeat(false);
        centralwidget = new QWidget(MainWindow);
        centralwidget->setObjectName("centralwidget");
        sizePolicy.setHeightForWidth(centralwidget->sizePolicy().hasHeightForWidth());
        centralwidget->setSizePolicy(sizePolicy);
        centralwidget->setCursor(QCursor(Qt::CursorShape::ArrowCursor));
        horizontalLayout = new QHBoxLayout(centralwidget);
        horizontalLayout->setObjectName("horizontalLayout");
        imageViewFrame = new QFrame(centralwidget);
        imageViewFrame->setObjectName("imageViewFrame");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(1);
        sizePolicy1.setHeightForWidth(imageViewFrame->sizePolicy().hasHeightForWidth());
        imageViewFrame->setSizePolicy(sizePolicy1);
        imageViewFrame->setFrameShape(QFrame::StyledPanel);
        imageViewFrame->setFrameShadow(QFrame::Sunken);

        horizontalLayout->addWidget(imageViewFrame);

        MainWindow->setCentralWidget(centralwidget);
        menubar = new QMenuBar(MainWindow);
        menubar->setObjectName("menubar");
        menubar->setGeometry(QRect(0, 0, 783, 21));
        menuDebug = new QMenu(menubar);
        menuDebug->setObjectName("menuDebug");
        menuUnits = new QMenu(menuDebug);
        menuUnits->setObjectName("menuUnits");
        menuFile = new QMenu(menubar);
        menuFile->setObjectName("menuFile");
        menuHelp = new QMenu(menubar);
        menuHelp->setObjectName("menuHelp");
        MainWindow->setMenuBar(menubar);
        statusBar = new QStatusBar(MainWindow);
        statusBar->setObjectName("statusBar");
        MainWindow->setStatusBar(statusBar);
        thumbnailsDockWidget = new QDockWidget(MainWindow);
        thumbnailsDockWidget->setObjectName("thumbnailsDockWidget");
        thumbnailsDockWidget->setFeatures(QDockWidget::DockWidgetFloatable|QDockWidget::DockWidgetMovable);
        thumbnailsDockWidget->setAllowedAreas(Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
        dockWidgetContents_2 = new QWidget();
        dockWidgetContents_2->setObjectName("dockWidgetContents_2");
        horizontalLayout_4 = new QHBoxLayout(dockWidgetContents_2);
        horizontalLayout_4->setSpacing(2);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(4, 2, -1, 9);
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        focusButton = new QToolButton(dockWidgetContents_2);
        focusButton->setObjectName("focusButton");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(focusButton->sizePolicy().hasHeightForWidth());
        focusButton->setSizePolicy(sizePolicy2);
        focusButton->setMinimumSize(QSize(22, 22));
        focusButton->setMaximumSize(QSize(22, 22));
        focusButton->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/eye.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        focusButton->setIcon(icon);
        focusButton->setIconSize(QSize(16, 16));
        focusButton->setCheckable(true);
        focusButton->setChecked(true);

        verticalLayout_2->addWidget(focusButton);

        line = new QFrame(dockWidgetContents_2);
        line->setObjectName("line");
        line->setFrameShadow(QFrame::Raised);
        line->setFrameShape(QFrame::Shape::HLine);

        verticalLayout_2->addWidget(line);

        prevPageBtn = new QToolButton(dockWidgetContents_2);
        prevPageBtn->setObjectName("prevPageBtn");
        sizePolicy2.setHeightForWidth(prevPageBtn->sizePolicy().hasHeightForWidth());
        prevPageBtn->setSizePolicy(sizePolicy2);
        prevPageBtn->setMinimumSize(QSize(22, 22));
        prevPageBtn->setMaximumSize(QSize(22, 22));
        prevPageBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        prevPageBtn->setText(QString::fromUtf8(""));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/triangle-up-arrow.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        prevPageBtn->setIcon(icon1);
        prevPageBtn->setIconSize(QSize(16, 16));

        verticalLayout_2->addWidget(prevPageBtn);

        nextPageBtn = new QToolButton(dockWidgetContents_2);
        nextPageBtn->setObjectName("nextPageBtn");
        sizePolicy2.setHeightForWidth(nextPageBtn->sizePolicy().hasHeightForWidth());
        nextPageBtn->setSizePolicy(sizePolicy2);
        nextPageBtn->setMinimumSize(QSize(22, 22));
        nextPageBtn->setMaximumSize(QSize(22, 22));
        nextPageBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        nextPageBtn->setText(QString::fromUtf8(""));
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/triangle-down-arrow.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        nextPageBtn->setIcon(icon2);
        nextPageBtn->setIconSize(QSize(16, 16));

        verticalLayout_2->addWidget(nextPageBtn);

        filterSelectedBtn = new QToolButton(dockWidgetContents_2);
        filterSelectedBtn->setObjectName("filterSelectedBtn");
        sizePolicy2.setHeightForWidth(filterSelectedBtn->sizePolicy().hasHeightForWidth());
        filterSelectedBtn->setSizePolicy(sizePolicy2);
        filterSelectedBtn->setMinimumSize(QSize(22, 22));
        filterSelectedBtn->setMaximumSize(QSize(22, 22));
        filterSelectedBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        filterSelectedBtn->setText(QString::fromUtf8(""));
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/check-mark.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        filterSelectedBtn->setIcon(icon3);
        filterSelectedBtn->setIconSize(QSize(16, 16));
        filterSelectedBtn->setCheckable(true);

        verticalLayout_2->addWidget(filterSelectedBtn);

        line_2 = new QFrame(dockWidgetContents_2);
        line_2->setObjectName("line_2");
        line_2->setFrameShadow(QFrame::Raised);
        line_2->setFrameShape(QFrame::Shape::HLine);

        verticalLayout_2->addWidget(line_2);

        magnifyThumbnailsBtn = new QToolButton(dockWidgetContents_2);
        magnifyThumbnailsBtn->setObjectName("magnifyThumbnailsBtn");
        sizePolicy2.setHeightForWidth(magnifyThumbnailsBtn->sizePolicy().hasHeightForWidth());
        magnifyThumbnailsBtn->setSizePolicy(sizePolicy2);
        magnifyThumbnailsBtn->setMinimumSize(QSize(22, 22));
        magnifyThumbnailsBtn->setMaximumSize(QSize(22, 22));
        magnifyThumbnailsBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        magnifyThumbnailsBtn->setText(QString::fromUtf8(""));
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/magnifying-glass.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        magnifyThumbnailsBtn->setIcon(icon4);
        magnifyThumbnailsBtn->setIconSize(QSize(16, 16));

        verticalLayout_2->addWidget(magnifyThumbnailsBtn);

        diminishThumbnailsBtn = new QToolButton(dockWidgetContents_2);
        diminishThumbnailsBtn->setObjectName("diminishThumbnailsBtn");
        sizePolicy2.setHeightForWidth(diminishThumbnailsBtn->sizePolicy().hasHeightForWidth());
        diminishThumbnailsBtn->setSizePolicy(sizePolicy2);
        diminishThumbnailsBtn->setMinimumSize(QSize(22, 22));
        diminishThumbnailsBtn->setMaximumSize(QSize(22, 22));
        diminishThumbnailsBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        diminishThumbnailsBtn->setText(QString::fromUtf8(""));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/diminishing-glass.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        diminishThumbnailsBtn->setIcon(icon5);
        diminishThumbnailsBtn->setIconSize(QSize(16, 16));

        verticalLayout_2->addWidget(diminishThumbnailsBtn);

        line_3 = new QFrame(dockWidgetContents_2);
        line_3->setObjectName("line_3");
        line_3->setFrameShadow(QFrame::Raised);
        line_3->setFrameShape(QFrame::Shape::HLine);

        verticalLayout_2->addWidget(line_3);

        gotoPageBtn = new QToolButton(dockWidgetContents_2);
        gotoPageBtn->setObjectName("gotoPageBtn");
        sizePolicy2.setHeightForWidth(gotoPageBtn->sizePolicy().hasHeightForWidth());
        gotoPageBtn->setSizePolicy(sizePolicy2);
        gotoPageBtn->setMinimumSize(QSize(22, 22));
        gotoPageBtn->setMaximumSize(QSize(22, 22));
        gotoPageBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        gotoPageBtn->setText(QString::fromUtf8(""));
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/right-pointing.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        gotoPageBtn->setIcon(icon6);
        gotoPageBtn->setIconSize(QSize(16, 16));

        verticalLayout_2->addWidget(gotoPageBtn);

        selectionModeBtn = new QToolButton(dockWidgetContents_2);
        selectionModeBtn->setObjectName("selectionModeBtn");
        sizePolicy2.setHeightForWidth(selectionModeBtn->sizePolicy().hasHeightForWidth());
        selectionModeBtn->setSizePolicy(sizePolicy2);
        selectionModeBtn->setMinimumSize(QSize(22, 22));
        selectionModeBtn->setMaximumSize(QSize(22, 22));
        selectionModeBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}\n"
"\n"
"QToolButton:checked:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(window-text);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:checked {\n"
"  padding: 0;\n"
"  border: 1px solid palette(window-text);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        selectionModeBtn->setText(QString::fromUtf8(""));
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/checkbox-styled.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        selectionModeBtn->setIcon(icon7);
        selectionModeBtn->setIconSize(QSize(16, 16));
        selectionModeBtn->setCheckable(true);

        verticalLayout_2->addWidget(selectionModeBtn);

        thumbColumnViewBtn = new QToolButton(dockWidgetContents_2);
        thumbColumnViewBtn->setObjectName("thumbColumnViewBtn");
        sizePolicy2.setHeightForWidth(thumbColumnViewBtn->sizePolicy().hasHeightForWidth());
        thumbColumnViewBtn->setSizePolicy(sizePolicy2);
        thumbColumnViewBtn->setMinimumSize(QSize(22, 22));
        thumbColumnViewBtn->setMaximumSize(QSize(22, 22));
        thumbColumnViewBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        thumbColumnViewBtn->setText(QString::fromUtf8(""));
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/single-column-view.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        thumbColumnViewBtn->setIcon(icon8);
        thumbColumnViewBtn->setIconSize(QSize(16, 16));
        thumbColumnViewBtn->setCheckable(true);

        verticalLayout_2->addWidget(thumbColumnViewBtn);

        sortingOrderBtn = new QToolButton(dockWidgetContents_2);
        sortingOrderBtn->setObjectName("sortingOrderBtn");
        sizePolicy2.setHeightForWidth(sortingOrderBtn->sizePolicy().hasHeightForWidth());
        sortingOrderBtn->setSizePolicy(sizePolicy2);
        sortingOrderBtn->setMinimumSize(QSize(22, 22));
        sortingOrderBtn->setMaximumSize(QSize(22, 22));
        sortingOrderBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        sortingOrderBtn->setText(QString::fromUtf8(""));
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/ascending_sorting_order.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        sortingOrderBtn->setIcon(icon9);
        sortingOrderBtn->setIconSize(QSize(16, 16));
        sortingOrderBtn->setCheckable(true);

        verticalLayout_2->addWidget(sortingOrderBtn);

        deviationHighlightingBtn = new QToolButton(dockWidgetContents_2);
        deviationHighlightingBtn->setObjectName("deviationHighlightingBtn");
        sizePolicy2.setHeightForWidth(deviationHighlightingBtn->sizePolicy().hasHeightForWidth());
        deviationHighlightingBtn->setSizePolicy(sizePolicy2);
        deviationHighlightingBtn->setMinimumSize(QSize(22, 22));
        deviationHighlightingBtn->setMaximumSize(QSize(22, 22));
        deviationHighlightingBtn->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        deviationHighlightingBtn->setText(QString::fromUtf8(""));
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/icons/six-spoked-asterisk.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        deviationHighlightingBtn->setIcon(icon10);
        deviationHighlightingBtn->setIconSize(QSize(16, 16));
        deviationHighlightingBtn->setCheckable(true);

        verticalLayout_2->addWidget(deviationHighlightingBtn);

        verticalSpacer = new QSpacerItem(1, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout_4->addLayout(verticalLayout_2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        thumbView = new QGraphicsView(dockWidgetContents_2);
        thumbView->setObjectName("thumbView");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Ignored, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(1);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(thumbView->sizePolicy().hasHeightForWidth());
        thumbView->setSizePolicy(sizePolicy3);
        thumbView->setMinimumSize(QSize(150, 0));
        thumbView->setMaximumSize(QSize(16777215, 16777215));
        thumbView->setAlignment(Qt::AlignHCenter|Qt::AlignTop);

        verticalLayout->addWidget(thumbView);

        sortOptions = new QComboBox(dockWidgetContents_2);
        sortOptions->setObjectName("sortOptions");

        verticalLayout->addWidget(sortOptions);


        horizontalLayout_4->addLayout(verticalLayout);

        thumbnailsDockWidget->setWidget(dockWidgetContents_2);
        MainWindow->addDockWidget(Qt::DockWidgetArea::RightDockWidgetArea, thumbnailsDockWidget);
        filterDockWidget = new QDockWidget(MainWindow);
        filterDockWidget->setObjectName("filterDockWidget");
        sizePolicy.setHeightForWidth(filterDockWidget->sizePolicy().hasHeightForWidth());
        filterDockWidget->setSizePolicy(sizePolicy);
        filterDockWidget->setMinimumSize(QSize(274, 204));
        filterDockWidget->setFeatures(QDockWidget::DockWidgetFloatable|QDockWidget::DockWidgetMovable);
        filterDockWidget->setAllowedAreas(Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea);
        dockWidgetContents_5 = new QWidget();
        dockWidgetContents_5->setObjectName("dockWidgetContents_5");
        sizePolicy.setHeightForWidth(dockWidgetContents_5->sizePolicy().hasHeightForWidth());
        dockWidgetContents_5->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(dockWidgetContents_5);
        verticalLayout_3->setObjectName("verticalLayout_3");
        filterList = new StageListView(dockWidgetContents_5);
        filterList->setObjectName("filterList");
        filterList->setMinimumSize(QSize(0, 0));
        filterList->setMaximumSize(QSize(16777215, 16777215));
        filterList->setAutoScroll(false);
        filterList->setEditTriggers(QAbstractItemView::NoEditTriggers);
        filterList->setTabKeyNavigation(false);
        filterList->setProperty("showDropIndicator", QVariant(false));
        filterList->setAlternatingRowColors(true);
        filterList->setSelectionMode(QAbstractItemView::SingleSelection);
        filterList->setSelectionBehavior(QAbstractItemView::SelectRows);
        filterList->setTextElideMode(Qt::ElideNone);
        filterList->setShowGrid(false);
        filterList->setWordWrap(false);
        filterList->setCornerButtonEnabled(false);

        verticalLayout_3->addWidget(filterList);

        scrollArea = new QScrollArea(dockWidgetContents_5);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 256, 185));
        verticalLayout_4 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_4->setSpacing(0);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        filterOptions = new NonOwningWidget(scrollAreaWidgetContents);
        filterOptions->setObjectName("filterOptions");

        verticalLayout_4->addWidget(filterOptions);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        filterDockWidget->setWidget(dockWidgetContents_5);
        MainWindow->addDockWidget(Qt::DockWidgetArea::LeftDockWidgetArea, filterDockWidget);

        menubar->addAction(menuFile->menuAction());
        menubar->addAction(menuDebug->menuAction());
        menubar->addAction(menuHelp->menuAction());
        menuDebug->addAction(actionFixDpi);
        menuDebug->addAction(actionRelinking);
        menuDebug->addSeparator();
        menuDebug->addAction(actionDebug);
        menuDebug->addSeparator();
        menuDebug->addAction(actionSettings);
        menuDebug->addAction(actionDefaults);
        menuDebug->addSeparator();
        menuDebug->addAction(menuUnits->menuAction());
        menuUnits->addAction(actionPixels);
        menuUnits->addAction(actionMilimeters);
        menuUnits->addAction(actionCentimetres);
        menuUnits->addAction(actionInches);
        menuFile->addAction(actionNewProject);
        menuFile->addAction(actionOpenProject);
        menuFile->addSeparator();
        menuFile->addAction(actionSaveProject);
        menuFile->addAction(actionSaveProjectAs);
        menuFile->addSeparator();
        menuFile->addAction(actionCloseProject);
        menuFile->addSeparator();
        menuFile->addAction(actionQuit);
        menuHelp->addAction(actionAbout);

        retranslateUi(MainWindow);

        QMetaObject::connectSlotsByName(MainWindow);
    } // setupUi

    void retranslateUi(QMainWindow *MainWindow)
    {
        MainWindow->setWindowTitle(QCoreApplication::translate("MainWindow", "MainWindow", nullptr));
        actionDebug->setText(QCoreApplication::translate("MainWindow", "Debug Mode", nullptr));
        actionSaveProject->setText(QCoreApplication::translate("MainWindow", "Save Project", nullptr));
        actionSaveProjectAs->setText(QCoreApplication::translate("MainWindow", "Save Project As ...", nullptr));
        actionNextPage->setText(QCoreApplication::translate("MainWindow", "Next Page", nullptr));
        actionPrevPage->setText(QCoreApplication::translate("MainWindow", "Previous Page", nullptr));
        actionNewProject->setText(QCoreApplication::translate("MainWindow", "New Project ...", nullptr));
        actionOpenProject->setText(QCoreApplication::translate("MainWindow", "Open Project ...", nullptr));
        actionPrevPageQ->setText(QCoreApplication::translate("MainWindow", "Previous Page", nullptr));
        actionNextPageW->setText(QCoreApplication::translate("MainWindow", "Next Page", nullptr));
        actionCloseProject->setText(QCoreApplication::translate("MainWindow", "Close Project", nullptr));
        actionQuit->setText(QCoreApplication::translate("MainWindow", "Quit", nullptr));
        actionSettings->setText(QCoreApplication::translate("MainWindow", "Settings ...", nullptr));
        actionFirstPage->setText(QCoreApplication::translate("MainWindow", "First Page", nullptr));
        actionLastPage->setText(QCoreApplication::translate("MainWindow", "Last Page", nullptr));
        actionAbout->setText(QCoreApplication::translate("MainWindow", "About", nullptr));
        actionFixDpi->setText(QCoreApplication::translate("MainWindow", "Fix DPI ...", nullptr));
        actionRelinking->setText(QCoreApplication::translate("MainWindow", "Relinking ...", nullptr));
        actionSwitchFilter1->setText(QCoreApplication::translate("MainWindow", "Switch filter to orientation", nullptr));
        actionSwitchFilter2->setText(QCoreApplication::translate("MainWindow", "Switch filter to orientation", nullptr));
#if QT_CONFIG(tooltip)
        actionSwitchFilter2->setToolTip(QCoreApplication::translate("MainWindow", "Switch filter to split pages", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSwitchFilter3->setText(QCoreApplication::translate("MainWindow", "Switch filter to orientation", nullptr));
#if QT_CONFIG(tooltip)
        actionSwitchFilter3->setToolTip(QCoreApplication::translate("MainWindow", "Switch filter to deskew", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSwitchFilter4->setText(QCoreApplication::translate("MainWindow", "Switch filter to orientation", nullptr));
#if QT_CONFIG(tooltip)
        actionSwitchFilter4->setToolTip(QCoreApplication::translate("MainWindow", "Switch filter to select content", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSwitchFilter5->setText(QCoreApplication::translate("MainWindow", "Switch filter to orientation", nullptr));
#if QT_CONFIG(tooltip)
        actionSwitchFilter5->setToolTip(QCoreApplication::translate("MainWindow", "Switch filter to margins", nullptr));
#endif // QT_CONFIG(tooltip)
        actionSwitchFilter6->setText(QCoreApplication::translate("MainWindow", "Switch filter to orientation", nullptr));
#if QT_CONFIG(tooltip)
        actionSwitchFilter6->setToolTip(QCoreApplication::translate("MainWindow", "Switch filter to output", nullptr));
#endif // QT_CONFIG(tooltip)
        actionPixels->setText(QCoreApplication::translate("MainWindow", "Pixels", nullptr));
        actionMilimeters->setText(QCoreApplication::translate("MainWindow", "Millimetres", nullptr));
        actionInches->setText(QCoreApplication::translate("MainWindow", "Inches", nullptr));
        actionCentimetres->setText(QCoreApplication::translate("MainWindow", "Centimetres", nullptr));
        actionDefaults->setText(QCoreApplication::translate("MainWindow", "Default parameters ...", nullptr));
        actionPrevSelectedPageQ->setText(QCoreApplication::translate("MainWindow", "Previous Selected Page", nullptr));
#if QT_CONFIG(tooltip)
        actionPrevSelectedPageQ->setToolTip(QCoreApplication::translate("MainWindow", "Previous Selected Page", nullptr));
#endif // QT_CONFIG(tooltip)
        actionNextSelectedPageW->setText(QCoreApplication::translate("MainWindow", "Next Selected Page", nullptr));
#if QT_CONFIG(tooltip)
        actionNextSelectedPageW->setToolTip(QCoreApplication::translate("MainWindow", "Next Selected Page", nullptr));
#endif // QT_CONFIG(tooltip)
        actionNextSelectedPage->setText(QCoreApplication::translate("MainWindow", "Next Selected Page", nullptr));
#if QT_CONFIG(tooltip)
        actionNextSelectedPage->setToolTip(QCoreApplication::translate("MainWindow", "Next Selected Page", nullptr));
#endif // QT_CONFIG(tooltip)
        actionPrevSelectedPage->setText(QCoreApplication::translate("MainWindow", "Previous Selected Page", nullptr));
#if QT_CONFIG(tooltip)
        actionPrevSelectedPage->setToolTip(QCoreApplication::translate("MainWindow", "Previous Selected Page", nullptr));
#endif // QT_CONFIG(tooltip)
        actionGotoPage->setText(QCoreApplication::translate("MainWindow", "Go To Page", nullptr));
        actionMagnifyThumbnails->setText(QCoreApplication::translate("MainWindow", "Magnify thumbnails", nullptr));
        actionDiminishThumbnails->setText(QCoreApplication::translate("MainWindow", "Diminish thumbnails", nullptr));
        actionReloadPage->setText(QCoreApplication::translate("MainWindow", "Reload current page", nullptr));
#if QT_CONFIG(tooltip)
        actionReloadPage->setToolTip(QCoreApplication::translate("MainWindow", "Reload current page", nullptr));
#endif // QT_CONFIG(tooltip)
        menuDebug->setTitle(QCoreApplication::translate("MainWindow", "Tools", nullptr));
        menuUnits->setTitle(QCoreApplication::translate("MainWindow", "Units", nullptr));
        menuFile->setTitle(QCoreApplication::translate("MainWindow", "File", nullptr));
        menuHelp->setTitle(QCoreApplication::translate("MainWindow", "Help", nullptr));
        thumbnailsDockWidget->setWindowTitle(QCoreApplication::translate("MainWindow", "Thumbnails", nullptr));
#if QT_CONFIG(tooltip)
        focusButton->setToolTip(QCoreApplication::translate("MainWindow", "Keep current page in view.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        focusButton->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        focusButton->setText(QString());
#if QT_CONFIG(tooltip)
        prevPageBtn->setToolTip(QCoreApplication::translate("MainWindow", "Previous page.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        nextPageBtn->setToolTip(QCoreApplication::translate("MainWindow", "Next page.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        filterSelectedBtn->setToolTip(QCoreApplication::translate("MainWindow", "Navigate through selected pages only.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        magnifyThumbnailsBtn->setToolTip(QCoreApplication::translate("MainWindow", "Magnify thumbnails.  (Ctrl++)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        diminishThumbnailsBtn->setToolTip(QCoreApplication::translate("MainWindow", "Diminish thumbnails.  (Ctrl+-)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        gotoPageBtn->setToolTip(QCoreApplication::translate("MainWindow", "Go To Page...  (Ctrl+G)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        selectionModeBtn->setToolTip(QCoreApplication::translate("MainWindow", "Enable multi page selection mode.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        thumbColumnViewBtn->setToolTip(QCoreApplication::translate("MainWindow", "Single/multi column display.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        sortingOrderBtn->setToolTip(QCoreApplication::translate("MainWindow", "Sorting order.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        deviationHighlightingBtn->setToolTip(QCoreApplication::translate("MainWindow", "Highlight deviation.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        thumbView->setStatusTip(QCoreApplication::translate("MainWindow", "Use Home, End, PgUp/Q, PgDown/W to navigate between pages or Shift+PgUp/Q and Shift+PgDown/W to navigate between selected ones. Alt+Wheel - scale thumbnails.", nullptr));
#endif // QT_CONFIG(statustip)
        filterDockWidget->setWindowTitle(QCoreApplication::translate("MainWindow", "Filters", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MainWindow: public Ui_MainWindow {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MAINWINDOW_H
