/********************************************************************************
** Form generated from reading UI file 'OptionsWidgetBinarizationWolf.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGETBINARIZATIONWOLF_H
#define UI_OPTIONSWIDGETBINARIZATIONWOLF_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QWidget>

namespace output {

class Ui_OptionsWidgetBinarizationWolf
{
public:
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout;
    QLabel *wolfDeltaLabel;
    QDoubleSpinBox *wolfDelta;
    QLabel *windowSizeLabel;
    QSpinBox *windowSize;
    QLabel *lowerBoundLabel;
    QSpinBox *lowerBound;
    QLabel *upperBoundLabel;
    QSpinBox *upperBound;
    QLabel *wolfCoefLabel;
    QDoubleSpinBox *wolfCoef;
    QSpacerItem *horizontalSpacer_2;

    void setupUi(QWidget *output__OptionsWidgetBinarizationWolf)
    {
        if (output__OptionsWidgetBinarizationWolf->objectName().isEmpty())
            output__OptionsWidgetBinarizationWolf->setObjectName("output__OptionsWidgetBinarizationWolf");
        output__OptionsWidgetBinarizationWolf->resize(148, 100);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(output__OptionsWidgetBinarizationWolf->sizePolicy().hasHeightForWidth());
        output__OptionsWidgetBinarizationWolf->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(output__OptionsWidgetBinarizationWolf);
        horizontalLayout->setSpacing(4);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(0, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        wolfDeltaLabel = new QLabel(output__OptionsWidgetBinarizationWolf);
        wolfDeltaLabel->setObjectName("wolfDeltaLabel");

        gridLayout->addWidget(wolfDeltaLabel, 0, 0, 1, 1);

        wolfDelta = new QDoubleSpinBox(output__OptionsWidgetBinarizationWolf);
        wolfDelta->setObjectName("wolfDelta");
        wolfDelta->setMinimum(-100.000000000000000);
        wolfDelta->setMaximum(100.000000000000000);
        wolfDelta->setSingleStep(1.000000000000000);

        gridLayout->addWidget(wolfDelta, 0, 1, 1, 1);

        windowSizeLabel = new QLabel(output__OptionsWidgetBinarizationWolf);
        windowSizeLabel->setObjectName("windowSizeLabel");

        gridLayout->addWidget(windowSizeLabel, 1, 0, 1, 1);

        windowSize = new QSpinBox(output__OptionsWidgetBinarizationWolf);
        windowSize->setObjectName("windowSize");
        windowSize->setMinimum(3);
        windowSize->setMaximum(9999);

        gridLayout->addWidget(windowSize, 1, 1, 1, 1);

        lowerBoundLabel = new QLabel(output__OptionsWidgetBinarizationWolf);
        lowerBoundLabel->setObjectName("lowerBoundLabel");

        gridLayout->addWidget(lowerBoundLabel, 2, 0, 1, 1);

        lowerBound = new QSpinBox(output__OptionsWidgetBinarizationWolf);
        lowerBound->setObjectName("lowerBound");
        lowerBound->setMinimum(1);
        lowerBound->setMaximum(254);

        gridLayout->addWidget(lowerBound, 2, 1, 1, 1);

        upperBoundLabel = new QLabel(output__OptionsWidgetBinarizationWolf);
        upperBoundLabel->setObjectName("upperBoundLabel");

        gridLayout->addWidget(upperBoundLabel, 3, 0, 1, 1);

        upperBound = new QSpinBox(output__OptionsWidgetBinarizationWolf);
        upperBound->setObjectName("upperBound");
        upperBound->setMinimum(1);
        upperBound->setMaximum(254);

        gridLayout->addWidget(upperBound, 3, 1, 1, 1);

        wolfCoefLabel = new QLabel(output__OptionsWidgetBinarizationWolf);
        wolfCoefLabel->setObjectName("wolfCoefLabel");

        gridLayout->addWidget(wolfCoefLabel, 4, 0, 1, 1);

        wolfCoef = new QDoubleSpinBox(output__OptionsWidgetBinarizationWolf);
        wolfCoef->setObjectName("wolfCoef");
        wolfCoef->setMinimum(0.010000000000000);
        wolfCoef->setMaximum(9.990000000000000);
        wolfCoef->setSingleStep(0.010000000000000);

        gridLayout->addWidget(wolfCoef, 4, 1, 1, 1);


        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(0, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        retranslateUi(output__OptionsWidgetBinarizationWolf);

        QMetaObject::connectSlotsByName(output__OptionsWidgetBinarizationWolf);
    } // setupUi

    void retranslateUi(QWidget *output__OptionsWidgetBinarizationWolf)
    {
        output__OptionsWidgetBinarizationWolf->setWindowTitle(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Form", nullptr));
        wolfDeltaLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Delta:", nullptr));
#if QT_CONFIG(tooltip)
        wolfDelta->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Default value is 0.", nullptr));
#endif // QT_CONFIG(tooltip)
        windowSizeLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Window size:", nullptr));
#if QT_CONFIG(tooltip)
        windowSize->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "The dimensions of a pixel neighborhood to consider.", nullptr));
#endif // QT_CONFIG(tooltip)
        lowerBoundLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Lower bound: ", nullptr));
#if QT_CONFIG(tooltip)
        lowerBound->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "The minimum possible gray level that can be made white.", nullptr));
#endif // QT_CONFIG(tooltip)
        upperBoundLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Upper Bound: ", nullptr));
#if QT_CONFIG(tooltip)
        upperBound->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "The maximum possible gray level that can be made black.", nullptr));
#endif // QT_CONFIG(tooltip)
        wolfCoefLabel->setText(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Coeff:", nullptr));
#if QT_CONFIG(tooltip)
        wolfCoef->setToolTip(QCoreApplication::translate("output::OptionsWidgetBinarizationWolf", "Default value is 0.3.", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

} // namespace output

namespace output {
namespace Ui {
    class OptionsWidgetBinarizationWolf: public Ui_OptionsWidgetBinarizationWolf {};
} // namespace Ui
} // namespace output

#endif // UI_OPTIONSWIDGETBINARIZATIONWOLF_H
