/****************************************************************************
** Meta object code from reading C++ file 'MainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../scantailor-advanced-1.1.1/src/app/MainWindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "openProject",
        "",
        "projectFile",
        "autoSaveProject",
        "goFirstPage",
        "goLastPage",
        "goNextPage",
        "goPrevPage",
        "goNextSelectedPage",
        "goPrevSelectedPage",
        "execGotoPageDialog",
        "goToPage",
        "PageId",
        "pageId",
        "ThumbnailSequence::SelectionAction",
        "selectionAction",
        "currentPageChanged",
        "PageInfo",
        "pageInfo",
        "QRectF",
        "thumbRect",
        "ThumbnailSequence::SelectionFlags",
        "flags",
        "pageContextMenuRequested",
        "QPoint",
        "screenPos",
        "selected",
        "pastLastPageContextMenuRequested",
        "thumbViewFocusToggled",
        "checked",
        "thumbViewScrolled",
        "filterSelectionChanged",
        "QItemSelection",
        "switchFilter1",
        "switchFilter2",
        "switchFilter3",
        "switchFilter4",
        "switchFilter5",
        "switchFilter6",
        "pageOrderingChanged",
        "idx",
        "reloadRequested",
        "startBatchProcessing",
        "stopBatchProcessing",
        "MainAreaAction",
        "mainArea",
        "invalidateThumbnail",
        "invalidateAllThumbnails",
        "showRelinkingDialog",
        "filterResult",
        "BackgroundTaskPtr",
        "task",
        "FilterResultPtr",
        "result",
        "debugToggled",
        "enabled",
        "fixDpiDialogRequested",
        "fixedDpiSubmitted",
        "saveProjectTriggered",
        "saveProjectAsTriggered",
        "newProject",
        "newProjectCreated",
        "ProjectCreationContext*",
        "context",
        "projectOpened",
        "ProjectOpeningContext*",
        "closeProject",
        "openSettingsDialog",
        "openDefaultParamsDialog",
        "onSettingsChanged",
        "showAboutDialog",
        "handleOutOfMemorySituation",
        "reloadCurrentPage"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'openProject'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'autoSaveProject'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goFirstPage'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goLastPage'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goNextPage'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goPrevPage'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goNextSelectedPage'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goPrevSelectedPage'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'execGotoPageDialog'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goToPage'
        QtMocHelpers::SlotData<void(const PageId &, ThumbnailSequence::SelectionAction)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'goToPage'
        QtMocHelpers::SlotData<void(const PageId &)>(12, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'currentPageChanged'
        QtMocHelpers::SlotData<void(const PageInfo &, const QRectF &, ThumbnailSequence::SelectionFlags)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 }, { 0x80000000 | 22, 23 },
        }}),
        // Slot 'pageContextMenuRequested'
        QtMocHelpers::SlotData<void(const PageInfo &, const QPoint &, bool)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 25, 26 }, { QMetaType::Bool, 27 },
        }}),
        // Slot 'pastLastPageContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'thumbViewFocusToggled'
        QtMocHelpers::SlotData<void(bool)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 30 },
        }}),
        // Slot 'thumbViewScrolled'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 33, 27 },
        }}),
        // Slot 'switchFilter1'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switchFilter2'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switchFilter3'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switchFilter4'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switchFilter5'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'switchFilter6'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'pageOrderingChanged'
        QtMocHelpers::SlotData<void(int)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 41 },
        }}),
        // Slot 'reloadRequested'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'startBatchProcessing'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'stopBatchProcessing'
        QtMocHelpers::SlotData<void(enum MainAreaAction)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 45, 46 },
        }}),
        // Slot 'stopBatchProcessing'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'invalidateThumbnail'
        QtMocHelpers::SlotData<void(const PageId &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'invalidateThumbnail'
        QtMocHelpers::SlotData<void(const PageInfo &)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'invalidateAllThumbnails'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showRelinkingDialog'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'filterResult'
        QtMocHelpers::SlotData<void(const BackgroundTaskPtr &, const FilterResultPtr &)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 51, 52 }, { 0x80000000 | 53, 54 },
        }}),
        // Slot 'debugToggled'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 56 },
        }}),
        // Slot 'fixDpiDialogRequested'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'fixedDpiSubmitted'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveProjectTriggered'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveProjectAsTriggered'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'newProject'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'newProjectCreated'
        QtMocHelpers::SlotData<void(ProjectCreationContext *)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 63, 64 },
        }}),
        // Slot 'openProject'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'projectOpened'
        QtMocHelpers::SlotData<void(ProjectOpeningContext *)>(65, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 66, 64 },
        }}),
        // Slot 'closeProject'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openSettingsDialog'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'openDefaultParamsDialog'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSettingsChanged'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showAboutDialog'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'handleOutOfMemorySituation'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'reloadCurrentPage'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->openProject((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->autoSaveProject(); break;
        case 2: _t->goFirstPage(); break;
        case 3: _t->goLastPage(); break;
        case 4: _t->goNextPage(); break;
        case 5: _t->goPrevPage(); break;
        case 6: _t->goNextSelectedPage(); break;
        case 7: _t->goPrevSelectedPage(); break;
        case 8: _t->execGotoPageDialog(); break;
        case 9: _t->goToPage((*reinterpret_cast<std::add_pointer_t<PageId>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<ThumbnailSequence::SelectionAction>>(_a[2]))); break;
        case 10: _t->goToPage((*reinterpret_cast<std::add_pointer_t<PageId>>(_a[1]))); break;
        case 11: _t->currentPageChanged((*reinterpret_cast<std::add_pointer_t<PageInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QRectF>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ThumbnailSequence::SelectionFlags>>(_a[3]))); break;
        case 12: _t->pageContextMenuRequested((*reinterpret_cast<std::add_pointer_t<PageInfo>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 13: _t->pastLastPageContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 14: _t->thumbViewFocusToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->thumbViewScrolled(); break;
        case 16: _t->filterSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1]))); break;
        case 17: _t->switchFilter1(); break;
        case 18: _t->switchFilter2(); break;
        case 19: _t->switchFilter3(); break;
        case 20: _t->switchFilter4(); break;
        case 21: _t->switchFilter5(); break;
        case 22: _t->switchFilter6(); break;
        case 23: _t->pageOrderingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->reloadRequested(); break;
        case 25: _t->startBatchProcessing(); break;
        case 26: _t->stopBatchProcessing((*reinterpret_cast<std::add_pointer_t<enum MainAreaAction>>(_a[1]))); break;
        case 27: _t->stopBatchProcessing(); break;
        case 28: _t->invalidateThumbnail((*reinterpret_cast<std::add_pointer_t<PageId>>(_a[1]))); break;
        case 29: _t->invalidateThumbnail((*reinterpret_cast<std::add_pointer_t<PageInfo>>(_a[1]))); break;
        case 30: _t->invalidateAllThumbnails(); break;
        case 31: _t->showRelinkingDialog(); break;
        case 32: _t->filterResult((*reinterpret_cast<std::add_pointer_t<BackgroundTaskPtr>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<FilterResultPtr>>(_a[2]))); break;
        case 33: _t->debugToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->fixDpiDialogRequested(); break;
        case 35: _t->fixedDpiSubmitted(); break;
        case 36: _t->saveProjectTriggered(); break;
        case 37: _t->saveProjectAsTriggered(); break;
        case 38: _t->newProject(); break;
        case 39: _t->newProjectCreated((*reinterpret_cast<std::add_pointer_t<ProjectCreationContext*>>(_a[1]))); break;
        case 40: _t->openProject(); break;
        case 41: _t->projectOpened((*reinterpret_cast<std::add_pointer_t<ProjectOpeningContext*>>(_a[1]))); break;
        case 42: _t->closeProject(); break;
        case 43: _t->openSettingsDialog(); break;
        case 44: _t->openDefaultParamsDialog(); break;
        case 45: _t->onSettingsChanged(); break;
        case 46: _t->showAboutDialog(); break;
        case 47: _t->handleOutOfMemorySituation(); break;
        case 48: _t->reloadCurrentPage(); break;
        default: ;
        }
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 49)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 49;
    }
    return _id;
}
QT_WARNING_POP
