/********************************************************************************
** Form generated from reading UI file 'ApplyDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_APPLYDIALOG_H
#define UI_APPLYDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

namespace fix_orientation {

class Ui_ApplyDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_5;
    QRadioButton *thisPageOnlyRB;
    QRadioButton *allPagesRB;
    QRadioButton *thisPageAndFollowersRB;
    QRadioButton *thisEveryOtherRB;
    QRadioButton *everyOtherRB;
    QRadioButton *selectedPagesRB;
    QHBoxLayout *horizontalLayout_3;
    QSpacerItem *horizontalSpacer_6;
    QLabel *selectedPagesHint;
    QRadioButton *everyOtherSelectedRB;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_5;
    QLabel *everyOtherSelectedHint;
    QSpacerItem *spacerItem;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *fix_orientation__ApplyDialog)
    {
        if (fix_orientation__ApplyDialog->objectName().isEmpty())
            fix_orientation__ApplyDialog->setObjectName("fix_orientation__ApplyDialog");
        fix_orientation__ApplyDialog->setWindowModality(Qt::WindowModal);
        fix_orientation__ApplyDialog->resize(364, 294);
        vboxLayout = new QVBoxLayout(fix_orientation__ApplyDialog);
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(fix_orientation__ApplyDialog);
        groupBox->setObjectName("groupBox");
        verticalLayout_5 = new QVBoxLayout(groupBox);
        verticalLayout_5->setObjectName("verticalLayout_5");
        thisPageOnlyRB = new QRadioButton(groupBox);
        thisPageOnlyRB->setObjectName("thisPageOnlyRB");
        thisPageOnlyRB->setChecked(true);

        verticalLayout_5->addWidget(thisPageOnlyRB);

        allPagesRB = new QRadioButton(groupBox);
        allPagesRB->setObjectName("allPagesRB");

        verticalLayout_5->addWidget(allPagesRB);

        thisPageAndFollowersRB = new QRadioButton(groupBox);
        thisPageAndFollowersRB->setObjectName("thisPageAndFollowersRB");

        verticalLayout_5->addWidget(thisPageAndFollowersRB);

        thisEveryOtherRB = new QRadioButton(groupBox);
        thisEveryOtherRB->setObjectName("thisEveryOtherRB");

        verticalLayout_5->addWidget(thisEveryOtherRB);

        everyOtherRB = new QRadioButton(groupBox);
        everyOtherRB->setObjectName("everyOtherRB");

        verticalLayout_5->addWidget(everyOtherRB);

        selectedPagesRB = new QRadioButton(groupBox);
        selectedPagesRB->setObjectName("selectedPagesRB");

        verticalLayout_5->addWidget(selectedPagesRB);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        horizontalSpacer_6 = new QSpacerItem(30, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_6);

        selectedPagesHint = new QLabel(groupBox);
        selectedPagesHint->setObjectName("selectedPagesHint");
        QFont font;
        font.setPointSize(7);
        selectedPagesHint->setFont(font);

        horizontalLayout_3->addWidget(selectedPagesHint);


        verticalLayout_5->addLayout(horizontalLayout_3);

        everyOtherSelectedRB = new QRadioButton(groupBox);
        everyOtherSelectedRB->setObjectName("everyOtherSelectedRB");

        verticalLayout_5->addWidget(everyOtherSelectedRB);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_5 = new QSpacerItem(30, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);

        everyOtherSelectedHint = new QLabel(groupBox);
        everyOtherSelectedHint->setObjectName("everyOtherSelectedHint");
        everyOtherSelectedHint->setFont(font);

        horizontalLayout_5->addWidget(everyOtherSelectedHint);


        verticalLayout_5->addLayout(horizontalLayout_5);


        vboxLayout->addWidget(groupBox);

        spacerItem = new QSpacerItem(17, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);

        buttonBox = new QDialogButtonBox(fix_orientation__ApplyDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(fix_orientation__ApplyDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, fix_orientation__ApplyDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(fix_orientation__ApplyDialog);
    } // setupUi

    void retranslateUi(QDialog *fix_orientation__ApplyDialog)
    {
        fix_orientation__ApplyDialog->setWindowTitle(QCoreApplication::translate("fix_orientation::ApplyDialog", "Fix Orientation", nullptr));
        groupBox->setTitle(QCoreApplication::translate("fix_orientation::ApplyDialog", "Apply to", nullptr));
        thisPageOnlyRB->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "This page only (already applied)", nullptr));
        allPagesRB->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "All pages", nullptr));
        thisPageAndFollowersRB->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "This page and the following ones", nullptr));
        thisEveryOtherRB->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "This page and the following every other page", nullptr));
        everyOtherRB->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "Every other page", nullptr));
        selectedPagesRB->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "Selected pages", nullptr));
        selectedPagesHint->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "Use Ctrl+Click / Shift+Click to select multiple pages.", nullptr));
        everyOtherSelectedRB->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "Every other selected page", nullptr));
        everyOtherSelectedHint->setText(QCoreApplication::translate("fix_orientation::ApplyDialog", "The current page will be included.", nullptr));
    } // retranslateUi

};

} // namespace fix_orientation

namespace fix_orientation {
namespace Ui {
    class ApplyDialog: public Ui_ApplyDialog {};
} // namespace Ui
} // namespace fix_orientation

#endif // UI_APPLYDIALOG_H
