// Copyright (C) 2019  Joseph Artsimovich <joseph.artsimovich@gmail.com>, 4lex4 <4lex49@zoho.com>
// Use of this source code is governed by the GNU GPLv3 license that can be found in the LICENSE file.

#include "Constants.h"

namespace constants {
const double SQRT_2 = 1.4142135623730950488016887242097;

const double PI = 3.141592653589793238462643383279502884;

const double DEG2RAD = PI / 180.0;

const double RAD2DEG = 180.0 / PI;

const double INCH2MM = 25.4;

const double MM2INCH = 1.0 / INCH2MM;

const double DPM2DPI = 0.0254;

const double DPI2DPM = 1.0 / DPM2DPI;
}  // namespace constants
