/********************************************************************************
** Form generated from reading UI file 'RemovePagesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_REMOVEPAGESDIALOG_H
#define UI_REMOVEPAGESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_RemovePagesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *icon;
    QSpacerItem *horizontalSpacer;
    QLabel *text;
    QLabel *multiPageWarning;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *RemovePagesDialog)
    {
        if (RemovePagesDialog->objectName().isEmpty())
            RemovePagesDialog->setObjectName("RemovePagesDialog");
        RemovePagesDialog->resize(352, 111);
        verticalLayout = new QVBoxLayout(RemovePagesDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        icon = new QLabel(RemovePagesDialog);
        icon->setObjectName("icon");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(icon->sizePolicy().hasHeightForWidth());
        icon->setSizePolicy(sizePolicy);
        icon->setText(QString::fromUtf8("ICON"));

        horizontalLayout->addWidget(icon);

        horizontalSpacer = new QSpacerItem(10, 20, QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        text = new QLabel(RemovePagesDialog);
        text->setObjectName("text");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(text->sizePolicy().hasHeightForWidth());
        text->setSizePolicy(sizePolicy1);
        text->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout->addWidget(text);


        verticalLayout->addLayout(horizontalLayout);

        multiPageWarning = new QLabel(RemovePagesDialog);
        multiPageWarning->setObjectName("multiPageWarning");
        multiPageWarning->setEnabled(true);
        multiPageWarning->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(multiPageWarning);

        verticalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(RemovePagesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(RemovePagesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, RemovePagesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, RemovePagesDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(RemovePagesDialog);
    } // setupUi

    void retranslateUi(QDialog *RemovePagesDialog)
    {
        RemovePagesDialog->setWindowTitle(QCoreApplication::translate("RemovePagesDialog", "Remove Pages", nullptr));
        text->setText(QCoreApplication::translate("RemovePagesDialog", "Remove %1 page(s) from project?", nullptr));
        multiPageWarning->setText(QCoreApplication::translate("RemovePagesDialog", "Corresponding output files will be deleted, while input files will remain.", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RemovePagesDialog: public Ui_RemovePagesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_REMOVEPAGESDIALOG_H
