/********************************************************************************
** Form generated from reading UI file 'SettingsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETTINGSDIALOG_H
#define UI_SETTINGSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SettingsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *enableOpenglCb;
    QLabel *openglDeviceLabel;
    QCheckBox *autoSaveProjectCB;
    QHBoxLayout *horizontalLayout;
    QLabel *colorSchemeLabel;
    QComboBox *colorSchemeBox;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_4;
    QLabel *languageLabel;
    QComboBox *languageBox;
    QSpacerItem *horizontalSpacer_4;
    QGroupBox *thumbnailsParams;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout_8;
    QHBoxLayout *horizontalLayout_9;
    QLabel *thumbnailQualityLabel;
    QSpinBox *thumbnailQualitySB;
    QSpacerItem *horizontalSpacer_7;
    QHBoxLayout *horizontalLayout_10;
    QLabel *thumbnailSizeLabel;
    QSpinBox *thumbnailSizeSB;
    QSpacerItem *horizontalSpacer_8;
    QCheckBox *singleColumnThumbnailsCB;
    QCheckBox *cancelingSelectionQuestionCB;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_2;
    QGridLayout *gridLayout;
    QLabel *tiffCompressionBWLabel;
    QComboBox *tiffCompressionBWBox;
    QLabel *tiffCompressionColorLabel;
    QComboBox *tiffCompressionColorBox;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *groupBox_4;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *blackOnWhiteDetectionCB;
    QCheckBox *blackOnWhiteDetectionAtOutputCB;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *highlightDeviationCB;
    QGroupBox *deviationParams;
    QHBoxLayout *horizontalLayout_3;
    QGridLayout *gridLayout_2;
    QLabel *deskewDeviationLabel;
    QLabel *selectContentDeviationLabel;
    QDoubleSpinBox *selectContentDeviationCoefSB;
    QLabel *marginsDeviationLabel;
    QDoubleSpinBox *deskewDeviationCoefSB;
    QDoubleSpinBox *deskewDeviationThresholdSB;
    QDoubleSpinBox *selectContentDeviationThresholdSB;
    QDoubleSpinBox *marginsDeviationCoefSB;
    QDoubleSpinBox *marginsDeviationThresholdSB;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SettingsDialog)
    {
        if (SettingsDialog->objectName().isEmpty())
            SettingsDialog->setObjectName("SettingsDialog");
        SettingsDialog->resize(337, 442);
        verticalLayout = new QVBoxLayout(SettingsDialog);
        verticalLayout->setObjectName("verticalLayout");
        tabWidget = new QTabWidget(SettingsDialog);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setDocumentMode(true);
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_4 = new QVBoxLayout(tab);
        verticalLayout_4->setObjectName("verticalLayout_4");
        groupBox = new QGroupBox(tab);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setObjectName("verticalLayout_2");
        enableOpenglCb = new QCheckBox(groupBox);
        enableOpenglCb->setObjectName("enableOpenglCb");

        verticalLayout_2->addWidget(enableOpenglCb);

        openglDeviceLabel = new QLabel(groupBox);
        openglDeviceLabel->setObjectName("openglDeviceLabel");

        verticalLayout_2->addWidget(openglDeviceLabel);

        autoSaveProjectCB = new QCheckBox(groupBox);
        autoSaveProjectCB->setObjectName("autoSaveProjectCB");

        verticalLayout_2->addWidget(autoSaveProjectCB);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        colorSchemeLabel = new QLabel(groupBox);
        colorSchemeLabel->setObjectName("colorSchemeLabel");

        horizontalLayout->addWidget(colorSchemeLabel);

        colorSchemeBox = new QComboBox(groupBox);
        colorSchemeBox->setObjectName("colorSchemeBox");

        horizontalLayout->addWidget(colorSchemeBox);

        horizontalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        languageLabel = new QLabel(groupBox);
        languageLabel->setObjectName("languageLabel");

        horizontalLayout_4->addWidget(languageLabel);

        languageBox = new QComboBox(groupBox);
        languageBox->setObjectName("languageBox");

        horizontalLayout_4->addWidget(languageBox);

        horizontalSpacer_4 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);


        verticalLayout_2->addLayout(horizontalLayout_4);

        enableOpenglCb->raise();
        autoSaveProjectCB->raise();
        openglDeviceLabel->raise();

        verticalLayout_4->addWidget(groupBox);

        thumbnailsParams = new QGroupBox(tab);
        thumbnailsParams->setObjectName("thumbnailsParams");
        verticalLayout_7 = new QVBoxLayout(thumbnailsParams);
        verticalLayout_7->setObjectName("verticalLayout_7");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        thumbnailQualityLabel = new QLabel(thumbnailsParams);
        thumbnailQualityLabel->setObjectName("thumbnailQualityLabel");

        horizontalLayout_9->addWidget(thumbnailQualityLabel);

        thumbnailQualitySB = new QSpinBox(thumbnailsParams);
        thumbnailQualitySB->setObjectName("thumbnailQualitySB");
        thumbnailQualitySB->setMinimum(100);
        thumbnailQualitySB->setMaximum(1500);
        thumbnailQualitySB->setSingleStep(25);

        horizontalLayout_9->addWidget(thumbnailQualitySB);


        horizontalLayout_8->addLayout(horizontalLayout_9);

        horizontalSpacer_7 = new QSpacerItem(18, 1, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_7);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        thumbnailSizeLabel = new QLabel(thumbnailsParams);
        thumbnailSizeLabel->setObjectName("thumbnailSizeLabel");

        horizontalLayout_10->addWidget(thumbnailSizeLabel);

        thumbnailSizeSB = new QSpinBox(thumbnailsParams);
        thumbnailSizeSB->setObjectName("thumbnailSizeSB");
        thumbnailSizeSB->setMinimum(100);
        thumbnailSizeSB->setMaximum(2000);
        thumbnailSizeSB->setSingleStep(25);

        horizontalLayout_10->addWidget(thumbnailSizeSB);


        horizontalLayout_8->addLayout(horizontalLayout_10);

        horizontalSpacer_8 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_8);


        verticalLayout_7->addLayout(horizontalLayout_8);

        singleColumnThumbnailsCB = new QCheckBox(thumbnailsParams);
        singleColumnThumbnailsCB->setObjectName("singleColumnThumbnailsCB");

        verticalLayout_7->addWidget(singleColumnThumbnailsCB);

        cancelingSelectionQuestionCB = new QCheckBox(thumbnailsParams);
        cancelingSelectionQuestionCB->setObjectName("cancelingSelectionQuestionCB");

        verticalLayout_7->addWidget(cancelingSelectionQuestionCB);


        verticalLayout_4->addWidget(thumbnailsParams);

        groupBox_2 = new QGroupBox(tab);
        groupBox_2->setObjectName("groupBox_2");
        horizontalLayout_2 = new QHBoxLayout(groupBox_2);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        tiffCompressionBWLabel = new QLabel(groupBox_2);
        tiffCompressionBWLabel->setObjectName("tiffCompressionBWLabel");

        gridLayout->addWidget(tiffCompressionBWLabel, 0, 0, 1, 1);

        tiffCompressionBWBox = new QComboBox(groupBox_2);
        tiffCompressionBWBox->setObjectName("tiffCompressionBWBox");

        gridLayout->addWidget(tiffCompressionBWBox, 0, 1, 1, 1);

        tiffCompressionColorLabel = new QLabel(groupBox_2);
        tiffCompressionColorLabel->setObjectName("tiffCompressionColorLabel");

        gridLayout->addWidget(tiffCompressionColorLabel, 1, 0, 1, 1);

        tiffCompressionColorBox = new QComboBox(groupBox_2);
        tiffCompressionColorBox->setObjectName("tiffCompressionColorBox");

        gridLayout->addWidget(tiffCompressionColorBox, 1, 1, 1, 1);


        horizontalLayout_2->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_4->addWidget(groupBox_2);

        verticalSpacer_2 = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_5 = new QVBoxLayout(tab_2);
        verticalLayout_5->setObjectName("verticalLayout_5");
        groupBox_4 = new QGroupBox(tab_2);
        groupBox_4->setObjectName("groupBox_4");
        verticalLayout_6 = new QVBoxLayout(groupBox_4);
        verticalLayout_6->setObjectName("verticalLayout_6");
        blackOnWhiteDetectionCB = new QCheckBox(groupBox_4);
        blackOnWhiteDetectionCB->setObjectName("blackOnWhiteDetectionCB");

        verticalLayout_6->addWidget(blackOnWhiteDetectionCB);

        blackOnWhiteDetectionAtOutputCB = new QCheckBox(groupBox_4);
        blackOnWhiteDetectionAtOutputCB->setObjectName("blackOnWhiteDetectionAtOutputCB");

        verticalLayout_6->addWidget(blackOnWhiteDetectionAtOutputCB);


        verticalLayout_5->addWidget(groupBox_4);

        groupBox_3 = new QGroupBox(tab_2);
        groupBox_3->setObjectName("groupBox_3");
        verticalLayout_3 = new QVBoxLayout(groupBox_3);
        verticalLayout_3->setObjectName("verticalLayout_3");
        highlightDeviationCB = new QCheckBox(groupBox_3);
        highlightDeviationCB->setObjectName("highlightDeviationCB");

        verticalLayout_3->addWidget(highlightDeviationCB);

        deviationParams = new QGroupBox(groupBox_3);
        deviationParams->setObjectName("deviationParams");
        horizontalLayout_3 = new QHBoxLayout(deviationParams);
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        deskewDeviationLabel = new QLabel(deviationParams);
        deskewDeviationLabel->setObjectName("deskewDeviationLabel");

        gridLayout_2->addWidget(deskewDeviationLabel, 0, 0, 1, 1);

        selectContentDeviationLabel = new QLabel(deviationParams);
        selectContentDeviationLabel->setObjectName("selectContentDeviationLabel");

        gridLayout_2->addWidget(selectContentDeviationLabel, 1, 0, 1, 1);

        selectContentDeviationCoefSB = new QDoubleSpinBox(deviationParams);
        selectContentDeviationCoefSB->setObjectName("selectContentDeviationCoefSB");
        selectContentDeviationCoefSB->setMinimum(0.100000000000000);
        selectContentDeviationCoefSB->setMaximum(9.990000000000000);
        selectContentDeviationCoefSB->setSingleStep(0.050000000000000);

        gridLayout_2->addWidget(selectContentDeviationCoefSB, 1, 1, 1, 1);

        marginsDeviationLabel = new QLabel(deviationParams);
        marginsDeviationLabel->setObjectName("marginsDeviationLabel");

        gridLayout_2->addWidget(marginsDeviationLabel, 2, 0, 1, 1);

        deskewDeviationCoefSB = new QDoubleSpinBox(deviationParams);
        deskewDeviationCoefSB->setObjectName("deskewDeviationCoefSB");
        deskewDeviationCoefSB->setMinimum(0.100000000000000);
        deskewDeviationCoefSB->setMaximum(9.990000000000000);
        deskewDeviationCoefSB->setSingleStep(0.050000000000000);

        gridLayout_2->addWidget(deskewDeviationCoefSB, 0, 1, 1, 1);

        deskewDeviationThresholdSB = new QDoubleSpinBox(deviationParams);
        deskewDeviationThresholdSB->setObjectName("deskewDeviationThresholdSB");
        deskewDeviationThresholdSB->setDecimals(1);
        deskewDeviationThresholdSB->setMaximum(999.000000000000000);
        deskewDeviationThresholdSB->setSingleStep(1.000000000000000);

        gridLayout_2->addWidget(deskewDeviationThresholdSB, 0, 2, 1, 1);

        selectContentDeviationThresholdSB = new QDoubleSpinBox(deviationParams);
        selectContentDeviationThresholdSB->setObjectName("selectContentDeviationThresholdSB");
        selectContentDeviationThresholdSB->setDecimals(1);
        selectContentDeviationThresholdSB->setMaximum(999.000000000000000);
        selectContentDeviationThresholdSB->setSingleStep(1.000000000000000);

        gridLayout_2->addWidget(selectContentDeviationThresholdSB, 1, 2, 1, 1);

        marginsDeviationCoefSB = new QDoubleSpinBox(deviationParams);
        marginsDeviationCoefSB->setObjectName("marginsDeviationCoefSB");
        marginsDeviationCoefSB->setMinimum(0.100000000000000);
        marginsDeviationCoefSB->setMaximum(9.990000000000000);
        marginsDeviationCoefSB->setSingleStep(0.050000000000000);

        gridLayout_2->addWidget(marginsDeviationCoefSB, 2, 1, 1, 1);

        marginsDeviationThresholdSB = new QDoubleSpinBox(deviationParams);
        marginsDeviationThresholdSB->setObjectName("marginsDeviationThresholdSB");
        marginsDeviationThresholdSB->setDecimals(1);
        marginsDeviationThresholdSB->setMaximum(999.000000000000000);
        marginsDeviationThresholdSB->setSingleStep(1.000000000000000);

        gridLayout_2->addWidget(marginsDeviationThresholdSB, 2, 2, 1, 1);


        horizontalLayout_3->addLayout(gridLayout_2);

        horizontalSpacer_3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_3->addWidget(deviationParams);


        verticalLayout_5->addWidget(groupBox_3);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

        tabWidget->addTab(tab_2, QString());

        verticalLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(SettingsDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(SettingsDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, SettingsDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SettingsDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(SettingsDialog);
    } // setupUi

    void retranslateUi(QDialog *SettingsDialog)
    {
        SettingsDialog->setWindowTitle(QCoreApplication::translate("SettingsDialog", "Settings", nullptr));
        groupBox->setTitle(QCoreApplication::translate("SettingsDialog", "User Interface", nullptr));
        enableOpenglCb->setText(QCoreApplication::translate("SettingsDialog", "Accelerate user interface with OpenGL", nullptr));
        openglDeviceLabel->setText(QCoreApplication::translate("SettingsDialog", "Device: %1", nullptr));
        autoSaveProjectCB->setText(QCoreApplication::translate("SettingsDialog", "Auto-save the existing project", nullptr));
        colorSchemeLabel->setText(QCoreApplication::translate("SettingsDialog", "Color Scheme: ", nullptr));
        languageLabel->setText(QCoreApplication::translate("SettingsDialog", "Language: ", nullptr));
        thumbnailsParams->setTitle(QCoreApplication::translate("SettingsDialog", "Thumbnails", nullptr));
        thumbnailQualityLabel->setText(QCoreApplication::translate("SettingsDialog", "Quality:", nullptr));
#if QT_CONFIG(tooltip)
        thumbnailQualitySB->setToolTip(QCoreApplication::translate("SettingsDialog", "The pixel size of the thumbnail image. The default value is 200.", nullptr));
#endif // QT_CONFIG(tooltip)
        thumbnailSizeLabel->setText(QCoreApplication::translate("SettingsDialog", "Size:", nullptr));
#if QT_CONFIG(tooltip)
        thumbnailSizeSB->setToolTip(QCoreApplication::translate("SettingsDialog", "The thumbnail size in the view. The default value is 250.", nullptr));
#endif // QT_CONFIG(tooltip)
        singleColumnThumbnailsCB->setText(QCoreApplication::translate("SettingsDialog", "Single column thumbnail display", nullptr));
        cancelingSelectionQuestionCB->setText(QCoreApplication::translate("SettingsDialog", "Show question on canceling multi page selection", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("SettingsDialog", "Saving", nullptr));
        tiffCompressionBWLabel->setText(QCoreApplication::translate("SettingsDialog", "B&W Compression: ", nullptr));
        tiffCompressionColorLabel->setText(QCoreApplication::translate("SettingsDialog", "Color Compression: ", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("SettingsDialog", "General", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("SettingsDialog", "White on black detection", nullptr));
#if QT_CONFIG(tooltip)
        blackOnWhiteDetectionCB->setToolTip(QCoreApplication::translate("SettingsDialog", "Auto detect pages with light content on dark background. The corrections to all the auto algorithms are made for such pages.", nullptr));
#endif // QT_CONFIG(tooltip)
        blackOnWhiteDetectionCB->setText(QCoreApplication::translate("SettingsDialog", "Auto detect light content on dark background", nullptr));
#if QT_CONFIG(tooltip)
        blackOnWhiteDetectionAtOutputCB->setToolTip(QCoreApplication::translate("SettingsDialog", "Whether to use auto detection at the output stage. The wrong result can be changed manually in the output filter options.", nullptr));
#endif // QT_CONFIG(tooltip)
        blackOnWhiteDetectionAtOutputCB->setText(QCoreApplication::translate("SettingsDialog", "Use auto detection at the output stage", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("SettingsDialog", "Deviation", nullptr));
        highlightDeviationCB->setText(QCoreApplication::translate("SettingsDialog", "Highlight the thumbnails of pages with high deviation", nullptr));
        deviationParams->setTitle(QCoreApplication::translate("SettingsDialog", "Params", nullptr));
        deskewDeviationLabel->setText(QCoreApplication::translate("SettingsDialog", "Deskew:", nullptr));
        selectContentDeviationLabel->setText(QCoreApplication::translate("SettingsDialog", "Select content:", nullptr));
#if QT_CONFIG(tooltip)
        selectContentDeviationCoefSB->setToolTip(QCoreApplication::translate("SettingsDialog", "Deviation multiplier: a higher value means lower sensivity.", nullptr));
#endif // QT_CONFIG(tooltip)
        marginsDeviationLabel->setText(QCoreApplication::translate("SettingsDialog", "Margins:", nullptr));
#if QT_CONFIG(tooltip)
        deskewDeviationCoefSB->setToolTip(QCoreApplication::translate("SettingsDialog", "Deviation multiplier: a higher value means lower sensivity.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        deskewDeviationThresholdSB->setToolTip(QCoreApplication::translate("SettingsDialog", "The minimum deviation to be highlighted.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        selectContentDeviationThresholdSB->setToolTip(QCoreApplication::translate("SettingsDialog", "The minimum deviation to be highlighted.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        marginsDeviationCoefSB->setToolTip(QCoreApplication::translate("SettingsDialog", "Deviation multiplier: a higher value means lower sensivity.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        marginsDeviationThresholdSB->setToolTip(QCoreApplication::translate("SettingsDialog", "The minimum deviation to be highlighted.", nullptr));
#endif // QT_CONFIG(tooltip)
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("SettingsDialog", "Processing", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SettingsDialog: public Ui_SettingsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SETTINGSDIALOG_H
