/********************************************************************************
** Form generated from reading UI file 'OptionsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGET_H
#define UI_OPTIONSWIDGET_H

#include <CollapsibleGroupBox.h>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace deskew {

class Ui_OptionsWidget
{
public:
    QVBoxLayout *vboxLayout;
    CollapsibleGroupBox *deskewGroupBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hboxLayout;
    QPushButton *autoBtn;
    QPushButton *manualBtn;
    QCheckBox *topEdgeCheckBox;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem;
    QDoubleSpinBox *angleSpinBox;
    QSpacerItem *spacerItem1;
    QHBoxLayout *hboxLayout2;
    QLabel *obliqueLabel;
    QDoubleSpinBox *obliqueSpinBox;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_9;
    QPushButton *applyDeskewBtn;
    QSpacerItem *horizontalSpacer_10;
    QSpacerItem *spacerItem2;

    void setupUi(QWidget *deskew__OptionsWidget)
    {
        if (deskew__OptionsWidget->objectName().isEmpty())
            deskew__OptionsWidget->setObjectName("deskew__OptionsWidget");
        deskew__OptionsWidget->resize(220, 146);
        vboxLayout = new QVBoxLayout(deskew__OptionsWidget);
        vboxLayout->setObjectName("vboxLayout");
        deskewGroupBox = new CollapsibleGroupBox(deskew__OptionsWidget);
        deskewGroupBox->setObjectName("deskewGroupBox");
        deskewGroupBox->setAlignment(Qt::AlignCenter);
        verticalLayout = new QVBoxLayout(deskewGroupBox);
        verticalLayout->setObjectName("verticalLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        autoBtn = new QPushButton(deskewGroupBox);
        autoBtn->setObjectName("autoBtn");
        autoBtn->setCheckable(true);
        autoBtn->setChecked(true);
        autoBtn->setAutoExclusive(true);

        hboxLayout->addWidget(autoBtn);

        manualBtn = new QPushButton(deskewGroupBox);
        manualBtn->setObjectName("manualBtn");
        manualBtn->setCheckable(true);
        manualBtn->setAutoExclusive(true);

        hboxLayout->addWidget(manualBtn);


        verticalLayout->addLayout(hboxLayout);

        topEdgeCheckBox = new QCheckBox(deskewGroupBox);
        topEdgeCheckBox->setObjectName("topEdgeCheckBox");
        topEdgeCheckBox->setChecked(false);

        verticalLayout->addWidget(topEdgeCheckBox);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        spacerItem = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem);

        angleSpinBox = new QDoubleSpinBox(deskewGroupBox);
        angleSpinBox->setObjectName("angleSpinBox");
        angleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        angleSpinBox->setSingleStep(0.010000000000000);

        hboxLayout1->addWidget(angleSpinBox);

        spacerItem1 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem1);


        verticalLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName("hboxLayout2");
        obliqueLabel = new QLabel(deskewGroupBox);
        obliqueLabel->setObjectName("obliqueLabel");

        hboxLayout2->addWidget(obliqueLabel);

        obliqueSpinBox = new QDoubleSpinBox(deskewGroupBox);
        obliqueSpinBox->setObjectName("obliqueSpinBox");
        obliqueSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        obliqueSpinBox->setMinimum(-5.000000000000000);
        obliqueSpinBox->setMaximum(5.000000000000000);
        obliqueSpinBox->setSingleStep(0.100000000000000);

        hboxLayout2->addWidget(obliqueSpinBox);


        verticalLayout->addLayout(hboxLayout2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_9 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_9);

        applyDeskewBtn = new QPushButton(deskewGroupBox);
        applyDeskewBtn->setObjectName("applyDeskewBtn");

        horizontalLayout->addWidget(applyDeskewBtn);

        horizontalSpacer_10 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_10);


        verticalLayout->addLayout(horizontalLayout);


        vboxLayout->addWidget(deskewGroupBox);

        spacerItem2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem2);


        retranslateUi(deskew__OptionsWidget);

        QMetaObject::connectSlotsByName(deskew__OptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *deskew__OptionsWidget)
    {
        deskew__OptionsWidget->setWindowTitle(QCoreApplication::translate("deskew::OptionsWidget", "Form", nullptr));
        deskewGroupBox->setTitle(QCoreApplication::translate("deskew::OptionsWidget", "Deskew", nullptr));
        autoBtn->setText(QCoreApplication::translate("deskew::OptionsWidget", "Auto", nullptr));
        manualBtn->setText(QCoreApplication::translate("deskew::OptionsWidget", "Manual", nullptr));
#if QT_CONFIG(tooltip)
        topEdgeCheckBox->setToolTip(QCoreApplication::translate("deskew::OptionsWidget", "Use the top page edge for deskew (recommended for book scans with dark background).", nullptr));
#endif // QT_CONFIG(tooltip)
        topEdgeCheckBox->setText(QCoreApplication::translate("deskew::OptionsWidget", "Top page edge (book scans)", nullptr));
        obliqueLabel->setText(QCoreApplication::translate("deskew::OptionsWidget", "Oblique (\302\260):", nullptr));
        applyDeskewBtn->setText(QCoreApplication::translate("deskew::OptionsWidget", "Apply To ...", nullptr));
    } // retranslateUi

};

} // namespace deskew

namespace deskew {
namespace Ui {
    class OptionsWidget: public Ui_OptionsWidget {};
} // namespace Ui
} // namespace deskew

#endif // UI_OPTIONSWIDGET_H
