// This file is autogenerated. Changes will be overwritten.
#include "EWIEGA46WW/moc_ApplyColorsDialog.cpp"
#include "EWIEGA46WW/moc_ChangeDewarpingDialog.cpp"
#include "EWIEGA46WW/moc_ChangeDpiDialog.cpp"
#include "EWIEGA46WW/moc_DespeckleView.cpp"
#include "EWIEGA46WW/moc_DewarpingView.cpp"
#include "EWIEGA46WW/moc_FillZoneEditor.cpp"
#include "EWIEGA46WW/moc_ImageView.cpp"
#include "EWIEGA46WW/moc_OptionsWidget.cpp"
#include "EWIEGA46WW/moc_OptionsWidgetBinarization.cpp"
#include "EWIEGA46WW/moc_OptionsWidgetBinarizationOtsu.cpp"
#include "EWIEGA46WW/moc_OptionsWidgetBinarizationSauvola.cpp"
#include "EWIEGA46WW/moc_OptionsWidgetBinarizationWolf.cpp"
#include "EWIEGA46WW/moc_PictureZoneEditor.cpp"
#include "EWIEGA46WW/moc_PictureZonePropDialog.cpp"
#include "EWIEGA46WW/moc_TabbedImageView.cpp"
