/********************************************************************************
** Form generated from reading UI file 'PictureZonePropDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PICTUREZONEPROPDIALOG_H
#define UI_PICTUREZONEPROPDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

namespace output {

class Ui_PictureZonePropDialog
{
public:
    QVBoxLayout *zoneverticalLayout;
    QRadioButton *zoneeraser3;
    QRadioButton *zonepainter2;
    QRadioButton *zoneeraser1;
    QRadioButton *zoneforeground;
    QRadioButton *zonebackground;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *output__PictureZonePropDialog)
    {
        if (output__PictureZonePropDialog->objectName().isEmpty())
            output__PictureZonePropDialog->setObjectName("output__PictureZonePropDialog");
        output__PictureZonePropDialog->resize(307, 116);
        zoneverticalLayout = new QVBoxLayout(output__PictureZonePropDialog);
        zoneverticalLayout->setObjectName("zoneverticalLayout");
        zoneeraser3 = new QRadioButton(output__PictureZonePropDialog);
        zoneeraser3->setObjectName("zoneeraser3");

        zoneverticalLayout->addWidget(zoneeraser3);

        zonepainter2 = new QRadioButton(output__PictureZonePropDialog);
        zonepainter2->setObjectName("zonepainter2");

        zoneverticalLayout->addWidget(zonepainter2);

        zoneeraser1 = new QRadioButton(output__PictureZonePropDialog);
        zoneeraser1->setObjectName("zoneeraser1");

        zoneverticalLayout->addWidget(zoneeraser1);

        zoneforeground = new QRadioButton(output__PictureZonePropDialog);
        zoneforeground->setObjectName("zoneforeground");

        zoneverticalLayout->addWidget(zoneforeground);

        zonebackground = new QRadioButton(output__PictureZonePropDialog);
        zonebackground->setObjectName("zonebackground");

        zoneverticalLayout->addWidget(zonebackground);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        zoneverticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(output__PictureZonePropDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        zoneverticalLayout->addWidget(buttonBox);


        retranslateUi(output__PictureZonePropDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, output__PictureZonePropDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, output__PictureZonePropDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(output__PictureZonePropDialog);
    } // setupUi

    void retranslateUi(QDialog *output__PictureZonePropDialog)
    {
        output__PictureZonePropDialog->setWindowTitle(QCoreApplication::translate("output::PictureZonePropDialog", "Zone Properties", nullptr));
        zoneeraser3->setText(QCoreApplication::translate("output::PictureZonePropDialog", "Subtract from all layers", nullptr));
        zonepainter2->setText(QCoreApplication::translate("output::PictureZonePropDialog", "Add to auto layer", nullptr));
        zoneeraser1->setText(QCoreApplication::translate("output::PictureZonePropDialog", "Subtract from auto layer", nullptr));
        zoneforeground->setText(QCoreApplication::translate("output::PictureZonePropDialog", "Add to foreground", nullptr));
        zonebackground->setText(QCoreApplication::translate("output::PictureZonePropDialog", "Add to background", nullptr));
    } // retranslateUi

};

} // namespace output

namespace output {
namespace Ui {
    class PictureZonePropDialog: public Ui_PictureZonePropDialog {};
} // namespace Ui
} // namespace output

#endif // UI_PICTUREZONEPROPDIALOG_H
