/********************************************************************************
** Form generated from reading UI file 'OptionsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGET_H
#define UI_OPTIONSWIDGET_H

#include <CollapsibleGroupBox.h>
#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace select_content {

class Ui_OptionsWidget
{
public:
    QVBoxLayout *verticalLayout_4;
    QLabel *splitPagesTipLabel;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    CollapsibleGroupBox *pageBoxGroup;
    QGridLayout *gridLayout;
    QPushButton *pageDetectDisableBtn;
    QPushButton *pageDetectAutoBtn;
    QPushButton *pageDetectManualBtn;
    QGroupBox *pageDetectOptions;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *fineTuneBtn;
    QWidget *dimensionsWidget;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_3;
    QGridLayout *gridLayout_3;
    QLabel *label;
    QLabel *label_2;
    QDoubleSpinBox *heightSpinBox;
    QDoubleSpinBox *widthSpinBox;
    QSpacerItem *horizontalSpacer;
    CollapsibleGroupBox *contentBoxGroup;
    QGridLayout *gridLayout_2;
    QPushButton *contentDetectAutoBtn;
    QPushButton *contentDetectDisableBtn;
    QPushButton *contentDetectManualBtn;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *applyToBtn;
    QSpacerItem *spacerItem1;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *select_content__OptionsWidget)
    {
        if (select_content__OptionsWidget->objectName().isEmpty())
            select_content__OptionsWidget->setObjectName("select_content__OptionsWidget");
        select_content__OptionsWidget->resize(216, 364);
        verticalLayout_4 = new QVBoxLayout(select_content__OptionsWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        splitPagesTipLabel = new QLabel(select_content__OptionsWidget);
        splitPagesTipLabel->setObjectName("splitPagesTipLabel");
        QFont font;
        font.setPointSize(8);
        splitPagesTipLabel->setFont(font);
        splitPagesTipLabel->setWordWrap(true);

        verticalLayout_4->addWidget(splitPagesTipLabel);

        groupBox = new QGroupBox(select_content__OptionsWidget);
        groupBox->setObjectName("groupBox");
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        pageBoxGroup = new CollapsibleGroupBox(groupBox);
        pageBoxGroup->setObjectName("pageBoxGroup");
        pageBoxGroup->setAlignment(Qt::AlignCenter);
        gridLayout = new QGridLayout(pageBoxGroup);
        gridLayout->setObjectName("gridLayout");
        pageDetectDisableBtn = new QPushButton(pageBoxGroup);
        pageDetectDisableBtn->setObjectName("pageDetectDisableBtn");
        pageDetectDisableBtn->setCheckable(true);
        pageDetectDisableBtn->setChecked(true);
        pageDetectDisableBtn->setAutoExclusive(true);

        gridLayout->addWidget(pageDetectDisableBtn, 0, 0, 1, 2);

        pageDetectAutoBtn = new QPushButton(pageBoxGroup);
        pageDetectAutoBtn->setObjectName("pageDetectAutoBtn");
        pageDetectAutoBtn->setCheckable(true);
        pageDetectAutoBtn->setAutoExclusive(true);

        gridLayout->addWidget(pageDetectAutoBtn, 1, 0, 1, 1);

        pageDetectManualBtn = new QPushButton(pageBoxGroup);
        pageDetectManualBtn->setObjectName("pageDetectManualBtn");
        pageDetectManualBtn->setCheckable(true);
        pageDetectManualBtn->setChecked(false);
        pageDetectManualBtn->setAutoExclusive(true);

        gridLayout->addWidget(pageDetectManualBtn, 1, 1, 1, 1);

        pageDetectOptions = new QGroupBox(pageBoxGroup);
        pageDetectOptions->setObjectName("pageDetectOptions");
        verticalLayout_2 = new QVBoxLayout(pageDetectOptions);
        verticalLayout_2->setSpacing(4);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(9, 9, 9, 9);
        fineTuneBtn = new QCheckBox(pageDetectOptions);
        fineTuneBtn->setObjectName("fineTuneBtn");

        verticalLayout_2->addWidget(fineTuneBtn);

        dimensionsWidget = new QWidget(pageDetectOptions);
        dimensionsWidget->setObjectName("dimensionsWidget");
        horizontalLayout_2 = new QHBoxLayout(dimensionsWidget);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        label = new QLabel(dimensionsWidget);
        label->setObjectName("label");

        gridLayout_3->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(dimensionsWidget);
        label_2->setObjectName("label_2");

        gridLayout_3->addWidget(label_2, 1, 0, 1, 1);

        heightSpinBox = new QDoubleSpinBox(dimensionsWidget);
        heightSpinBox->setObjectName("heightSpinBox");
        heightSpinBox->setDecimals(1);
        heightSpinBox->setMinimum(0.100000000000000);
        heightSpinBox->setMaximum(99999.000000000000000);

        gridLayout_3->addWidget(heightSpinBox, 1, 1, 1, 1);

        widthSpinBox = new QDoubleSpinBox(dimensionsWidget);
        widthSpinBox->setObjectName("widthSpinBox");
        widthSpinBox->setDecimals(1);
        widthSpinBox->setMinimum(0.100000000000000);
        widthSpinBox->setMaximum(99999.000000000000000);

        gridLayout_3->addWidget(widthSpinBox, 0, 1, 1, 1);


        horizontalLayout_2->addLayout(gridLayout_3);

        horizontalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout_2->addWidget(dimensionsWidget);


        gridLayout->addWidget(pageDetectOptions, 2, 0, 1, 2);


        verticalLayout->addWidget(pageBoxGroup);

        contentBoxGroup = new CollapsibleGroupBox(groupBox);
        contentBoxGroup->setObjectName("contentBoxGroup");
        contentBoxGroup->setAlignment(Qt::AlignCenter);
        gridLayout_2 = new QGridLayout(contentBoxGroup);
        gridLayout_2->setObjectName("gridLayout_2");
        contentDetectAutoBtn = new QPushButton(contentBoxGroup);
        contentDetectAutoBtn->setObjectName("contentDetectAutoBtn");
        contentDetectAutoBtn->setCheckable(true);
        contentDetectAutoBtn->setChecked(true);
        contentDetectAutoBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(contentDetectAutoBtn, 1, 1, 1, 1);

        contentDetectDisableBtn = new QPushButton(contentBoxGroup);
        contentDetectDisableBtn->setObjectName("contentDetectDisableBtn");
        contentDetectDisableBtn->setCheckable(true);
        contentDetectDisableBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(contentDetectDisableBtn, 0, 1, 1, 2);

        contentDetectManualBtn = new QPushButton(contentBoxGroup);
        contentDetectManualBtn->setObjectName("contentDetectManualBtn");
        contentDetectManualBtn->setCheckable(true);
        contentDetectManualBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(contentDetectManualBtn, 1, 2, 1, 1);


        verticalLayout->addWidget(contentBoxGroup);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        spacerItem = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        applyToBtn = new QPushButton(groupBox);
        applyToBtn->setObjectName("applyToBtn");

        hboxLayout->addWidget(applyToBtn);

        spacerItem1 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem1);


        verticalLayout->addLayout(hboxLayout);


        verticalLayout_4->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(1, 83, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        retranslateUi(select_content__OptionsWidget);

        QMetaObject::connectSlotsByName(select_content__OptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *select_content__OptionsWidget)
    {
        select_content__OptionsWidget->setWindowTitle(QCoreApplication::translate("select_content::OptionsWidget", "Form", nullptr));
        splitPagesTipLabel->setText(QCoreApplication::translate("select_content::OptionsWidget", "If content is cropped incorrectly, check the Split Pages step (Manual + Apply cut often works better with columns).", nullptr));
        groupBox->setTitle(QString());
        pageBoxGroup->setTitle(QCoreApplication::translate("select_content::OptionsWidget", "Page Box", nullptr));
        pageDetectDisableBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Disable", nullptr));
        pageDetectAutoBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Auto", nullptr));
        pageDetectManualBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Manual", nullptr));
        pageDetectOptions->setTitle(QCoreApplication::translate("select_content::OptionsWidget", "Options", nullptr));
#if QT_CONFIG(tooltip)
        fineTuneBtn->setToolTip(QCoreApplication::translate("select_content::OptionsWidget", "Shift with corners while they are in black. ", nullptr));
#endif // QT_CONFIG(tooltip)
        fineTuneBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Fine Tune Page Corners", nullptr));
        label->setText(QCoreApplication::translate("select_content::OptionsWidget", "Width", nullptr));
        label_2->setText(QCoreApplication::translate("select_content::OptionsWidget", "Height", nullptr));
        contentBoxGroup->setTitle(QCoreApplication::translate("select_content::OptionsWidget", "Content Box", nullptr));
        contentDetectAutoBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Auto", nullptr));
        contentDetectDisableBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Disable", nullptr));
        contentDetectManualBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Manual", nullptr));
        applyToBtn->setText(QCoreApplication::translate("select_content::OptionsWidget", "Apply to ...", nullptr));
    } // retranslateUi

};

} // namespace select_content

namespace select_content {
namespace Ui {
    class OptionsWidget: public Ui_OptionsWidget {};
} // namespace Ui
} // namespace select_content

#endif // UI_OPTIONSWIDGET_H
