// Copyright (C) 2020  Joseph Artsimovich <joseph.artsimovich@gmail.com>, 4lex4 <4lex49@zoho.com>
// Use of this source code is governed by the GNU GPLv3 license that can be found in the LICENSE file.

#ifndef SCANTAILOR_ZONES_ZONECREATIONMODE_H_
#define SCANTAILOR_ZONES_ZONECREATIONMODE_H_

enum class ZoneCreationMode { POLYGONAL, RECTANGULAR, LASSO };

#endif  // SCANTAILOR_ZONES_ZONECREATIONMODE_H_
