/********************************************************************************
** Form generated from reading UI file 'OptionsWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSWIDGET_H
#define UI_OPTIONSWIDGET_H

#include <CollapsibleGroupBox.h>
#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

namespace page_layout {

class Ui_OptionsWidget
{
public:
    QVBoxLayout *verticalLayout_4;
    CollapsibleGroupBox *marginsGroup;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_4;
    QCheckBox *autoMargins;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_3;
    QGridLayout *gridLayout;
    QToolButton *topBottomLink;
    QLabel *label;
    QLabel *label_4;
    QLabel *label_3;
    QToolButton *leftRightLink;
    QDoubleSpinBox *topMarginSpinBox;
    QDoubleSpinBox *bottomMarginSpinBox;
    QDoubleSpinBox *leftMarginSpinBox;
    QLabel *label_2;
    QDoubleSpinBox *rightMarginSpinBox;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_9;
    QPushButton *applyMarginsBtn;
    QPushButton *fixDpiBtn;
    QSpacerItem *horizontalSpacer_10;
    CollapsibleGroupBox *alignmentGroup;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_7;
    QSpacerItem *horizontalSpacer_7;
    QCheckBox *alignWithOthersCB;
    QSpacerItem *horizontalSpacer_8;
    QHBoxLayout *horizontalLayout_9;
    QSpacerItem *horizontalSpacer_17;
    QGridLayout *gridLayout_3;
    QLabel *hAlignmentModeLabel;
    QLabel *vAlignmentModeLabel;
    QComboBox *hAlignmentModeCB;
    QComboBox *vAlignmentModeCB;
    QSpacerItem *horizontalSpacer_18;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_6;
    QGridLayout *gridLayout_2;
    QToolButton *alignTopLeftBtn;
    QToolButton *alignTopBtn;
    QToolButton *alignTopRightBtn;
    QToolButton *alignLeftBtn;
    QToolButton *alignCenterBtn;
    QToolButton *alignRightBtn;
    QToolButton *alignBottomLeftBtn;
    QToolButton *alignBottomBtn;
    QToolButton *alignBottomRightBtn;
    QSpacerItem *horizontalSpacer_5;
    QHBoxLayout *horizontalLayout_6;
    QSpacerItem *horizontalSpacer_11;
    QPushButton *applyAlignmentBtn;
    QPushButton *matchSizeToAllBtn;
    QSpacerItem *horizontalSpacer_12;
    CollapsibleGroupBox *groupBox;
    QVBoxLayout *verticalLayout_6;
    QTextBrowser *textBrowser;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *page_layout__OptionsWidget)
    {
        if (page_layout__OptionsWidget->objectName().isEmpty())
            page_layout__OptionsWidget->setObjectName("page_layout__OptionsWidget");
        page_layout__OptionsWidget->resize(232, 804);
        verticalLayout_4 = new QVBoxLayout(page_layout__OptionsWidget);
        verticalLayout_4->setObjectName("verticalLayout_4");
        marginsGroup = new CollapsibleGroupBox(page_layout__OptionsWidget);
        marginsGroup->setObjectName("marginsGroup");
        marginsGroup->setAlignment(Qt::AlignCenter);
        verticalLayout = new QVBoxLayout(marginsGroup);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalSpacer_4 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);

        autoMargins = new QCheckBox(marginsGroup);
        autoMargins->setObjectName("autoMargins");
        autoMargins->setChecked(false);

        horizontalLayout->addWidget(autoMargins);

        horizontalSpacer_2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalSpacer_3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        topBottomLink = new QToolButton(marginsGroup);
        topBottomLink->setObjectName("topBottomLink");
        topBottomLink->setMinimumSize(QSize(20, 36));
        topBottomLink->setMaximumSize(QSize(20, 36));
        topBottomLink->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/stock-vchain.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        topBottomLink->setIcon(icon);
        topBottomLink->setIconSize(QSize(18, 24));

        gridLayout->addWidget(topBottomLink, 0, 2, 2, 1);

        label = new QLabel(marginsGroup);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        label_4 = new QLabel(marginsGroup);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 3, 0, 1, 1);

        label_3 = new QLabel(marginsGroup);
        label_3->setObjectName("label_3");

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        leftRightLink = new QToolButton(marginsGroup);
        leftRightLink->setObjectName("leftRightLink");
        leftRightLink->setMinimumSize(QSize(20, 36));
        leftRightLink->setMaximumSize(QSize(20, 36));
        leftRightLink->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        leftRightLink->setIcon(icon);
        leftRightLink->setIconSize(QSize(18, 24));
        leftRightLink->setChecked(false);

        gridLayout->addWidget(leftRightLink, 2, 2, 2, 1);

        topMarginSpinBox = new QDoubleSpinBox(marginsGroup);
        topMarginSpinBox->setObjectName("topMarginSpinBox");
        topMarginSpinBox->setDecimals(1);
        topMarginSpinBox->setMinimum(-9999.000000000000000);
        topMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout->addWidget(topMarginSpinBox, 0, 1, 1, 1);

        bottomMarginSpinBox = new QDoubleSpinBox(marginsGroup);
        bottomMarginSpinBox->setObjectName("bottomMarginSpinBox");
        bottomMarginSpinBox->setDecimals(1);
        bottomMarginSpinBox->setMinimum(-9999.000000000000000);
        bottomMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout->addWidget(bottomMarginSpinBox, 1, 1, 1, 1);

        leftMarginSpinBox = new QDoubleSpinBox(marginsGroup);
        leftMarginSpinBox->setObjectName("leftMarginSpinBox");
        leftMarginSpinBox->setDecimals(1);
        leftMarginSpinBox->setMinimum(-9999.000000000000000);
        leftMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout->addWidget(leftMarginSpinBox, 2, 1, 1, 1);

        label_2 = new QLabel(marginsGroup);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        rightMarginSpinBox = new QDoubleSpinBox(marginsGroup);
        rightMarginSpinBox->setObjectName("rightMarginSpinBox");
        rightMarginSpinBox->setDecimals(1);
        rightMarginSpinBox->setMinimum(-9999.000000000000000);
        rightMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout->addWidget(rightMarginSpinBox, 3, 1, 1, 1);


        horizontalLayout_2->addLayout(gridLayout);

        horizontalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_9 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_9);

        applyMarginsBtn = new QPushButton(marginsGroup);
        applyMarginsBtn->setObjectName("applyMarginsBtn");

        horizontalLayout_5->addWidget(applyMarginsBtn);

        fixDpiBtn = new QPushButton(marginsGroup);
        fixDpiBtn->setObjectName("fixDpiBtn");

        horizontalLayout_5->addWidget(fixDpiBtn);

        horizontalSpacer_10 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_10);


        verticalLayout->addLayout(horizontalLayout_5);


        verticalLayout_4->addWidget(marginsGroup);

        alignmentGroup = new CollapsibleGroupBox(page_layout__OptionsWidget);
        alignmentGroup->setObjectName("alignmentGroup");
        alignmentGroup->setAlignment(Qt::AlignCenter);
        verticalLayout_2 = new QVBoxLayout(alignmentGroup);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(-1, 9, 9, -1);
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalSpacer_7 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_7);

        alignWithOthersCB = new QCheckBox(alignmentGroup);
        alignWithOthersCB->setObjectName("alignWithOthersCB");
        alignWithOthersCB->setChecked(true);

        horizontalLayout_7->addWidget(alignWithOthersCB);

        horizontalSpacer_8 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_8);


        verticalLayout_2->addLayout(horizontalLayout_7);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        horizontalSpacer_17 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_17);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName("gridLayout_3");
        hAlignmentModeLabel = new QLabel(alignmentGroup);
        hAlignmentModeLabel->setObjectName("hAlignmentModeLabel");

        gridLayout_3->addWidget(hAlignmentModeLabel, 0, 0, 1, 1);

        vAlignmentModeLabel = new QLabel(alignmentGroup);
        vAlignmentModeLabel->setObjectName("vAlignmentModeLabel");

        gridLayout_3->addWidget(vAlignmentModeLabel, 1, 0, 1, 1);

        hAlignmentModeCB = new QComboBox(alignmentGroup);
        hAlignmentModeCB->addItem(QString());
        hAlignmentModeCB->addItem(QString());
        hAlignmentModeCB->addItem(QString());
        hAlignmentModeCB->setObjectName("hAlignmentModeCB");

        gridLayout_3->addWidget(hAlignmentModeCB, 0, 1, 1, 1);

        vAlignmentModeCB = new QComboBox(alignmentGroup);
        vAlignmentModeCB->addItem(QString());
        vAlignmentModeCB->addItem(QString());
        vAlignmentModeCB->addItem(QString());
        vAlignmentModeCB->setObjectName("vAlignmentModeCB");

        gridLayout_3->addWidget(vAlignmentModeCB, 1, 1, 1, 1);


        horizontalLayout_9->addLayout(gridLayout_3);

        horizontalSpacer_18 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_18);


        verticalLayout_2->addLayout(horizontalLayout_9);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalSpacer_6 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_6);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(16);
        gridLayout_2->setObjectName("gridLayout_2");
        alignTopLeftBtn = new QToolButton(alignmentGroup);
        alignTopLeftBtn->setObjectName("alignTopLeftBtn");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/stock-gravity-north-west.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignTopLeftBtn->setIcon(icon1);
        alignTopLeftBtn->setIconSize(QSize(24, 24));
        alignTopLeftBtn->setCheckable(true);
        alignTopLeftBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignTopLeftBtn, 0, 0, 1, 1);

        alignTopBtn = new QToolButton(alignmentGroup);
        alignTopBtn->setObjectName("alignTopBtn");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/stock-gravity-north.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignTopBtn->setIcon(icon2);
        alignTopBtn->setIconSize(QSize(24, 24));
        alignTopBtn->setCheckable(true);
        alignTopBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignTopBtn, 0, 1, 1, 1);

        alignTopRightBtn = new QToolButton(alignmentGroup);
        alignTopRightBtn->setObjectName("alignTopRightBtn");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/stock-gravity-north-east.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignTopRightBtn->setIcon(icon3);
        alignTopRightBtn->setIconSize(QSize(24, 24));
        alignTopRightBtn->setCheckable(true);
        alignTopRightBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignTopRightBtn, 0, 2, 1, 1);

        alignLeftBtn = new QToolButton(alignmentGroup);
        alignLeftBtn->setObjectName("alignLeftBtn");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/stock-gravity-west.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignLeftBtn->setIcon(icon4);
        alignLeftBtn->setIconSize(QSize(24, 24));
        alignLeftBtn->setCheckable(true);
        alignLeftBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignLeftBtn, 1, 0, 1, 1);

        alignCenterBtn = new QToolButton(alignmentGroup);
        alignCenterBtn->setObjectName("alignCenterBtn");
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/stock-center.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignCenterBtn->setIcon(icon5);
        alignCenterBtn->setIconSize(QSize(24, 24));
        alignCenterBtn->setCheckable(true);
        alignCenterBtn->setChecked(true);
        alignCenterBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignCenterBtn, 1, 1, 1, 1);

        alignRightBtn = new QToolButton(alignmentGroup);
        alignRightBtn->setObjectName("alignRightBtn");
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/stock-gravity-east.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignRightBtn->setIcon(icon6);
        alignRightBtn->setIconSize(QSize(24, 24));
        alignRightBtn->setCheckable(true);
        alignRightBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignRightBtn, 1, 2, 1, 1);

        alignBottomLeftBtn = new QToolButton(alignmentGroup);
        alignBottomLeftBtn->setObjectName("alignBottomLeftBtn");
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/stock-gravity-south-west.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignBottomLeftBtn->setIcon(icon7);
        alignBottomLeftBtn->setIconSize(QSize(24, 24));
        alignBottomLeftBtn->setCheckable(true);
        alignBottomLeftBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignBottomLeftBtn, 2, 0, 1, 1);

        alignBottomBtn = new QToolButton(alignmentGroup);
        alignBottomBtn->setObjectName("alignBottomBtn");
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/stock-gravity-south.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignBottomBtn->setIcon(icon8);
        alignBottomBtn->setIconSize(QSize(24, 24));
        alignBottomBtn->setCheckable(true);
        alignBottomBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignBottomBtn, 2, 1, 1, 1);

        alignBottomRightBtn = new QToolButton(alignmentGroup);
        alignBottomRightBtn->setObjectName("alignBottomRightBtn");
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/stock-gravity-south-east.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignBottomRightBtn->setIcon(icon9);
        alignBottomRightBtn->setIconSize(QSize(24, 24));
        alignBottomRightBtn->setCheckable(true);
        alignBottomRightBtn->setAutoExclusive(true);

        gridLayout_2->addWidget(alignBottomRightBtn, 2, 2, 1, 1);


        horizontalLayout_4->addLayout(gridLayout_2);

        horizontalSpacer_5 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_5);


        verticalLayout_2->addLayout(horizontalLayout_4);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        horizontalSpacer_11 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_11);

        applyAlignmentBtn = new QPushButton(alignmentGroup);
        applyAlignmentBtn->setObjectName("applyAlignmentBtn");

        horizontalLayout_6->addWidget(applyAlignmentBtn);

        matchSizeToAllBtn = new QPushButton(alignmentGroup);
        matchSizeToAllBtn->setObjectName("matchSizeToAllBtn");

        horizontalLayout_6->addWidget(matchSizeToAllBtn);

        horizontalSpacer_12 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_12);


        verticalLayout_2->addLayout(horizontalLayout_6);


        verticalLayout_4->addWidget(alignmentGroup);

        groupBox = new CollapsibleGroupBox(page_layout__OptionsWidget);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy);
        groupBox->setAlignment(Qt::AlignCenter);
        verticalLayout_6 = new QVBoxLayout(groupBox);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setSizeConstraint(QLayout::SetDefaultConstraint);
        textBrowser = new QTextBrowser(groupBox);
        textBrowser->setObjectName("textBrowser");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textBrowser->sizePolicy().hasHeightForWidth());
        textBrowser->setSizePolicy(sizePolicy1);
        textBrowser->setMinimumSize(QSize(0, 220));
        textBrowser->setBaseSize(QSize(0, 0));
        textBrowser->setAcceptDrops(false);
        textBrowser->setToolTipDuration(-1);
        textBrowser->setAutoFillBackground(false);
        textBrowser->setFrameShape(QFrame::NoFrame);
        textBrowser->setFrameShadow(QFrame::Plain);
        textBrowser->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        textBrowser->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        textBrowser->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        textBrowser->setAutoFormatting(QTextEdit::AutoNone);
        textBrowser->setLineWrapMode(QTextEdit::WidgetWidth);
        textBrowser->setReadOnly(true);
        textBrowser->setTextInteractionFlags(Qt::TextSelectableByKeyboard|Qt::TextSelectableByMouse);

        verticalLayout_6->addWidget(textBrowser);


        verticalLayout_4->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(188, 11, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_4->addItem(verticalSpacer);


        retranslateUi(page_layout__OptionsWidget);

        hAlignmentModeCB->setCurrentIndex(1);
        vAlignmentModeCB->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(page_layout__OptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *page_layout__OptionsWidget)
    {
        page_layout__OptionsWidget->setWindowTitle(QCoreApplication::translate("page_layout::OptionsWidget", "Form", nullptr));
        marginsGroup->setTitle(QCoreApplication::translate("page_layout::OptionsWidget", "Margins", nullptr));
        autoMargins->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Auto Margins", nullptr));
        topBottomLink->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        label->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Top", nullptr));
        label_4->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Right", nullptr));
        label_3->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Left", nullptr));
        leftRightLink->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        label_2->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Bottom", nullptr));
        applyMarginsBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Apply To ...", nullptr));
        fixDpiBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Fix DPI...", nullptr));
        alignmentGroup->setTitle(QCoreApplication::translate("page_layout::OptionsWidget", "Alignment", nullptr));
        alignWithOthersCB->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Match size with other pages", nullptr));
        hAlignmentModeLabel->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Horizontal mode:", nullptr));
        vAlignmentModeLabel->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Vertical mode:", nullptr));
        hAlignmentModeCB->setItemText(0, QCoreApplication::translate("page_layout::OptionsWidget", "Auto", "auto"));
        hAlignmentModeCB->setItemText(1, QCoreApplication::translate("page_layout::OptionsWidget", "Manual", "manual"));
        hAlignmentModeCB->setItemText(2, QCoreApplication::translate("page_layout::OptionsWidget", "Original", "original"));

        vAlignmentModeCB->setItemText(0, QCoreApplication::translate("page_layout::OptionsWidget", "Auto", "auto"));
        vAlignmentModeCB->setItemText(1, QCoreApplication::translate("page_layout::OptionsWidget", "Manual", "manual"));
        vAlignmentModeCB->setItemText(2, QCoreApplication::translate("page_layout::OptionsWidget", "Original", "original"));

        alignTopLeftBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignTopBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignTopRightBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignLeftBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignCenterBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignRightBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignBottomLeftBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignBottomBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        alignBottomRightBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "...", nullptr));
        applyAlignmentBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Apply To ...", nullptr));
        matchSizeToAllBtn->setText(QCoreApplication::translate("page_layout::OptionsWidget", "Match size to all pages", nullptr));
#if QT_CONFIG(tooltip)
        matchSizeToAllBtn->setToolTip(QCoreApplication::translate("page_layout::OptionsWidget", "Apply \"Match size with other pages\" to every page so they all share the same output size.", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(QCoreApplication::translate("page_layout::OptionsWidget", "Guides Help", nullptr));
#if QT_CONFIG(tooltip)
        textBrowser->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        textBrowser->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        textBrowser->setHtml(QCoreApplication::translate("page_layout::OptionsWidget", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'MS Shell Dlg 2'; font-size:8.25pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:7pt; font-weight:600;\">\342\200\243 Right-click</span><span style=\" font-size:7pt;\"> to create/remove guides from the </span><span style=\" font-size:7pt; font-weight:600;\">context menu</span><span style=\" font-size:7pt;\"> called.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:7pt; font-weight:600;\">\342\200\243 Right-click</span><span style=\" font-size:7pt;\"> on a guide to delete that guide from the </span><"
                        "span style=\" font-size:7pt; font-weight:600;\">context menu</span><span style=\" font-size:7pt;\"> called.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:7pt; font-weight:600;\">\342\200\243 Ctrl+Alt+LMB</span><span style=\" font-size:7pt;\"> - drag the guide under the cursor.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:7pt; font-weight:600;\">\342\200\243 Shift/Ctrl+LMB</span><span style=\" font-size:7pt;\"> on the content rectangle - drag the page content. Hold </span><span style=\" font-size:7pt; font-weight:600;\">Shift</span><span style=\" font-size:7pt;\"> pressed to restrict moving along the horizontal axis only or </span><span style=\" font-size:7pt; font-weight:600;\">Ctrl</span><span style=\" font-size:7pt;\"> for the vertical one. Hold </span><span style=\" font-size:7pt;"
                        " font-weight:600;\">Shift+Ctrl</span><span style=\" font-size:7pt;\"> for usual dragging.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:7pt; font-weight:600;\">\342\200\243 Double-click</span><span style=\" font-size:7pt;\"> on content - automatically attach that content to the nearest guide. Hold </span><span style=\" font-size:7pt; font-weight:600;\">Shift</span><span style=\" font-size:7pt;\"> pressed to select vertical guides only or </span><span style=\" font-size:7pt; font-weight:600;\">Ctrl</span><span style=\" font-size:7pt;\"> for horizontal ones. Hold </span><span style=\" font-size:7pt; font-weight:600;\">Shift+Ctrl</span><span style=\" font-size:7pt;\"> to attach that to both the nearest vertical and horizontal guides.</span></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:7pt; fon"
                        "t-weight:600;\">\342\200\243</span><span style=\" font-size:7pt;\"> Use the </span><span style=\" font-size:7pt; font-weight:600;\">context menu</span><span style=\" font-size:7pt;\"> to enable/disable showing the hard margins rectangle.</span></p></body></html>", nullptr));
    } // retranslateUi

};

} // namespace page_layout

namespace page_layout {
namespace Ui {
    class OptionsWidget: public Ui_OptionsWidget {};
} // namespace Ui
} // namespace page_layout

#endif // UI_OPTIONSWIDGET_H
