/****************************************************************************
** Meta object code from reading C++ file 'OptionsWidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../scantailor-advanced-1.1.1/src/core/filters/output/OptionsWidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'OptionsWidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN6output13OptionsWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto output::OptionsWidget::qt_create_metaobjectdata<qt_meta_tag_ZN6output13OptionsWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "output::OptionsWidget",
        "despeckleLevelChanged",
        "",
        "level",
        "bool*",
        "handled",
        "depthPerceptionChanged",
        "val",
        "tabChanged",
        "ImageViewTab",
        "tab",
        "distortionModelChanged",
        "dewarping::DistortionModel",
        "model",
        "changeDpiButtonClicked",
        "applyColorsButtonClicked",
        "applySplittingButtonClicked",
        "dpiChanged",
        "std::set<PageId>",
        "pages",
        "Dpi",
        "dpi",
        "applyColorsConfirmed",
        "applySplittingOptionsConfirmed",
        "colorModeChanged",
        "idx",
        "blackOnWhiteToggled",
        "value",
        "applyProcessingParamsClicked",
        "applyProcessingParamsConfirmed",
        "thresholdMethodChanged",
        "fillingColorChanged",
        "pictureShapeChanged",
        "pictureShapeSensitivityChanged",
        "higherSearchSensivityToggled",
        "checked",
        "wienerCoefChanged",
        "wienerWindowSizeChanged",
        "colorSegmentationToggled",
        "reduceNoiseChanged",
        "redAdjustmentChanged",
        "greenAdjustmentChanged",
        "blueAdjustmentChanged",
        "posterizeToggled",
        "posterizeLevelChanged",
        "posterizeNormalizationToggled",
        "posterizeForceBwToggled",
        "fillMarginsToggled",
        "fillOffcutToggled",
        "equalizeIlluminationToggled",
        "equalizeIlluminationColorToggled",
        "savitzkyGolaySmoothingToggled",
        "morphologicalSmoothingToggled",
        "splittingToggled",
        "bwForegroundToggled",
        "colorForegroundToggled",
        "originalBackgroundToggled",
        "binarizationSettingsChanged",
        "despeckleToggled",
        "despeckleSliderReleased",
        "despeckleSliderValueChanged",
        "applyDespeckleButtonClicked",
        "applyDespeckleConfirmed",
        "changeDewarpingButtonClicked",
        "dewarpingChanged",
        "DewarpingOptions",
        "opt",
        "applyDepthPerceptionButtonClicked",
        "applyDepthPerceptionConfirmed",
        "depthPerceptionChangedSlot",
        "updateBinarizationOptionsDisplay",
        "sendReloadRequested"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'despeckleLevelChanged'
        QtMocHelpers::SignalData<void(double, bool *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 3 }, { 0x80000000 | 4, 5 },
        }}),
        // Signal 'depthPerceptionChanged'
        QtMocHelpers::SignalData<void(double)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 },
        }}),
        // Slot 'tabChanged'
        QtMocHelpers::SlotData<void(ImageViewTab)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'distortionModelChanged'
        QtMocHelpers::SlotData<void(const dewarping::DistortionModel &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'changeDpiButtonClicked'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyColorsButtonClicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applySplittingButtonClicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'dpiChanged'
        QtMocHelpers::SlotData<void(const std::set<PageId> &, const Dpi &)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 20, 21 },
        }}),
        // Slot 'applyColorsConfirmed'
        QtMocHelpers::SlotData<void(const std::set<PageId> &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'applySplittingOptionsConfirmed'
        QtMocHelpers::SlotData<void(const std::set<PageId> &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'colorModeChanged'
        QtMocHelpers::SlotData<void(int)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'blackOnWhiteToggled'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'applyProcessingParamsClicked'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyProcessingParamsConfirmed'
        QtMocHelpers::SlotData<void(const std::set<PageId> &)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'thresholdMethodChanged'
        QtMocHelpers::SlotData<void(int)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'fillingColorChanged'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'pictureShapeChanged'
        QtMocHelpers::SlotData<void(int)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'pictureShapeSensitivityChanged'
        QtMocHelpers::SlotData<void(int)>(33, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'higherSearchSensivityToggled'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'wienerCoefChanged'
        QtMocHelpers::SlotData<void(double)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Double, 27 },
        }}),
        // Slot 'wienerWindowSizeChanged'
        QtMocHelpers::SlotData<void(int)>(37, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'colorSegmentationToggled'
        QtMocHelpers::SlotData<void(bool)>(38, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'reduceNoiseChanged'
        QtMocHelpers::SlotData<void(int)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'redAdjustmentChanged'
        QtMocHelpers::SlotData<void(int)>(40, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'greenAdjustmentChanged'
        QtMocHelpers::SlotData<void(int)>(41, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'blueAdjustmentChanged'
        QtMocHelpers::SlotData<void(int)>(42, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'posterizeToggled'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'posterizeLevelChanged'
        QtMocHelpers::SlotData<void(int)>(44, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'posterizeNormalizationToggled'
        QtMocHelpers::SlotData<void(bool)>(45, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'posterizeForceBwToggled'
        QtMocHelpers::SlotData<void(bool)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'fillMarginsToggled'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'fillOffcutToggled'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'equalizeIlluminationToggled'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'equalizeIlluminationColorToggled'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'savitzkyGolaySmoothingToggled'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'morphologicalSmoothingToggled'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'splittingToggled'
        QtMocHelpers::SlotData<void(bool)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'bwForegroundToggled'
        QtMocHelpers::SlotData<void(bool)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'colorForegroundToggled'
        QtMocHelpers::SlotData<void(bool)>(55, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'originalBackgroundToggled'
        QtMocHelpers::SlotData<void(bool)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'binarizationSettingsChanged'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'despeckleToggled'
        QtMocHelpers::SlotData<void(bool)>(58, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 35 },
        }}),
        // Slot 'despeckleSliderReleased'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'despeckleSliderValueChanged'
        QtMocHelpers::SlotData<void(int)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 27 },
        }}),
        // Slot 'applyDespeckleButtonClicked'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyDespeckleConfirmed'
        QtMocHelpers::SlotData<void(const std::set<PageId> &)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'changeDewarpingButtonClicked'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'dewarpingChanged'
        QtMocHelpers::SlotData<void(const std::set<PageId> &, const DewarpingOptions &)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 }, { 0x80000000 | 65, 66 },
        }}),
        // Slot 'applyDepthPerceptionButtonClicked'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyDepthPerceptionConfirmed'
        QtMocHelpers::SlotData<void(const std::set<PageId> &)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 19 },
        }}),
        // Slot 'depthPerceptionChangedSlot'
        QtMocHelpers::SlotData<void(int)>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Slot 'updateBinarizationOptionsDisplay'
        QtMocHelpers::SlotData<void(int)>(70, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Slot 'sendReloadRequested'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<OptionsWidget, qt_meta_tag_ZN6output13OptionsWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject output::OptionsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<FilterOptionsWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6output13OptionsWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6output13OptionsWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6output13OptionsWidgetE_t>.metaTypes,
    nullptr
} };

void output::OptionsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<OptionsWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->despeckleLevelChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[2]))); break;
        case 1: _t->depthPerceptionChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 2: _t->tabChanged((*reinterpret_cast<std::add_pointer_t<ImageViewTab>>(_a[1]))); break;
        case 3: _t->distortionModelChanged((*reinterpret_cast<std::add_pointer_t<dewarping::DistortionModel>>(_a[1]))); break;
        case 4: _t->changeDpiButtonClicked(); break;
        case 5: _t->applyColorsButtonClicked(); break;
        case 6: _t->applySplittingButtonClicked(); break;
        case 7: _t->dpiChanged((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Dpi>>(_a[2]))); break;
        case 8: _t->applyColorsConfirmed((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1]))); break;
        case 9: _t->applySplittingOptionsConfirmed((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1]))); break;
        case 10: _t->colorModeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 11: _t->blackOnWhiteToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->applyProcessingParamsClicked(); break;
        case 13: _t->applyProcessingParamsConfirmed((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1]))); break;
        case 14: _t->thresholdMethodChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->fillingColorChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 16: _t->pictureShapeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->pictureShapeSensitivityChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 18: _t->higherSearchSensivityToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->wienerCoefChanged((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 20: _t->wienerWindowSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->colorSegmentationToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 22: _t->reduceNoiseChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 23: _t->redAdjustmentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 24: _t->greenAdjustmentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 25: _t->blueAdjustmentChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 26: _t->posterizeToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->posterizeLevelChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 28: _t->posterizeNormalizationToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->posterizeForceBwToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->fillMarginsToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 31: _t->fillOffcutToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->equalizeIlluminationToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 33: _t->equalizeIlluminationColorToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 34: _t->savitzkyGolaySmoothingToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->morphologicalSmoothingToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 36: _t->splittingToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->bwForegroundToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->colorForegroundToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->originalBackgroundToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 40: _t->binarizationSettingsChanged(); break;
        case 41: _t->despeckleToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: _t->despeckleSliderReleased(); break;
        case 43: _t->despeckleSliderValueChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 44: _t->applyDespeckleButtonClicked(); break;
        case 45: _t->applyDespeckleConfirmed((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1]))); break;
        case 46: _t->changeDewarpingButtonClicked(); break;
        case 47: _t->dewarpingChanged((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DewarpingOptions>>(_a[2]))); break;
        case 48: _t->applyDepthPerceptionButtonClicked(); break;
        case 49: _t->applyDepthPerceptionConfirmed((*reinterpret_cast<std::add_pointer_t<std::set<PageId>>>(_a[1]))); break;
        case 50: _t->depthPerceptionChangedSlot((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 51: _t->updateBinarizationOptionsDisplay((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 52: _t->sendReloadRequested(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(double , bool * )>(_a, &OptionsWidget::despeckleLevelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (OptionsWidget::*)(double )>(_a, &OptionsWidget::depthPerceptionChanged, 1))
            return;
    }
}

const QMetaObject *output::OptionsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *output::OptionsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6output13OptionsWidgetE_t>.strings))
        return static_cast<void*>(this);
    return FilterOptionsWidget::qt_metacast(_clname);
}

int output::OptionsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = FilterOptionsWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 53)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 53;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 53)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 53;
    }
    return _id;
}

// SIGNAL 0
void output::OptionsWidget::despeckleLevelChanged(double _t1, bool * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void output::OptionsWidget::depthPerceptionChanged(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
