/********************************************************************************
** Form generated from reading UI file 'SystemLoadWidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SYSTEMLOADWIDGET_H
#define UI_SYSTEMLOADWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SystemLoadWidget
{
public:
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpacerItem *horizontalSpacer;
    QToolButton *minusBtn;
    QSlider *slider;
    QToolButton *plusBtn;

    void setupUi(QWidget *SystemLoadWidget)
    {
        if (SystemLoadWidget->objectName().isEmpty())
            SystemLoadWidget->setObjectName("SystemLoadWidget");
        SystemLoadWidget->resize(234, 40);
        horizontalLayout = new QHBoxLayout(SystemLoadWidget);
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        label = new QLabel(SystemLoadWidget);
        label->setObjectName("label");

        horizontalLayout->addWidget(label);

        horizontalSpacer = new QSpacerItem(6, 1, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        minusBtn = new QToolButton(SystemLoadWidget);
        minusBtn->setObjectName("minusBtn");
        minusBtn->setStyleSheet(QString::fromUtf8("#minusBtn {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#minusBtn:disabled {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#minusBtn:default {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#minusBtn:hover {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#minusBtn:pressed {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}"));
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/minus.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        minusBtn->setIcon(icon);
        minusBtn->setAutoRaise(true);

        horizontalLayout->addWidget(minusBtn);

        slider = new QSlider(SystemLoadWidget);
        slider->setObjectName("slider");
        slider->setTracking(false);
        slider->setOrientation(Qt::Horizontal);
        slider->setTickPosition(QSlider::NoTicks);

        horizontalLayout->addWidget(slider);

        plusBtn = new QToolButton(SystemLoadWidget);
        plusBtn->setObjectName("plusBtn");
        plusBtn->setStyleSheet(QString::fromUtf8("#plusBtn {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#plusBtn:disabled {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#plusBtn:default {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#plusBtn:hover {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}\n"
"\n"
"#plusBtn:pressed {\n"
"	border: none;\n"
"	background-color: transparent;\n"
"}"));
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/plus.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        plusBtn->setIcon(icon1);
        plusBtn->setAutoRaise(true);

        horizontalLayout->addWidget(plusBtn);


        retranslateUi(SystemLoadWidget);

        QMetaObject::connectSlotsByName(SystemLoadWidget);
    } // setupUi

    void retranslateUi(QWidget *SystemLoadWidget)
    {
        SystemLoadWidget->setWindowTitle(QCoreApplication::translate("SystemLoadWidget", "Form", nullptr));
        label->setText(QCoreApplication::translate("SystemLoadWidget", "System load", nullptr));
        minusBtn->setText(QCoreApplication::translate("SystemLoadWidget", "...", nullptr));
        plusBtn->setText(QCoreApplication::translate("SystemLoadWidget", "...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SystemLoadWidget: public Ui_SystemLoadWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SYSTEMLOADWIDGET_H
