/********************************************************************************
** Form generated from reading UI file 'DefaultParamsDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DEFAULTPARAMSDIALOG_H
#define UI_DEFAULTPARAMSDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_DefaultParamsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QComboBox *profileCB;
    QSpacerItem *horizontalSpacer;
    QPushButton *profileSaveButton;
    QPushButton *profileDeleteButton;
    QSpacerItem *horizontalSpacer_2;
    QLabel *label_7;
    QLabel *unitsLabel;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea_2;
    QWidget *scrollAreaWidgetContents_2;
    QGridLayout *gridLayout;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox;
    QVBoxLayout *_2;
    QHBoxLayout *_3;
    QSpacerItem *spacerItem;
    QToolButton *rotateLeftBtn;
    QToolButton *rotateRightBtn;
    QSpacerItem *spacerItem1;
    QHBoxLayout *_4;
    QSpacerItem *spacerItem2;
    QLabel *rotationIndicator;
    QSpacerItem *spacerItem3;
    QHBoxLayout *_5;
    QSpacerItem *spacerItem4;
    QPushButton *resetBtn;
    QSpacerItem *spacerItem5;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_2;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QGridLayout *gridLayout_2;
    QGroupBox *layoutModeGroupBox;
    QHBoxLayout *horizontalLayout_25;
    QSpacerItem *horizontalSpacer_57;
    QComboBox *layoutModeCB;
    QSpacerItem *horizontalSpacer_58;
    QGroupBox *pageLayoutGroup;
    QVBoxLayout *verticalLayout_8;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer_5;
    QToolButton *singlePageUncutBtn;
    QToolButton *pagePlusOffcutBtn;
    QToolButton *twoPagesBtn;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *horizontalSpacer_7;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *verticalSpacer_3;
    QSpacerItem *verticalSpacer_4;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_4;
    QScrollArea *scrollArea_3;
    QWidget *scrollAreaWidgetContents_3;
    QGridLayout *gridLayout_3;
    QSpacerItem *verticalSpacer_5;
    QSpacerItem *horizontalSpacer_9;
    QSpacerItem *horizontalSpacer_10;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *_6;
    QPushButton *deskewAutoBtn;
    QPushButton *deskewManualBtn;
    QHBoxLayout *_7;
    QSpacerItem *spacerItem6;
    QDoubleSpinBox *angleSpinBox;
    QSpacerItem *spacerItem7;
    QSpacerItem *verticalSpacer_6;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_5;
    QScrollArea *scrollArea_4;
    QWidget *scrollAreaWidgetContents_5;
    QGridLayout *gridLayout_7;
    QSpacerItem *verticalSpacer_8;
    QSpacerItem *verticalSpacer_7;
    QGroupBox *groupBox_5;
    QGridLayout *gridLayout_5;
    QPushButton *pageDetectDisableBtn;
    QGroupBox *pageDetectOptions;
    QVBoxLayout *verticalLayout_11;
    QCheckBox *fineTuneBtn;
    QWidget *dimensionsWidget;
    QHBoxLayout *horizontalLayout_4;
    QSpacerItem *horizontalSpacer_11;
    QGridLayout *gridLayout_6;
    QLabel *label;
    QLabel *label_2;
    QDoubleSpinBox *heightSpinBox;
    QDoubleSpinBox *widthSpinBox;
    QSpacerItem *horizontalSpacer_12;
    QPushButton *pageDetectAutoBtn;
    QPushButton *pageDetectManualBtn;
    QGroupBox *groupBox_4;
    QGridLayout *gridLayout_4;
    QPushButton *contentDetectDisableBtn;
    QPushButton *contentDetectAutoBtn;
    QSpacerItem *horizontalSpacer_13;
    QSpacerItem *horizontalSpacer_14;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_6;
    QScrollArea *scrollArea_5;
    QWidget *scrollAreaWidgetContents_6;
    QGridLayout *gridLayout_10;
    QSpacerItem *horizontalSpacer_27;
    QVBoxLayout *verticalLayout_15;
    QGroupBox *marginsGroup;
    QVBoxLayout *verticalLayout_14;
    QHBoxLayout *horizontalLayout_8;
    QSpacerItem *horizontalSpacer_21;
    QCheckBox *autoMargins;
    QSpacerItem *horizontalSpacer_22;
    QWidget *marginsWidget;
    QHBoxLayout *horizontalLayout_9;
    QSpacerItem *horizontalSpacer_23;
    QGridLayout *gridLayout_9;
    QToolButton *topBottomLink;
    QLabel *label_3;
    QLabel *label_4;
    QLabel *label_5;
    QToolButton *leftRightLink;
    QDoubleSpinBox *rightMarginSpinBox;
    QDoubleSpinBox *bottomMarginSpinBox;
    QDoubleSpinBox *topMarginSpinBox;
    QLabel *label_6;
    QDoubleSpinBox *leftMarginSpinBox;
    QSpacerItem *horizontalSpacer_24;
    QSpacerItem *verticalSpacer_13;
    QSpacerItem *horizontalSpacer_28;
    QSpacerItem *verticalSpacer_10;
    QSpacerItem *verticalSpacer_9;
    QGroupBox *alignmentGroup;
    QVBoxLayout *verticalLayout_12;
    QHBoxLayout *horizontalLayout_26;
    QSpacerItem *horizontalSpacer_15;
    QCheckBox *alignWithOthersCB;
    QSpacerItem *horizontalSpacer_16;
    QHBoxLayout *horizontalLayout_20;
    QSpacerItem *horizontalSpacer_74;
    QGridLayout *gridLayout_15;
    QLabel *hAlignmentModeLabel;
    QLabel *vAlignmentModeLabel;
    QComboBox *hAlignmentModeCB;
    QComboBox *vAlignmentModeCB;
    QSpacerItem *horizontalSpacer_75;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer_17;
    QGridLayout *gridLayout_8;
    QToolButton *alignTopLeftBtn;
    QToolButton *alignTopBtn;
    QToolButton *alignTopRightBtn;
    QToolButton *alignLeftBtn;
    QToolButton *alignCenterBtn;
    QToolButton *alignRightBtn;
    QToolButton *alignBottomLeftBtn;
    QToolButton *alignBottomBtn;
    QToolButton *alignBottomRightBtn;
    QSpacerItem *horizontalSpacer_18;
    QWidget *tab_6;
    QVBoxLayout *verticalLayout_7;
    QScrollArea *scrollArea_6;
    QWidget *scrollAreaWidgetContents_7;
    QGridLayout *gridLayout_14;
    QVBoxLayout *verticalLayout_33;
    QGroupBox *modePanel;
    QGridLayout *gridLayout_13;
    QVBoxLayout *verticalLayout_16;
    QGroupBox *commonOptions;
    QHBoxLayout *horizontalLayout_14;
    QSpacerItem *horizontalSpacer_35;
    QVBoxLayout *verticalLayout_13;
    QCheckBox *fillOffcutCB;
    QCheckBox *fillMarginsCB;
    QCheckBox *equalizeIlluminationCB;
    QCheckBox *equalizeIlluminationColorCB;
    QCheckBox *savitzkyGolaySmoothingCB;
    QCheckBox *morphologicalSmoothingCB;
    QSpacerItem *horizontalSpacer_36;
    QGroupBox *fillingOptions;
    QVBoxLayout *verticalLayout_18;
    QHBoxLayout *horizontalLayout_23;
    QSpacerItem *horizontalSpacer_43;
    QLabel *fillingColorLabel;
    QComboBox *fillingColorBox;
    QSpacerItem *horizontalSpacer_44;
    QGroupBox *colorOperationsOptions;
    QHBoxLayout *horizontalLayout_28;
    QSpacerItem *horizontalSpacer_60;
    QVBoxLayout *verticalLayout_31;
    QCheckBox *colorSegmentationCB;
    QWidget *segmenterOptionsWidget;
    QVBoxLayout *verticalLayout_32;
    QHBoxLayout *horizontalLayout_29;
    QSpacerItem *horizontalSpacer_61;
    QLabel *rLabel;
    QSpinBox *redAdjustmentSB;
    QLabel *gLabel;
    QSpinBox *greenAdjustmentSB;
    QLabel *bLabel;
    QSpinBox *blueAdjustmentSB;
    QSpacerItem *horizontalSpacer_62;
    QHBoxLayout *horizontalLayout_30;
    QSpacerItem *horizontalSpacer_63;
    QLabel *reduceNoiseLabel;
    QSpinBox *reduceNoiseSB;
    QSpacerItem *horizontalSpacer_64;
    QCheckBox *posterizeCB;
    QWidget *posterizeOptionsWidget;
    QVBoxLayout *verticalLayout_34;
    QHBoxLayout *horizontalLayout_31;
    QSpacerItem *horizontalSpacer_65;
    QLabel *posterizeLevelLabel;
    QSpinBox *posterizeLevelSB;
    QSpacerItem *horizontalSpacer_66;
    QHBoxLayout *horizontalLayout_33;
    QSpacerItem *horizontalSpacer_70;
    QCheckBox *posterizeNormalizationCB;
    QSpacerItem *horizontalSpacer_71;
    QHBoxLayout *horizontalLayout_32;
    QSpacerItem *horizontalSpacer_67;
    QCheckBox *posterizeForceBwCB;
    QSpacerItem *horizontalSpacer_68;
    QSpacerItem *horizontalSpacer_69;
    QSpacerItem *verticalSpacer_14;
    QVBoxLayout *verticalLayout_27;
    QGroupBox *thresholdOptions;
    QVBoxLayout *verticalLayout_19;
    QHBoxLayout *horizontalLayout_22;
    QSpacerItem *horizontalSpacer_42;
    QLabel *thresholMethodLabel;
    QComboBox *thresholdMethodBox;
    QSpacerItem *horizontalSpacer_41;
    QStackedWidget *binarizationOptions;
    QWidget *page;
    QVBoxLayout *verticalLayout_30;
    QVBoxLayout *verticalLayout_24;
    QHBoxLayout *horizontalLayout_11;
    QSpacerItem *horizontalSpacer_45;
    QLabel *thresholLabel;
    QSpacerItem *horizontalSpacer_46;
    QSlider *thresholdSlider;
    QHBoxLayout *horizontalLayout_10;
    QLabel *lighterThresholdLink;
    QToolButton *neutralThresholdBtn;
    QLabel *darkerThresholdLink;
    QSpacerItem *verticalSpacer_18;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_25;
    QHBoxLayout *horizontalLayout_12;
    QSpacerItem *horizontalSpacer_20;
    QGridLayout *gridLayout_11;
    QLabel *sauvolaCoefLabel;
    QSpinBox *sauvolaWindowSize;
    QLabel *windowSizeLabel;
    QDoubleSpinBox *sauvolaCoef;
    QSpacerItem *horizontalSpacer_19;
    QSpacerItem *verticalSpacer_19;
    QWidget *page_3;
    QVBoxLayout *verticalLayout_26;
    QHBoxLayout *horizontalLayout_17;
    QSpacerItem *horizontalSpacer_26;
    QGridLayout *gridLayout_12;
    QSpinBox *wolfWindowSize;
    QLabel *windowSizeLabel_2;
    QSpinBox *lowerBound;
    QLabel *upperBoundLabel;
    QLabel *lowerBoundLabel;
    QSpinBox *upperBound;
    QLabel *wolfCoefLabel;
    QDoubleSpinBox *wolfCoef;
    QSpacerItem *horizontalSpacer_25;
    QSpacerItem *verticalSpacer_20;
    QGroupBox *pictureShapeOptions;
    QVBoxLayout *verticalLayout_20;
    QHBoxLayout *horizontalLayout_15;
    QSpacerItem *horizontalSpacer_37;
    QComboBox *pictureShapeSelector;
    QSpacerItem *horizontalSpacer_38;
    QWidget *pictureShapeSensitivityOptions;
    QHBoxLayout *horizontalLayout_16;
    QSpacerItem *horizontalSpacer_39;
    QLabel *pictureShapeSensitivityLabel;
    QSpinBox *pictureShapeSensitivitySB;
    QSpacerItem *horizontalSpacer_40;
    QHBoxLayout *horizontalLayout_34;
    QSpacerItem *horizontalSpacer_72;
    QCheckBox *higherSearchSensitivityCB;
    QSpacerItem *horizontalSpacer_73;
    QSpacerItem *verticalSpacer_21;
    QHBoxLayout *horizontalLayout_13;
    QSpacerItem *horizontalSpacer_33;
    QComboBox *colorModeSelector;
    QSpacerItem *horizontalSpacer_34;
    QSpacerItem *verticalSpacer_15;
    QVBoxLayout *verticalLayout_17;
    QGroupBox *dpiPanel;
    QHBoxLayout *horizontalLayout_7;
    QSpacerItem *horizontalSpacer_30;
    QComboBox *dpiSelector;
    QSpacerItem *horizontalSpacer_29;
    QGroupBox *splittingOptions;
    QVBoxLayout *verticalLayout_21;
    QHBoxLayout *horizontalLayout_18;
    QSpacerItem *horizontalSpacer_47;
    QVBoxLayout *verticalLayout_22;
    QCheckBox *splittingCB;
    QRadioButton *bwForegroundRB;
    QHBoxLayout *horizontalLayout_27;
    QSpacerItem *horizontalSpacer_59;
    QCheckBox *originalBackgroundCB;
    QRadioButton *colorForegroundRB;
    QSpacerItem *horizontalSpacer_48;
    QGroupBox *despecklePanel;
    QVBoxLayout *verticalLayout_23;
    QVBoxLayout *verticalLayout_35;
    QHBoxLayout *horizontalLayout_37;
    QSpacerItem *horizontalSpacer_51;
    QVBoxLayout *verticalLayout_36;
    QCheckBox *despeckleCB;
    QSpacerItem *horizontalSpacer_52;
    QSlider *despeckleSlider;
    QGroupBox *dewarpingPanel;
    QVBoxLayout *verticalLayout_28;
    QHBoxLayout *horizontalLayout_19;
    QSpacerItem *horizontalSpacer_49;
    QComboBox *dewarpingModeCB;
    QSpacerItem *horizontalSpacer_50;
    QHBoxLayout *horizontalLayout_21;
    QSpacerItem *horizontalSpacer_53;
    QCheckBox *dewarpingPostDeskewCB;
    QSpacerItem *horizontalSpacer_54;
    QGroupBox *depthPerceptionPanel;
    QVBoxLayout *verticalLayout_29;
    QSlider *depthPerceptionSlider;
    QSpacerItem *verticalSpacer_22;
    QSpacerItem *horizontalSpacer_31;
    QSpacerItem *verticalSpacer_17;
    QSpacerItem *horizontalSpacer_32;
    QSpacerItem *verticalSpacer_16;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *DefaultParamsDialog)
    {
        if (DefaultParamsDialog->objectName().isEmpty())
            DefaultParamsDialog->setObjectName("DefaultParamsDialog");
        DefaultParamsDialog->resize(657, 555);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DefaultParamsDialog->sizePolicy().hasHeightForWidth());
        DefaultParamsDialog->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(DefaultParamsDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        profileCB = new QComboBox(DefaultParamsDialog);
        profileCB->setObjectName("profileCB");
        profileCB->setMinimumSize(QSize(100, 0));

        horizontalLayout->addWidget(profileCB);

        horizontalSpacer = new QSpacerItem(20, 1, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        profileSaveButton = new QPushButton(DefaultParamsDialog);
        profileSaveButton->setObjectName("profileSaveButton");

        horizontalLayout->addWidget(profileSaveButton);

        profileDeleteButton = new QPushButton(DefaultParamsDialog);
        profileDeleteButton->setObjectName("profileDeleteButton");

        horizontalLayout->addWidget(profileDeleteButton);

        horizontalSpacer_2 = new QSpacerItem(5, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        label_7 = new QLabel(DefaultParamsDialog);
        label_7->setObjectName("label_7");

        horizontalLayout->addWidget(label_7);

        unitsLabel = new QLabel(DefaultParamsDialog);
        unitsLabel->setObjectName("unitsLabel");
        unitsLabel->setMinimumSize(QSize(22, 0));
        QFont font;
        font.setBold(true);
        unitsLabel->setFont(font);
        unitsLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout->addWidget(unitsLabel);


        verticalLayout->addLayout(horizontalLayout);

        tabWidget = new QTabWidget(DefaultParamsDialog);
        tabWidget->setObjectName("tabWidget");
        tabWidget->setTabPosition(QTabWidget::North);
        tabWidget->setDocumentMode(true);
        tab = new QWidget();
        tab->setObjectName("tab");
        verticalLayout_3 = new QVBoxLayout(tab);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_3->setContentsMargins(0, 9, 0, 3);
        scrollArea_2 = new QScrollArea(tab);
        scrollArea_2->setObjectName("scrollArea_2");
        scrollArea_2->setFrameShape(QFrame::StyledPanel);
        scrollArea_2->setFrameShadow(QFrame::Sunken);
        scrollArea_2->setLineWidth(1);
        scrollArea_2->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName("scrollAreaWidgetContents_2");
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 637, 442));
        gridLayout = new QGridLayout(scrollAreaWidgetContents_2);
        gridLayout->setObjectName("gridLayout");
        horizontalSpacer_3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_3, 1, 0, 1, 1);

        horizontalSpacer_4 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout->addItem(horizontalSpacer_4, 1, 2, 1, 1);

        verticalSpacer = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer, 0, 1, 1, 1);

        groupBox = new QGroupBox(scrollAreaWidgetContents_2);
        groupBox->setObjectName("groupBox");
        groupBox->setAlignment(Qt::AlignCenter);
        _2 = new QVBoxLayout(groupBox);
        _2->setObjectName("_2");
        _3 = new QHBoxLayout();
        _3->setObjectName("_3");
        spacerItem = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _3->addItem(spacerItem);

        rotateLeftBtn = new QToolButton(groupBox);
        rotateLeftBtn->setObjectName("rotateLeftBtn");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/icons/object-rotate-left.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        rotateLeftBtn->setIcon(icon);
        rotateLeftBtn->setIconSize(QSize(24, 24));

        _3->addWidget(rotateLeftBtn);

        rotateRightBtn = new QToolButton(groupBox);
        rotateRightBtn->setObjectName("rotateRightBtn");
        QIcon icon1;
        icon1.addFile(QString::fromUtf8(":/icons/object-rotate-right.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        rotateRightBtn->setIcon(icon1);
        rotateRightBtn->setIconSize(QSize(24, 24));

        _3->addWidget(rotateRightBtn);

        spacerItem1 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _3->addItem(spacerItem1);


        _2->addLayout(_3);

        _4 = new QHBoxLayout();
        _4->setObjectName("_4");
        spacerItem2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _4->addItem(spacerItem2);

        rotationIndicator = new QLabel(groupBox);
        rotationIndicator->setObjectName("rotationIndicator");
        rotationIndicator->setPixmap(QPixmap(QString::fromUtf8(":/icons/big-up-arrow.svg")));

        _4->addWidget(rotationIndicator);

        spacerItem3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _4->addItem(spacerItem3);


        _2->addLayout(_4);

        _5 = new QHBoxLayout();
        _5->setObjectName("_5");
        spacerItem4 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _5->addItem(spacerItem4);

        resetBtn = new QPushButton(groupBox);
        resetBtn->setObjectName("resetBtn");

        _5->addWidget(resetBtn);

        spacerItem5 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _5->addItem(spacerItem5);


        _2->addLayout(_5);


        gridLayout->addWidget(groupBox, 1, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 2, 1, 1, 1);

        scrollArea_2->setWidget(scrollAreaWidgetContents_2);

        verticalLayout_3->addWidget(scrollArea_2);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName("tab_2");
        verticalLayout_2 = new QVBoxLayout(tab_2);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout_2->setContentsMargins(0, -1, 0, 3);
        scrollArea = new QScrollArea(tab_2);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 637, 442));
        gridLayout_2 = new QGridLayout(scrollAreaWidgetContents);
        gridLayout_2->setObjectName("gridLayout_2");
        layoutModeGroupBox = new QGroupBox(scrollAreaWidgetContents);
        layoutModeGroupBox->setObjectName("layoutModeGroupBox");
        layoutModeGroupBox->setAlignment(Qt::AlignCenter);
        horizontalLayout_25 = new QHBoxLayout(layoutModeGroupBox);
        horizontalLayout_25->setObjectName("horizontalLayout_25");
        horizontalSpacer_57 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_25->addItem(horizontalSpacer_57);

        layoutModeCB = new QComboBox(layoutModeGroupBox);
        layoutModeCB->setObjectName("layoutModeCB");

        horizontalLayout_25->addWidget(layoutModeCB);

        horizontalSpacer_58 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_25->addItem(horizontalSpacer_58);


        gridLayout_2->addWidget(layoutModeGroupBox, 1, 1, 1, 1);

        pageLayoutGroup = new QGroupBox(scrollAreaWidgetContents);
        pageLayoutGroup->setObjectName("pageLayoutGroup");
        pageLayoutGroup->setAlignment(Qt::AlignCenter);
        verticalLayout_8 = new QVBoxLayout(pageLayoutGroup);
        verticalLayout_8->setObjectName("verticalLayout_8");
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        horizontalLayout_2->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalSpacer_5 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_5);

        singlePageUncutBtn = new QToolButton(pageLayoutGroup);
        singlePageUncutBtn->setObjectName("singlePageUncutBtn");
        QIcon icon2;
        icon2.addFile(QString::fromUtf8(":/icons/single_page_uncut.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        singlePageUncutBtn->setIcon(icon2);
        singlePageUncutBtn->setIconSize(QSize(32, 32));
        singlePageUncutBtn->setCheckable(true);
        singlePageUncutBtn->setChecked(true);
        singlePageUncutBtn->setAutoExclusive(true);
        singlePageUncutBtn->setAutoRaise(false);

        horizontalLayout_2->addWidget(singlePageUncutBtn);

        pagePlusOffcutBtn = new QToolButton(pageLayoutGroup);
        pagePlusOffcutBtn->setObjectName("pagePlusOffcutBtn");
        QIcon icon3;
        icon3.addFile(QString::fromUtf8(":/icons/right_page_plus_offcut.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        pagePlusOffcutBtn->setIcon(icon3);
        pagePlusOffcutBtn->setIconSize(QSize(32, 32));
        pagePlusOffcutBtn->setCheckable(true);
        pagePlusOffcutBtn->setChecked(false);
        pagePlusOffcutBtn->setAutoExclusive(true);

        horizontalLayout_2->addWidget(pagePlusOffcutBtn);

        twoPagesBtn = new QToolButton(pageLayoutGroup);
        twoPagesBtn->setObjectName("twoPagesBtn");
        QIcon icon4;
        icon4.addFile(QString::fromUtf8(":/icons/two_pages.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        twoPagesBtn->setIcon(icon4);
        twoPagesBtn->setIconSize(QSize(32, 32));
        twoPagesBtn->setCheckable(true);
        twoPagesBtn->setAutoExclusive(true);

        horizontalLayout_2->addWidget(twoPagesBtn);

        horizontalSpacer_6 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_6);


        verticalLayout_8->addLayout(horizontalLayout_2);


        gridLayout_2->addWidget(pageLayoutGroup, 2, 1, 1, 1);

        horizontalSpacer_7 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_7, 2, 0, 1, 1);

        horizontalSpacer_8 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_8, 2, 2, 1, 1);

        verticalSpacer_3 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_3, 0, 1, 1, 1);

        verticalSpacer_4 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_2->addItem(verticalSpacer_4, 3, 1, 1, 1);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_2->addWidget(scrollArea);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName("tab_3");
        verticalLayout_4 = new QVBoxLayout(tab_3);
        verticalLayout_4->setObjectName("verticalLayout_4");
        verticalLayout_4->setContentsMargins(0, -1, 0, 3);
        scrollArea_3 = new QScrollArea(tab_3);
        scrollArea_3->setObjectName("scrollArea_3");
        scrollArea_3->setWidgetResizable(true);
        scrollAreaWidgetContents_3 = new QWidget();
        scrollAreaWidgetContents_3->setObjectName("scrollAreaWidgetContents_3");
        scrollAreaWidgetContents_3->setGeometry(QRect(0, 0, 208, 116));
        gridLayout_3 = new QGridLayout(scrollAreaWidgetContents_3);
        gridLayout_3->setObjectName("gridLayout_3");
        verticalSpacer_5 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_5, 0, 1, 1, 1);

        horizontalSpacer_9 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_9, 1, 0, 1, 1);

        horizontalSpacer_10 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_10, 1, 2, 1, 1);

        groupBox_3 = new QGroupBox(scrollAreaWidgetContents_3);
        groupBox_3->setObjectName("groupBox_3");
        groupBox_3->setAlignment(Qt::AlignCenter);
        verticalLayout_10 = new QVBoxLayout(groupBox_3);
        verticalLayout_10->setObjectName("verticalLayout_10");
        _6 = new QHBoxLayout();
        _6->setObjectName("_6");
        deskewAutoBtn = new QPushButton(groupBox_3);
        deskewAutoBtn->setObjectName("deskewAutoBtn");
        deskewAutoBtn->setCheckable(true);
        deskewAutoBtn->setChecked(true);
        deskewAutoBtn->setAutoExclusive(true);

        _6->addWidget(deskewAutoBtn);

        deskewManualBtn = new QPushButton(groupBox_3);
        deskewManualBtn->setObjectName("deskewManualBtn");
        deskewManualBtn->setCheckable(true);
        deskewManualBtn->setAutoExclusive(true);

        _6->addWidget(deskewManualBtn);


        verticalLayout_10->addLayout(_6);

        _7 = new QHBoxLayout();
        _7->setObjectName("_7");
        spacerItem6 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _7->addItem(spacerItem6);

        angleSpinBox = new QDoubleSpinBox(groupBox_3);
        angleSpinBox->setObjectName("angleSpinBox");
        angleSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        angleSpinBox->setSingleStep(0.010000000000000);

        _7->addWidget(angleSpinBox);

        spacerItem7 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        _7->addItem(spacerItem7);


        verticalLayout_10->addLayout(_7);


        gridLayout_3->addWidget(groupBox_3, 1, 1, 1, 1);

        verticalSpacer_6 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_3->addItem(verticalSpacer_6, 2, 1, 1, 1);

        scrollArea_3->setWidget(scrollAreaWidgetContents_3);

        verticalLayout_4->addWidget(scrollArea_3);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName("tab_4");
        verticalLayout_5 = new QVBoxLayout(tab_4);
        verticalLayout_5->setObjectName("verticalLayout_5");
        verticalLayout_5->setContentsMargins(0, -1, 0, 3);
        scrollArea_4 = new QScrollArea(tab_4);
        scrollArea_4->setObjectName("scrollArea_4");
        scrollArea_4->setWidgetResizable(true);
        scrollAreaWidgetContents_5 = new QWidget();
        scrollAreaWidgetContents_5->setObjectName("scrollAreaWidgetContents_5");
        scrollAreaWidgetContents_5->setGeometry(QRect(0, 0, 208, 314));
        gridLayout_7 = new QGridLayout(scrollAreaWidgetContents_5);
        gridLayout_7->setObjectName("gridLayout_7");
        verticalSpacer_8 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_7->addItem(verticalSpacer_8, 3, 1, 1, 1);

        verticalSpacer_7 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_7->addItem(verticalSpacer_7, 0, 1, 1, 1);

        groupBox_5 = new QGroupBox(scrollAreaWidgetContents_5);
        groupBox_5->setObjectName("groupBox_5");
        groupBox_5->setAlignment(Qt::AlignCenter);
        gridLayout_5 = new QGridLayout(groupBox_5);
        gridLayout_5->setObjectName("gridLayout_5");
        pageDetectDisableBtn = new QPushButton(groupBox_5);
        pageDetectDisableBtn->setObjectName("pageDetectDisableBtn");
        pageDetectDisableBtn->setCheckable(true);
        pageDetectDisableBtn->setChecked(true);
        pageDetectDisableBtn->setAutoExclusive(true);

        gridLayout_5->addWidget(pageDetectDisableBtn, 0, 0, 1, 2);

        pageDetectOptions = new QGroupBox(groupBox_5);
        pageDetectOptions->setObjectName("pageDetectOptions");
        verticalLayout_11 = new QVBoxLayout(pageDetectOptions);
        verticalLayout_11->setSpacing(4);
        verticalLayout_11->setObjectName("verticalLayout_11");
        verticalLayout_11->setContentsMargins(9, 9, 9, 9);
        fineTuneBtn = new QCheckBox(pageDetectOptions);
        fineTuneBtn->setObjectName("fineTuneBtn");

        verticalLayout_11->addWidget(fineTuneBtn);

        dimensionsWidget = new QWidget(pageDetectOptions);
        dimensionsWidget->setObjectName("dimensionsWidget");
        horizontalLayout_4 = new QHBoxLayout(dimensionsWidget);
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        horizontalLayout_4->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_11 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_11);

        gridLayout_6 = new QGridLayout();
        gridLayout_6->setObjectName("gridLayout_6");
        label = new QLabel(dimensionsWidget);
        label->setObjectName("label");

        gridLayout_6->addWidget(label, 0, 0, 1, 1);

        label_2 = new QLabel(dimensionsWidget);
        label_2->setObjectName("label_2");

        gridLayout_6->addWidget(label_2, 1, 0, 1, 1);

        heightSpinBox = new QDoubleSpinBox(dimensionsWidget);
        heightSpinBox->setObjectName("heightSpinBox");
        heightSpinBox->setDecimals(1);
        heightSpinBox->setMinimum(0.100000000000000);
        heightSpinBox->setMaximum(99999.000000000000000);

        gridLayout_6->addWidget(heightSpinBox, 1, 1, 1, 1);

        widthSpinBox = new QDoubleSpinBox(dimensionsWidget);
        widthSpinBox->setObjectName("widthSpinBox");
        widthSpinBox->setDecimals(1);
        widthSpinBox->setMinimum(0.100000000000000);
        widthSpinBox->setMaximum(99999.000000000000000);

        gridLayout_6->addWidget(widthSpinBox, 0, 1, 1, 1);


        horizontalLayout_4->addLayout(gridLayout_6);

        horizontalSpacer_12 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_12);


        verticalLayout_11->addWidget(dimensionsWidget);


        gridLayout_5->addWidget(pageDetectOptions, 2, 0, 1, 2);

        pageDetectAutoBtn = new QPushButton(groupBox_5);
        pageDetectAutoBtn->setObjectName("pageDetectAutoBtn");
        pageDetectAutoBtn->setCheckable(true);
        pageDetectAutoBtn->setAutoExclusive(true);

        gridLayout_5->addWidget(pageDetectAutoBtn, 1, 0, 1, 1);

        pageDetectManualBtn = new QPushButton(groupBox_5);
        pageDetectManualBtn->setObjectName("pageDetectManualBtn");
        pageDetectManualBtn->setCheckable(true);
        pageDetectManualBtn->setChecked(false);
        pageDetectManualBtn->setAutoExclusive(true);

        gridLayout_5->addWidget(pageDetectManualBtn, 1, 1, 1, 1);


        gridLayout_7->addWidget(groupBox_5, 1, 1, 1, 1);

        groupBox_4 = new QGroupBox(scrollAreaWidgetContents_5);
        groupBox_4->setObjectName("groupBox_4");
        groupBox_4->setAlignment(Qt::AlignCenter);
        gridLayout_4 = new QGridLayout(groupBox_4);
        gridLayout_4->setObjectName("gridLayout_4");
        contentDetectDisableBtn = new QPushButton(groupBox_4);
        contentDetectDisableBtn->setObjectName("contentDetectDisableBtn");
        contentDetectDisableBtn->setCheckable(true);
        contentDetectDisableBtn->setAutoExclusive(true);

        gridLayout_4->addWidget(contentDetectDisableBtn, 0, 1, 1, 2);

        contentDetectAutoBtn = new QPushButton(groupBox_4);
        contentDetectAutoBtn->setObjectName("contentDetectAutoBtn");
        contentDetectAutoBtn->setCheckable(true);
        contentDetectAutoBtn->setChecked(true);
        contentDetectAutoBtn->setAutoExclusive(true);

        gridLayout_4->addWidget(contentDetectAutoBtn, 1, 1, 1, 2);


        gridLayout_7->addWidget(groupBox_4, 2, 1, 1, 1);

        horizontalSpacer_13 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_7->addItem(horizontalSpacer_13, 1, 0, 2, 1);

        horizontalSpacer_14 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_7->addItem(horizontalSpacer_14, 1, 2, 2, 1);

        scrollArea_4->setWidget(scrollAreaWidgetContents_5);

        verticalLayout_5->addWidget(scrollArea_4);

        tabWidget->addTab(tab_4, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName("tab_5");
        verticalLayout_6 = new QVBoxLayout(tab_5);
        verticalLayout_6->setObjectName("verticalLayout_6");
        verticalLayout_6->setContentsMargins(0, -1, 0, 3);
        scrollArea_5 = new QScrollArea(tab_5);
        scrollArea_5->setObjectName("scrollArea_5");
        scrollArea_5->setWidgetResizable(true);
        scrollAreaWidgetContents_6 = new QWidget();
        scrollAreaWidgetContents_6->setObjectName("scrollAreaWidgetContents_6");
        scrollAreaWidgetContents_6->setGeometry(QRect(0, 0, 406, 237));
        gridLayout_10 = new QGridLayout(scrollAreaWidgetContents_6);
        gridLayout_10->setObjectName("gridLayout_10");
        horizontalSpacer_27 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_10->addItem(horizontalSpacer_27, 1, 0, 1, 1);

        verticalLayout_15 = new QVBoxLayout();
        verticalLayout_15->setObjectName("verticalLayout_15");
        marginsGroup = new QGroupBox(scrollAreaWidgetContents_6);
        marginsGroup->setObjectName("marginsGroup");
        marginsGroup->setAlignment(Qt::AlignCenter);
        verticalLayout_14 = new QVBoxLayout(marginsGroup);
        verticalLayout_14->setObjectName("verticalLayout_14");
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        horizontalSpacer_21 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_21);

        autoMargins = new QCheckBox(marginsGroup);
        autoMargins->setObjectName("autoMargins");
        autoMargins->setChecked(false);

        horizontalLayout_8->addWidget(autoMargins);

        horizontalSpacer_22 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_22);


        verticalLayout_14->addLayout(horizontalLayout_8);

        marginsWidget = new QWidget(marginsGroup);
        marginsWidget->setObjectName("marginsWidget");
        horizontalLayout_9 = new QHBoxLayout(marginsWidget);
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        horizontalSpacer_23 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_23);

        gridLayout_9 = new QGridLayout();
        gridLayout_9->setObjectName("gridLayout_9");
        topBottomLink = new QToolButton(marginsWidget);
        topBottomLink->setObjectName("topBottomLink");
        topBottomLink->setMinimumSize(QSize(20, 36));
        topBottomLink->setMaximumSize(QSize(20, 36));
        topBottomLink->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        QIcon icon5;
        icon5.addFile(QString::fromUtf8(":/icons/stock-vchain.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        topBottomLink->setIcon(icon5);
        topBottomLink->setIconSize(QSize(18, 24));

        gridLayout_9->addWidget(topBottomLink, 0, 2, 2, 1);

        label_3 = new QLabel(marginsWidget);
        label_3->setObjectName("label_3");

        gridLayout_9->addWidget(label_3, 0, 0, 1, 1);

        label_4 = new QLabel(marginsWidget);
        label_4->setObjectName("label_4");

        gridLayout_9->addWidget(label_4, 3, 0, 1, 1);

        label_5 = new QLabel(marginsWidget);
        label_5->setObjectName("label_5");

        gridLayout_9->addWidget(label_5, 2, 0, 1, 1);

        leftRightLink = new QToolButton(marginsWidget);
        leftRightLink->setObjectName("leftRightLink");
        leftRightLink->setMinimumSize(QSize(20, 36));
        leftRightLink->setMaximumSize(QSize(20, 36));
        leftRightLink->setStyleSheet(QString::fromUtf8("QToolButton:!hover {\n"
"  padding: 0;\n"
"  border: none;\n"
"  background-color: transparent;\n"
"}\n"
"\n"
"QToolButton:hover {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(base);\n"
"}\n"
"\n"
"QToolButton:pressed {\n"
"  padding: 0;\n"
"  border: 1px solid palette(highlight);\n"
"  background-color: palette(alternative-base);\n"
"}"));
        leftRightLink->setIcon(icon5);
        leftRightLink->setIconSize(QSize(18, 24));
        leftRightLink->setChecked(false);

        gridLayout_9->addWidget(leftRightLink, 2, 2, 2, 1);

        rightMarginSpinBox = new QDoubleSpinBox(marginsWidget);
        rightMarginSpinBox->setObjectName("rightMarginSpinBox");
        rightMarginSpinBox->setDecimals(1);
        rightMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout_9->addWidget(rightMarginSpinBox, 3, 1, 1, 1);

        bottomMarginSpinBox = new QDoubleSpinBox(marginsWidget);
        bottomMarginSpinBox->setObjectName("bottomMarginSpinBox");
        bottomMarginSpinBox->setDecimals(1);
        bottomMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout_9->addWidget(bottomMarginSpinBox, 1, 1, 1, 1);

        topMarginSpinBox = new QDoubleSpinBox(marginsWidget);
        topMarginSpinBox->setObjectName("topMarginSpinBox");
        topMarginSpinBox->setDecimals(1);
        topMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout_9->addWidget(topMarginSpinBox, 0, 1, 1, 1);

        label_6 = new QLabel(marginsWidget);
        label_6->setObjectName("label_6");

        gridLayout_9->addWidget(label_6, 1, 0, 1, 1);

        leftMarginSpinBox = new QDoubleSpinBox(marginsWidget);
        leftMarginSpinBox->setObjectName("leftMarginSpinBox");
        leftMarginSpinBox->setDecimals(1);
        leftMarginSpinBox->setMaximum(9999.000000000000000);

        gridLayout_9->addWidget(leftMarginSpinBox, 2, 1, 1, 1);


        horizontalLayout_9->addLayout(gridLayout_9);

        horizontalSpacer_24 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_24);


        verticalLayout_14->addWidget(marginsWidget);


        verticalLayout_15->addWidget(marginsGroup);

        verticalSpacer_13 = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_15->addItem(verticalSpacer_13);


        gridLayout_10->addLayout(verticalLayout_15, 1, 1, 1, 1);

        horizontalSpacer_28 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_10->addItem(horizontalSpacer_28, 1, 3, 1, 1);

        verticalSpacer_10 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_10->addItem(verticalSpacer_10, 2, 1, 1, 2);

        verticalSpacer_9 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_10->addItem(verticalSpacer_9, 0, 1, 1, 2);

        alignmentGroup = new QGroupBox(scrollAreaWidgetContents_6);
        alignmentGroup->setObjectName("alignmentGroup");
        alignmentGroup->setAlignment(Qt::AlignCenter);
        verticalLayout_12 = new QVBoxLayout(alignmentGroup);
        verticalLayout_12->setObjectName("verticalLayout_12");
        horizontalLayout_26 = new QHBoxLayout();
        horizontalLayout_26->setObjectName("horizontalLayout_26");
        horizontalSpacer_15 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_26->addItem(horizontalSpacer_15);

        alignWithOthersCB = new QCheckBox(alignmentGroup);
        alignWithOthersCB->setObjectName("alignWithOthersCB");
        alignWithOthersCB->setChecked(true);

        horizontalLayout_26->addWidget(alignWithOthersCB);

        horizontalSpacer_16 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_26->addItem(horizontalSpacer_16);


        verticalLayout_12->addLayout(horizontalLayout_26);

        horizontalLayout_20 = new QHBoxLayout();
        horizontalLayout_20->setObjectName("horizontalLayout_20");
        horizontalSpacer_74 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_74);

        gridLayout_15 = new QGridLayout();
        gridLayout_15->setObjectName("gridLayout_15");
        hAlignmentModeLabel = new QLabel(alignmentGroup);
        hAlignmentModeLabel->setObjectName("hAlignmentModeLabel");

        gridLayout_15->addWidget(hAlignmentModeLabel, 0, 0, 1, 1);

        vAlignmentModeLabel = new QLabel(alignmentGroup);
        vAlignmentModeLabel->setObjectName("vAlignmentModeLabel");

        gridLayout_15->addWidget(vAlignmentModeLabel, 1, 0, 1, 1);

        hAlignmentModeCB = new QComboBox(alignmentGroup);
        hAlignmentModeCB->addItem(QString());
        hAlignmentModeCB->addItem(QString());
        hAlignmentModeCB->addItem(QString());
        hAlignmentModeCB->setObjectName("hAlignmentModeCB");

        gridLayout_15->addWidget(hAlignmentModeCB, 0, 1, 1, 1);

        vAlignmentModeCB = new QComboBox(alignmentGroup);
        vAlignmentModeCB->addItem(QString());
        vAlignmentModeCB->addItem(QString());
        vAlignmentModeCB->addItem(QString());
        vAlignmentModeCB->setObjectName("vAlignmentModeCB");

        gridLayout_15->addWidget(vAlignmentModeCB, 1, 1, 1, 1);


        horizontalLayout_20->addLayout(gridLayout_15);

        horizontalSpacer_75 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_20->addItem(horizontalSpacer_75);


        verticalLayout_12->addLayout(horizontalLayout_20);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        horizontalSpacer_17 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_17);

        gridLayout_8 = new QGridLayout();
        gridLayout_8->setSpacing(16);
        gridLayout_8->setObjectName("gridLayout_8");
        alignTopLeftBtn = new QToolButton(alignmentGroup);
        alignTopLeftBtn->setObjectName("alignTopLeftBtn");
        QIcon icon6;
        icon6.addFile(QString::fromUtf8(":/icons/stock-gravity-north-west.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignTopLeftBtn->setIcon(icon6);
        alignTopLeftBtn->setIconSize(QSize(24, 24));
        alignTopLeftBtn->setCheckable(true);
        alignTopLeftBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignTopLeftBtn, 0, 0, 1, 1);

        alignTopBtn = new QToolButton(alignmentGroup);
        alignTopBtn->setObjectName("alignTopBtn");
        QIcon icon7;
        icon7.addFile(QString::fromUtf8(":/icons/stock-gravity-north.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignTopBtn->setIcon(icon7);
        alignTopBtn->setIconSize(QSize(24, 24));
        alignTopBtn->setCheckable(true);
        alignTopBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignTopBtn, 0, 1, 1, 1);

        alignTopRightBtn = new QToolButton(alignmentGroup);
        alignTopRightBtn->setObjectName("alignTopRightBtn");
        QIcon icon8;
        icon8.addFile(QString::fromUtf8(":/icons/stock-gravity-north-east.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignTopRightBtn->setIcon(icon8);
        alignTopRightBtn->setIconSize(QSize(24, 24));
        alignTopRightBtn->setCheckable(true);
        alignTopRightBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignTopRightBtn, 0, 2, 1, 1);

        alignLeftBtn = new QToolButton(alignmentGroup);
        alignLeftBtn->setObjectName("alignLeftBtn");
        QIcon icon9;
        icon9.addFile(QString::fromUtf8(":/icons/stock-gravity-west.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignLeftBtn->setIcon(icon9);
        alignLeftBtn->setIconSize(QSize(24, 24));
        alignLeftBtn->setCheckable(true);
        alignLeftBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignLeftBtn, 1, 0, 1, 1);

        alignCenterBtn = new QToolButton(alignmentGroup);
        alignCenterBtn->setObjectName("alignCenterBtn");
        QIcon icon10;
        icon10.addFile(QString::fromUtf8(":/icons/stock-center.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignCenterBtn->setIcon(icon10);
        alignCenterBtn->setIconSize(QSize(24, 24));
        alignCenterBtn->setCheckable(true);
        alignCenterBtn->setChecked(true);
        alignCenterBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignCenterBtn, 1, 1, 1, 1);

        alignRightBtn = new QToolButton(alignmentGroup);
        alignRightBtn->setObjectName("alignRightBtn");
        QIcon icon11;
        icon11.addFile(QString::fromUtf8(":/icons/stock-gravity-east.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignRightBtn->setIcon(icon11);
        alignRightBtn->setIconSize(QSize(24, 24));
        alignRightBtn->setCheckable(true);
        alignRightBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignRightBtn, 1, 2, 1, 1);

        alignBottomLeftBtn = new QToolButton(alignmentGroup);
        alignBottomLeftBtn->setObjectName("alignBottomLeftBtn");
        QIcon icon12;
        icon12.addFile(QString::fromUtf8(":/icons/stock-gravity-south-west.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignBottomLeftBtn->setIcon(icon12);
        alignBottomLeftBtn->setIconSize(QSize(24, 24));
        alignBottomLeftBtn->setCheckable(true);
        alignBottomLeftBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignBottomLeftBtn, 2, 0, 1, 1);

        alignBottomBtn = new QToolButton(alignmentGroup);
        alignBottomBtn->setObjectName("alignBottomBtn");
        QIcon icon13;
        icon13.addFile(QString::fromUtf8(":/icons/stock-gravity-south.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignBottomBtn->setIcon(icon13);
        alignBottomBtn->setIconSize(QSize(24, 24));
        alignBottomBtn->setCheckable(true);
        alignBottomBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignBottomBtn, 2, 1, 1, 1);

        alignBottomRightBtn = new QToolButton(alignmentGroup);
        alignBottomRightBtn->setObjectName("alignBottomRightBtn");
        QIcon icon14;
        icon14.addFile(QString::fromUtf8(":/icons/stock-gravity-south-east.svg"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        alignBottomRightBtn->setIcon(icon14);
        alignBottomRightBtn->setIconSize(QSize(24, 24));
        alignBottomRightBtn->setCheckable(true);
        alignBottomRightBtn->setAutoExclusive(true);

        gridLayout_8->addWidget(alignBottomRightBtn, 2, 2, 1, 1);


        horizontalLayout_5->addLayout(gridLayout_8);

        horizontalSpacer_18 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_18);


        verticalLayout_12->addLayout(horizontalLayout_5);


        gridLayout_10->addWidget(alignmentGroup, 1, 2, 1, 1);

        scrollArea_5->setWidget(scrollAreaWidgetContents_6);

        verticalLayout_6->addWidget(scrollArea_5);

        tabWidget->addTab(tab_5, QString());
        tab_6 = new QWidget();
        tab_6->setObjectName("tab_6");
        verticalLayout_7 = new QVBoxLayout(tab_6);
        verticalLayout_7->setObjectName("verticalLayout_7");
        verticalLayout_7->setContentsMargins(0, -1, 0, 3);
        scrollArea_6 = new QScrollArea(tab_6);
        scrollArea_6->setObjectName("scrollArea_6");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(scrollArea_6->sizePolicy().hasHeightForWidth());
        scrollArea_6->setSizePolicy(sizePolicy1);
        scrollArea_6->setWidgetResizable(true);
        scrollAreaWidgetContents_7 = new QWidget();
        scrollAreaWidgetContents_7->setObjectName("scrollAreaWidgetContents_7");
        scrollAreaWidgetContents_7->setGeometry(QRect(0, 0, 615, 496));
        gridLayout_14 = new QGridLayout(scrollAreaWidgetContents_7);
        gridLayout_14->setObjectName("gridLayout_14");
        verticalLayout_33 = new QVBoxLayout();
        verticalLayout_33->setObjectName("verticalLayout_33");
        modePanel = new QGroupBox(scrollAreaWidgetContents_7);
        modePanel->setObjectName("modePanel");
        modePanel->setAlignment(Qt::AlignCenter);
        modePanel->setFlat(false);
        modePanel->setCheckable(false);
        gridLayout_13 = new QGridLayout(modePanel);
        gridLayout_13->setObjectName("gridLayout_13");
        verticalLayout_16 = new QVBoxLayout();
        verticalLayout_16->setObjectName("verticalLayout_16");
        commonOptions = new QGroupBox(modePanel);
        commonOptions->setObjectName("commonOptions");
        commonOptions->setStyleSheet(QString::fromUtf8(""));
        commonOptions->setAlignment(Qt::AlignCenter);
        horizontalLayout_14 = new QHBoxLayout(commonOptions);
        horizontalLayout_14->setObjectName("horizontalLayout_14");
        horizontalLayout_14->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_35 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_35);

        verticalLayout_13 = new QVBoxLayout();
        verticalLayout_13->setSpacing(2);
        verticalLayout_13->setObjectName("verticalLayout_13");
        verticalLayout_13->setContentsMargins(5, 5, 5, 5);
        fillOffcutCB = new QCheckBox(commonOptions);
        fillOffcutCB->setObjectName("fillOffcutCB");

        verticalLayout_13->addWidget(fillOffcutCB);

        fillMarginsCB = new QCheckBox(commonOptions);
        fillMarginsCB->setObjectName("fillMarginsCB");

        verticalLayout_13->addWidget(fillMarginsCB);

        equalizeIlluminationCB = new QCheckBox(commonOptions);
        equalizeIlluminationCB->setObjectName("equalizeIlluminationCB");

        verticalLayout_13->addWidget(equalizeIlluminationCB);

        equalizeIlluminationColorCB = new QCheckBox(commonOptions);
        equalizeIlluminationColorCB->setObjectName("equalizeIlluminationColorCB");

        verticalLayout_13->addWidget(equalizeIlluminationColorCB);

        savitzkyGolaySmoothingCB = new QCheckBox(commonOptions);
        savitzkyGolaySmoothingCB->setObjectName("savitzkyGolaySmoothingCB");

        verticalLayout_13->addWidget(savitzkyGolaySmoothingCB);

        morphologicalSmoothingCB = new QCheckBox(commonOptions);
        morphologicalSmoothingCB->setObjectName("morphologicalSmoothingCB");

        verticalLayout_13->addWidget(morphologicalSmoothingCB);


        horizontalLayout_14->addLayout(verticalLayout_13);

        horizontalSpacer_36 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_14->addItem(horizontalSpacer_36);


        verticalLayout_16->addWidget(commonOptions);

        fillingOptions = new QGroupBox(modePanel);
        fillingOptions->setObjectName("fillingOptions");
        fillingOptions->setAlignment(Qt::AlignCenter);
        fillingOptions->setFlat(false);
        verticalLayout_18 = new QVBoxLayout(fillingOptions);
        verticalLayout_18->setObjectName("verticalLayout_18");
        horizontalLayout_23 = new QHBoxLayout();
        horizontalLayout_23->setObjectName("horizontalLayout_23");
        horizontalSpacer_43 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_23->addItem(horizontalSpacer_43);

        fillingColorLabel = new QLabel(fillingOptions);
        fillingColorLabel->setObjectName("fillingColorLabel");

        horizontalLayout_23->addWidget(fillingColorLabel);

        fillingColorBox = new QComboBox(fillingOptions);
        fillingColorBox->setObjectName("fillingColorBox");

        horizontalLayout_23->addWidget(fillingColorBox);

        horizontalSpacer_44 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_23->addItem(horizontalSpacer_44);


        verticalLayout_18->addLayout(horizontalLayout_23);


        verticalLayout_16->addWidget(fillingOptions);

        colorOperationsOptions = new QGroupBox(modePanel);
        colorOperationsOptions->setObjectName("colorOperationsOptions");
        colorOperationsOptions->setStyleSheet(QString::fromUtf8(""));
        colorOperationsOptions->setAlignment(Qt::AlignCenter);
        horizontalLayout_28 = new QHBoxLayout(colorOperationsOptions);
        horizontalLayout_28->setObjectName("horizontalLayout_28");
        horizontalLayout_28->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_60 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_28->addItem(horizontalSpacer_60);

        verticalLayout_31 = new QVBoxLayout();
        verticalLayout_31->setSpacing(2);
        verticalLayout_31->setObjectName("verticalLayout_31");
        verticalLayout_31->setContentsMargins(5, 5, 5, 5);
        colorSegmentationCB = new QCheckBox(colorOperationsOptions);
        colorSegmentationCB->setObjectName("colorSegmentationCB");

        verticalLayout_31->addWidget(colorSegmentationCB);

        segmenterOptionsWidget = new QWidget(colorOperationsOptions);
        segmenterOptionsWidget->setObjectName("segmenterOptionsWidget");
        verticalLayout_32 = new QVBoxLayout(segmenterOptionsWidget);
        verticalLayout_32->setObjectName("verticalLayout_32");
        verticalLayout_32->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_29 = new QHBoxLayout();
        horizontalLayout_29->setSpacing(0);
        horizontalLayout_29->setObjectName("horizontalLayout_29");
        horizontalSpacer_61 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_29->addItem(horizontalSpacer_61);

        rLabel = new QLabel(segmenterOptionsWidget);
        rLabel->setObjectName("rLabel");
        rLabel->setMargin(1);
        rLabel->setIndent(0);

        horizontalLayout_29->addWidget(rLabel);

        redAdjustmentSB = new QSpinBox(segmenterOptionsWidget);
        redAdjustmentSB->setObjectName("redAdjustmentSB");
        redAdjustmentSB->setFrame(false);
        redAdjustmentSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        redAdjustmentSB->setMinimum(-99);
        redAdjustmentSB->setMaximum(99);
        redAdjustmentSB->setValue(0);

        horizontalLayout_29->addWidget(redAdjustmentSB);

        gLabel = new QLabel(segmenterOptionsWidget);
        gLabel->setObjectName("gLabel");
        gLabel->setMargin(1);
        gLabel->setIndent(3);

        horizontalLayout_29->addWidget(gLabel);

        greenAdjustmentSB = new QSpinBox(segmenterOptionsWidget);
        greenAdjustmentSB->setObjectName("greenAdjustmentSB");
        greenAdjustmentSB->setFrame(false);
        greenAdjustmentSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        greenAdjustmentSB->setMinimum(-99);
        greenAdjustmentSB->setMaximum(99);
        greenAdjustmentSB->setValue(0);

        horizontalLayout_29->addWidget(greenAdjustmentSB);

        bLabel = new QLabel(segmenterOptionsWidget);
        bLabel->setObjectName("bLabel");
        bLabel->setMargin(1);
        bLabel->setIndent(3);

        horizontalLayout_29->addWidget(bLabel);

        blueAdjustmentSB = new QSpinBox(segmenterOptionsWidget);
        blueAdjustmentSB->setObjectName("blueAdjustmentSB");
        blueAdjustmentSB->setFrame(false);
        blueAdjustmentSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        blueAdjustmentSB->setMinimum(-99);
        blueAdjustmentSB->setMaximum(99);
        blueAdjustmentSB->setValue(0);

        horizontalLayout_29->addWidget(blueAdjustmentSB);

        horizontalSpacer_62 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_29->addItem(horizontalSpacer_62);


        verticalLayout_32->addLayout(horizontalLayout_29);

        horizontalLayout_30 = new QHBoxLayout();
        horizontalLayout_30->setObjectName("horizontalLayout_30");
        horizontalSpacer_63 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_30->addItem(horizontalSpacer_63);

        reduceNoiseLabel = new QLabel(segmenterOptionsWidget);
        reduceNoiseLabel->setObjectName("reduceNoiseLabel");

        horizontalLayout_30->addWidget(reduceNoiseLabel);

        reduceNoiseSB = new QSpinBox(segmenterOptionsWidget);
        reduceNoiseSB->setObjectName("reduceNoiseSB");
        reduceNoiseSB->setFrame(true);
        reduceNoiseSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        reduceNoiseSB->setMinimum(0);
        reduceNoiseSB->setMaximum(999);
        reduceNoiseSB->setValue(0);

        horizontalLayout_30->addWidget(reduceNoiseSB);

        horizontalSpacer_64 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_30->addItem(horizontalSpacer_64);


        verticalLayout_32->addLayout(horizontalLayout_30);


        verticalLayout_31->addWidget(segmenterOptionsWidget);

        posterizeCB = new QCheckBox(colorOperationsOptions);
        posterizeCB->setObjectName("posterizeCB");

        verticalLayout_31->addWidget(posterizeCB);

        posterizeOptionsWidget = new QWidget(colorOperationsOptions);
        posterizeOptionsWidget->setObjectName("posterizeOptionsWidget");
        verticalLayout_34 = new QVBoxLayout(posterizeOptionsWidget);
        verticalLayout_34->setObjectName("verticalLayout_34");
        verticalLayout_34->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_31 = new QHBoxLayout();
        horizontalLayout_31->setObjectName("horizontalLayout_31");
        horizontalLayout_31->setContentsMargins(-1, 0, -1, -1);
        horizontalSpacer_65 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_31->addItem(horizontalSpacer_65);

        posterizeLevelLabel = new QLabel(posterizeOptionsWidget);
        posterizeLevelLabel->setObjectName("posterizeLevelLabel");

        horizontalLayout_31->addWidget(posterizeLevelLabel);

        posterizeLevelSB = new QSpinBox(posterizeOptionsWidget);
        posterizeLevelSB->setObjectName("posterizeLevelSB");
        posterizeLevelSB->setFrame(true);
        posterizeLevelSB->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        posterizeLevelSB->setMinimum(2);
        posterizeLevelSB->setMaximum(255);

        horizontalLayout_31->addWidget(posterizeLevelSB);

        horizontalSpacer_66 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_31->addItem(horizontalSpacer_66);


        verticalLayout_34->addLayout(horizontalLayout_31);

        horizontalLayout_33 = new QHBoxLayout();
        horizontalLayout_33->setObjectName("horizontalLayout_33");
        horizontalLayout_33->setContentsMargins(-1, 0, -1, -1);
        horizontalSpacer_70 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_33->addItem(horizontalSpacer_70);

        posterizeNormalizationCB = new QCheckBox(posterizeOptionsWidget);
        posterizeNormalizationCB->setObjectName("posterizeNormalizationCB");

        horizontalLayout_33->addWidget(posterizeNormalizationCB);

        horizontalSpacer_71 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_33->addItem(horizontalSpacer_71);


        verticalLayout_34->addLayout(horizontalLayout_33);

        horizontalLayout_32 = new QHBoxLayout();
        horizontalLayout_32->setObjectName("horizontalLayout_32");
        horizontalLayout_32->setContentsMargins(-1, 0, -1, -1);
        horizontalSpacer_67 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_32->addItem(horizontalSpacer_67);

        posterizeForceBwCB = new QCheckBox(posterizeOptionsWidget);
        posterizeForceBwCB->setObjectName("posterizeForceBwCB");

        horizontalLayout_32->addWidget(posterizeForceBwCB);

        horizontalSpacer_68 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_32->addItem(horizontalSpacer_68);


        verticalLayout_34->addLayout(horizontalLayout_32);


        verticalLayout_31->addWidget(posterizeOptionsWidget);


        horizontalLayout_28->addLayout(verticalLayout_31);

        horizontalSpacer_69 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_28->addItem(horizontalSpacer_69);


        verticalLayout_16->addWidget(colorOperationsOptions);

        verticalSpacer_14 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_16->addItem(verticalSpacer_14);


        gridLayout_13->addLayout(verticalLayout_16, 1, 0, 1, 1);

        verticalLayout_27 = new QVBoxLayout();
        verticalLayout_27->setObjectName("verticalLayout_27");
        verticalLayout_27->setSizeConstraint(QLayout::SetDefaultConstraint);
        thresholdOptions = new QGroupBox(modePanel);
        thresholdOptions->setObjectName("thresholdOptions");
        thresholdOptions->setAlignment(Qt::AlignCenter);
        thresholdOptions->setFlat(false);
        verticalLayout_19 = new QVBoxLayout(thresholdOptions);
        verticalLayout_19->setObjectName("verticalLayout_19");
        horizontalLayout_22 = new QHBoxLayout();
        horizontalLayout_22->setObjectName("horizontalLayout_22");
        horizontalSpacer_42 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_22->addItem(horizontalSpacer_42);

        thresholMethodLabel = new QLabel(thresholdOptions);
        thresholMethodLabel->setObjectName("thresholMethodLabel");

        horizontalLayout_22->addWidget(thresholMethodLabel);

        thresholdMethodBox = new QComboBox(thresholdOptions);
        thresholdMethodBox->setObjectName("thresholdMethodBox");

        horizontalLayout_22->addWidget(thresholdMethodBox);

        horizontalSpacer_41 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_22->addItem(horizontalSpacer_41);


        verticalLayout_19->addLayout(horizontalLayout_22);

        binarizationOptions = new QStackedWidget(thresholdOptions);
        binarizationOptions->setObjectName("binarizationOptions");
        sizePolicy.setHeightForWidth(binarizationOptions->sizePolicy().hasHeightForWidth());
        binarizationOptions->setSizePolicy(sizePolicy);
        page = new QWidget();
        page->setObjectName("page");
        sizePolicy.setHeightForWidth(page->sizePolicy().hasHeightForWidth());
        page->setSizePolicy(sizePolicy);
        verticalLayout_30 = new QVBoxLayout(page);
        verticalLayout_30->setObjectName("verticalLayout_30");
        verticalLayout_24 = new QVBoxLayout();
        verticalLayout_24->setObjectName("verticalLayout_24");
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setSpacing(6);
        horizontalLayout_11->setObjectName("horizontalLayout_11");
        horizontalLayout_11->setSizeConstraint(QLayout::SetDefaultConstraint);
        horizontalSpacer_45 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_45);

        thresholLabel = new QLabel(page);
        thresholLabel->setObjectName("thresholLabel");
        thresholLabel->setFrameShape(QFrame::NoFrame);

        horizontalLayout_11->addWidget(thresholLabel);

        horizontalSpacer_46 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_46);


        verticalLayout_24->addLayout(horizontalLayout_11);

        thresholdSlider = new QSlider(page);
        thresholdSlider->setObjectName("thresholdSlider");
        thresholdSlider->setMinimum(-30);
        thresholdSlider->setMaximum(30);
        thresholdSlider->setPageStep(5);
        thresholdSlider->setTracking(true);
        thresholdSlider->setOrientation(Qt::Horizontal);
        thresholdSlider->setInvertedAppearance(false);
        thresholdSlider->setInvertedControls(false);
        thresholdSlider->setTickPosition(QSlider::TicksBelow);

        verticalLayout_24->addWidget(thresholdSlider);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setSpacing(0);
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        lighterThresholdLink = new QLabel(page);
        lighterThresholdLink->setObjectName("lighterThresholdLink");

        horizontalLayout_10->addWidget(lighterThresholdLink);

        neutralThresholdBtn = new QToolButton(page);
        neutralThresholdBtn->setObjectName("neutralThresholdBtn");
        neutralThresholdBtn->setCursor(QCursor(Qt::CursorShape::PointingHandCursor));
        neutralThresholdBtn->setIconSize(QSize(12, 12));
        neutralThresholdBtn->setToolButtonStyle(Qt::ToolButtonIconOnly);
        neutralThresholdBtn->setAutoRaise(true);
        neutralThresholdBtn->setArrowType(Qt::UpArrow);

        horizontalLayout_10->addWidget(neutralThresholdBtn);

        darkerThresholdLink = new QLabel(page);
        darkerThresholdLink->setObjectName("darkerThresholdLink");
        darkerThresholdLink->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        horizontalLayout_10->addWidget(darkerThresholdLink);


        verticalLayout_24->addLayout(horizontalLayout_10);


        verticalLayout_30->addLayout(verticalLayout_24);

        verticalSpacer_18 = new QSpacerItem(1, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_30->addItem(verticalSpacer_18);

        binarizationOptions->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        sizePolicy.setHeightForWidth(page_2->sizePolicy().hasHeightForWidth());
        page_2->setSizePolicy(sizePolicy);
        verticalLayout_25 = new QVBoxLayout(page_2);
        verticalLayout_25->setObjectName("verticalLayout_25");
        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName("horizontalLayout_12");
        horizontalSpacer_20 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_20);

        gridLayout_11 = new QGridLayout();
        gridLayout_11->setObjectName("gridLayout_11");
        sauvolaCoefLabel = new QLabel(page_2);
        sauvolaCoefLabel->setObjectName("sauvolaCoefLabel");

        gridLayout_11->addWidget(sauvolaCoefLabel, 1, 0, 1, 1);

        sauvolaWindowSize = new QSpinBox(page_2);
        sauvolaWindowSize->setObjectName("sauvolaWindowSize");
        sauvolaWindowSize->setMinimum(5);
        sauvolaWindowSize->setMaximum(9999);

        gridLayout_11->addWidget(sauvolaWindowSize, 0, 1, 1, 1);

        windowSizeLabel = new QLabel(page_2);
        windowSizeLabel->setObjectName("windowSizeLabel");

        gridLayout_11->addWidget(windowSizeLabel, 0, 0, 1, 1);

        sauvolaCoef = new QDoubleSpinBox(page_2);
        sauvolaCoef->setObjectName("sauvolaCoef");
        sauvolaCoef->setMinimum(0.010000000000000);
        sauvolaCoef->setMaximum(9.990000000000000);
        sauvolaCoef->setSingleStep(0.010000000000000);

        gridLayout_11->addWidget(sauvolaCoef, 1, 1, 1, 1);


        horizontalLayout_12->addLayout(gridLayout_11);

        horizontalSpacer_19 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_19);


        verticalLayout_25->addLayout(horizontalLayout_12);

        verticalSpacer_19 = new QSpacerItem(1, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_25->addItem(verticalSpacer_19);

        binarizationOptions->addWidget(page_2);
        page_3 = new QWidget();
        page_3->setObjectName("page_3");
        sizePolicy.setHeightForWidth(page_3->sizePolicy().hasHeightForWidth());
        page_3->setSizePolicy(sizePolicy);
        verticalLayout_26 = new QVBoxLayout(page_3);
        verticalLayout_26->setObjectName("verticalLayout_26");
        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName("horizontalLayout_17");
        horizontalSpacer_26 = new QSpacerItem(9, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_26);

        gridLayout_12 = new QGridLayout();
        gridLayout_12->setObjectName("gridLayout_12");
        wolfWindowSize = new QSpinBox(page_3);
        wolfWindowSize->setObjectName("wolfWindowSize");
        wolfWindowSize->setMinimum(5);
        wolfWindowSize->setMaximum(9999);

        gridLayout_12->addWidget(wolfWindowSize, 0, 1, 1, 1);

        windowSizeLabel_2 = new QLabel(page_3);
        windowSizeLabel_2->setObjectName("windowSizeLabel_2");

        gridLayout_12->addWidget(windowSizeLabel_2, 0, 0, 1, 1);

        lowerBound = new QSpinBox(page_3);
        lowerBound->setObjectName("lowerBound");
        lowerBound->setMinimum(1);
        lowerBound->setMaximum(254);

        gridLayout_12->addWidget(lowerBound, 1, 1, 1, 1);

        upperBoundLabel = new QLabel(page_3);
        upperBoundLabel->setObjectName("upperBoundLabel");

        gridLayout_12->addWidget(upperBoundLabel, 2, 0, 1, 1);

        lowerBoundLabel = new QLabel(page_3);
        lowerBoundLabel->setObjectName("lowerBoundLabel");

        gridLayout_12->addWidget(lowerBoundLabel, 1, 0, 1, 1);

        upperBound = new QSpinBox(page_3);
        upperBound->setObjectName("upperBound");
        upperBound->setMinimum(1);
        upperBound->setMaximum(254);

        gridLayout_12->addWidget(upperBound, 2, 1, 1, 1);

        wolfCoefLabel = new QLabel(page_3);
        wolfCoefLabel->setObjectName("wolfCoefLabel");

        gridLayout_12->addWidget(wolfCoefLabel, 3, 0, 1, 1);

        wolfCoef = new QDoubleSpinBox(page_3);
        wolfCoef->setObjectName("wolfCoef");
        wolfCoef->setMinimum(0.010000000000000);
        wolfCoef->setMaximum(9.990000000000000);
        wolfCoef->setSingleStep(0.010000000000000);

        gridLayout_12->addWidget(wolfCoef, 3, 1, 1, 1);


        horizontalLayout_17->addLayout(gridLayout_12);

        horizontalSpacer_25 = new QSpacerItem(9, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_17->addItem(horizontalSpacer_25);


        verticalLayout_26->addLayout(horizontalLayout_17);

        verticalSpacer_20 = new QSpacerItem(1, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_26->addItem(verticalSpacer_20);

        binarizationOptions->addWidget(page_3);

        verticalLayout_19->addWidget(binarizationOptions);


        verticalLayout_27->addWidget(thresholdOptions);

        pictureShapeOptions = new QGroupBox(modePanel);
        pictureShapeOptions->setObjectName("pictureShapeOptions");
        pictureShapeOptions->setAlignment(Qt::AlignCenter);
        verticalLayout_20 = new QVBoxLayout(pictureShapeOptions);
        verticalLayout_20->setObjectName("verticalLayout_20");
        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName("horizontalLayout_15");
        horizontalSpacer_37 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_37);

        pictureShapeSelector = new QComboBox(pictureShapeOptions);
        pictureShapeSelector->setObjectName("pictureShapeSelector");

        horizontalLayout_15->addWidget(pictureShapeSelector);

        horizontalSpacer_38 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_38);


        verticalLayout_20->addLayout(horizontalLayout_15);

        pictureShapeSensitivityOptions = new QWidget(pictureShapeOptions);
        pictureShapeSensitivityOptions->setObjectName("pictureShapeSensitivityOptions");
        horizontalLayout_16 = new QHBoxLayout(pictureShapeSensitivityOptions);
        horizontalLayout_16->setObjectName("horizontalLayout_16");
        horizontalLayout_16->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer_39 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_39);

        pictureShapeSensitivityLabel = new QLabel(pictureShapeSensitivityOptions);
        pictureShapeSensitivityLabel->setObjectName("pictureShapeSensitivityLabel");

        horizontalLayout_16->addWidget(pictureShapeSensitivityLabel);

        pictureShapeSensitivitySB = new QSpinBox(pictureShapeSensitivityOptions);
        pictureShapeSensitivitySB->setObjectName("pictureShapeSensitivitySB");
        pictureShapeSensitivitySB->setMinimum(0);
        pictureShapeSensitivitySB->setMaximum(100);

        horizontalLayout_16->addWidget(pictureShapeSensitivitySB);

        horizontalSpacer_40 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_40);


        verticalLayout_20->addWidget(pictureShapeSensitivityOptions);

        horizontalLayout_34 = new QHBoxLayout();
        horizontalLayout_34->setObjectName("horizontalLayout_34");
        horizontalSpacer_72 = new QSpacerItem(1, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_34->addItem(horizontalSpacer_72);

        higherSearchSensitivityCB = new QCheckBox(pictureShapeOptions);
        higherSearchSensitivityCB->setObjectName("higherSearchSensitivityCB");

        horizontalLayout_34->addWidget(higherSearchSensitivityCB);

        horizontalSpacer_73 = new QSpacerItem(1, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_34->addItem(horizontalSpacer_73);


        verticalLayout_20->addLayout(horizontalLayout_34);


        verticalLayout_27->addWidget(pictureShapeOptions);

        verticalSpacer_21 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_27->addItem(verticalSpacer_21);


        gridLayout_13->addLayout(verticalLayout_27, 1, 1, 1, 1);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName("horizontalLayout_13");
        horizontalSpacer_33 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_33);

        colorModeSelector = new QComboBox(modePanel);
        colorModeSelector->setObjectName("colorModeSelector");

        horizontalLayout_13->addWidget(colorModeSelector);

        horizontalSpacer_34 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_34);


        gridLayout_13->addLayout(horizontalLayout_13, 0, 0, 1, 2);


        verticalLayout_33->addWidget(modePanel);

        verticalSpacer_15 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_33->addItem(verticalSpacer_15);


        gridLayout_14->addLayout(verticalLayout_33, 1, 1, 1, 1);

        verticalLayout_17 = new QVBoxLayout();
        verticalLayout_17->setObjectName("verticalLayout_17");
        dpiPanel = new QGroupBox(scrollAreaWidgetContents_7);
        dpiPanel->setObjectName("dpiPanel");
        dpiPanel->setStyleSheet(QString::fromUtf8(""));
        dpiPanel->setAlignment(Qt::AlignCenter);
        dpiPanel->setFlat(false);
        horizontalLayout_7 = new QHBoxLayout(dpiPanel);
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        horizontalSpacer_30 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_30);

        dpiSelector = new QComboBox(dpiPanel);
        dpiSelector->setObjectName("dpiSelector");
        dpiSelector->setInsertPolicy(QComboBox::NoInsert);
        dpiSelector->setDuplicatesEnabled(true);

        horizontalLayout_7->addWidget(dpiSelector);

        horizontalSpacer_29 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_29);


        verticalLayout_17->addWidget(dpiPanel);

        splittingOptions = new QGroupBox(scrollAreaWidgetContents_7);
        splittingOptions->setObjectName("splittingOptions");
        splittingOptions->setStyleSheet(QString::fromUtf8(""));
        splittingOptions->setAlignment(Qt::AlignCenter);
        verticalLayout_21 = new QVBoxLayout(splittingOptions);
        verticalLayout_21->setObjectName("verticalLayout_21");
        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setObjectName("horizontalLayout_18");
        horizontalSpacer_47 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_18->addItem(horizontalSpacer_47);

        verticalLayout_22 = new QVBoxLayout();
        verticalLayout_22->setObjectName("verticalLayout_22");
        splittingCB = new QCheckBox(splittingOptions);
        splittingCB->setObjectName("splittingCB");

        verticalLayout_22->addWidget(splittingCB);

        bwForegroundRB = new QRadioButton(splittingOptions);
        bwForegroundRB->setObjectName("bwForegroundRB");
        bwForegroundRB->setChecked(true);

        verticalLayout_22->addWidget(bwForegroundRB);

        horizontalLayout_27 = new QHBoxLayout();
        horizontalLayout_27->setObjectName("horizontalLayout_27");
        horizontalSpacer_59 = new QSpacerItem(15, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Minimum);

        horizontalLayout_27->addItem(horizontalSpacer_59);

        originalBackgroundCB = new QCheckBox(splittingOptions);
        originalBackgroundCB->setObjectName("originalBackgroundCB");

        horizontalLayout_27->addWidget(originalBackgroundCB);


        verticalLayout_22->addLayout(horizontalLayout_27);

        colorForegroundRB = new QRadioButton(splittingOptions);
        colorForegroundRB->setObjectName("colorForegroundRB");

        verticalLayout_22->addWidget(colorForegroundRB);


        horizontalLayout_18->addLayout(verticalLayout_22);

        horizontalSpacer_48 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_18->addItem(horizontalSpacer_48);


        verticalLayout_21->addLayout(horizontalLayout_18);


        verticalLayout_17->addWidget(splittingOptions);

        despecklePanel = new QGroupBox(scrollAreaWidgetContents_7);
        despecklePanel->setObjectName("despecklePanel");
        despecklePanel->setAlignment(Qt::AlignCenter);
        despecklePanel->setFlat(false);
        verticalLayout_23 = new QVBoxLayout(despecklePanel);
        verticalLayout_23->setSpacing(6);
        verticalLayout_23->setObjectName("verticalLayout_23");
        verticalLayout_23->setContentsMargins(15, 9, 15, -1);
        verticalLayout_35 = new QVBoxLayout();
        verticalLayout_35->setSpacing(4);
        verticalLayout_35->setObjectName("verticalLayout_35");
        verticalLayout_35->setContentsMargins(0, 0, 0, -1);
        horizontalLayout_37 = new QHBoxLayout();
        horizontalLayout_37->setObjectName("horizontalLayout_37");
        horizontalSpacer_51 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_37->addItem(horizontalSpacer_51);

        verticalLayout_36 = new QVBoxLayout();
        verticalLayout_36->setObjectName("verticalLayout_36");
        despeckleCB = new QCheckBox(despecklePanel);
        despeckleCB->setObjectName("despeckleCB");

        verticalLayout_36->addWidget(despeckleCB);


        horizontalLayout_37->addLayout(verticalLayout_36);

        horizontalSpacer_52 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_37->addItem(horizontalSpacer_52);


        verticalLayout_35->addLayout(horizontalLayout_37);

        despeckleSlider = new QSlider(despecklePanel);
        despeckleSlider->setObjectName("despeckleSlider");
        despeckleSlider->setMinimum(10);
        despeckleSlider->setMaximum(30);
        despeckleSlider->setSingleStep(2);
        despeckleSlider->setPageStep(5);
        despeckleSlider->setValue(10);
        despeckleSlider->setOrientation(Qt::Horizontal);

        verticalLayout_35->addWidget(despeckleSlider);


        verticalLayout_23->addLayout(verticalLayout_35);


        verticalLayout_17->addWidget(despecklePanel);

        dewarpingPanel = new QGroupBox(scrollAreaWidgetContents_7);
        dewarpingPanel->setObjectName("dewarpingPanel");
        dewarpingPanel->setAlignment(Qt::AlignCenter);
        dewarpingPanel->setFlat(false);
        verticalLayout_28 = new QVBoxLayout(dewarpingPanel);
        verticalLayout_28->setObjectName("verticalLayout_28");
        horizontalLayout_19 = new QHBoxLayout();
        horizontalLayout_19->setObjectName("horizontalLayout_19");
        horizontalSpacer_49 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_19->addItem(horizontalSpacer_49);

        dewarpingModeCB = new QComboBox(dewarpingPanel);
        dewarpingModeCB->setObjectName("dewarpingModeCB");

        horizontalLayout_19->addWidget(dewarpingModeCB);

        horizontalSpacer_50 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_19->addItem(horizontalSpacer_50);


        verticalLayout_28->addLayout(horizontalLayout_19);

        horizontalLayout_21 = new QHBoxLayout();
        horizontalLayout_21->setObjectName("horizontalLayout_21");
        horizontalSpacer_53 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_21->addItem(horizontalSpacer_53);

        dewarpingPostDeskewCB = new QCheckBox(dewarpingPanel);
        dewarpingPostDeskewCB->setObjectName("dewarpingPostDeskewCB");

        horizontalLayout_21->addWidget(dewarpingPostDeskewCB);

        horizontalSpacer_54 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_21->addItem(horizontalSpacer_54);


        verticalLayout_28->addLayout(horizontalLayout_21);


        verticalLayout_17->addWidget(dewarpingPanel);

        depthPerceptionPanel = new QGroupBox(scrollAreaWidgetContents_7);
        depthPerceptionPanel->setObjectName("depthPerceptionPanel");
        depthPerceptionPanel->setAlignment(Qt::AlignCenter);
        depthPerceptionPanel->setFlat(false);
        verticalLayout_29 = new QVBoxLayout(depthPerceptionPanel);
        verticalLayout_29->setObjectName("verticalLayout_29");
        verticalLayout_29->setContentsMargins(15, -1, 15, -1);
        depthPerceptionSlider = new QSlider(depthPerceptionPanel);
        depthPerceptionSlider->setObjectName("depthPerceptionSlider");
        depthPerceptionSlider->setMinimum(10);
        depthPerceptionSlider->setMaximum(30);
        depthPerceptionSlider->setPageStep(5);
        depthPerceptionSlider->setValue(10);
        depthPerceptionSlider->setOrientation(Qt::Horizontal);

        verticalLayout_29->addWidget(depthPerceptionSlider);


        verticalLayout_17->addWidget(depthPerceptionPanel);

        verticalSpacer_22 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout_17->addItem(verticalSpacer_22);


        gridLayout_14->addLayout(verticalLayout_17, 1, 2, 1, 1);

        horizontalSpacer_31 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_14->addItem(horizontalSpacer_31, 1, 0, 1, 1);

        verticalSpacer_17 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_14->addItem(verticalSpacer_17, 0, 1, 1, 1);

        horizontalSpacer_32 = new QSpacerItem(1, 1, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        gridLayout_14->addItem(horizontalSpacer_32, 1, 3, 1, 1);

        verticalSpacer_16 = new QSpacerItem(1, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout_14->addItem(verticalSpacer_16, 2, 1, 1, 2);

        scrollArea_6->setWidget(scrollAreaWidgetContents_7);

        verticalLayout_7->addWidget(scrollArea_6);

        tabWidget->addTab(tab_6, QString());

        verticalLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(DefaultParamsDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(DefaultParamsDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, DefaultParamsDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, DefaultParamsDialog, qOverload<>(&QDialog::reject));

        tabWidget->setCurrentIndex(0);
        hAlignmentModeCB->setCurrentIndex(1);
        vAlignmentModeCB->setCurrentIndex(1);
        binarizationOptions->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(DefaultParamsDialog);
    } // setupUi

    void retranslateUi(QDialog *DefaultParamsDialog)
    {
        DefaultParamsDialog->setWindowTitle(QCoreApplication::translate("DefaultParamsDialog", "Default parameters", nullptr));
        profileSaveButton->setText(QCoreApplication::translate("DefaultParamsDialog", "Save", nullptr));
        profileDeleteButton->setText(QCoreApplication::translate("DefaultParamsDialog", "Delete", nullptr));
        label_7->setText(QCoreApplication::translate("DefaultParamsDialog", "Units:", nullptr));
        unitsLabel->setText(QString());
        groupBox->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Rotate", nullptr));
        rotateLeftBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        rotateRightBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        rotationIndicator->setText(QString());
        resetBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Reset", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("DefaultParamsDialog", "Fix Orientation", nullptr));
        layoutModeGroupBox->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Mode", nullptr));
        pageLayoutGroup->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Page Layout", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("DefaultParamsDialog", "Split Pages", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Deskew", nullptr));
        deskewAutoBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Auto", nullptr));
        deskewManualBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Manual", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("DefaultParamsDialog", "Deskew", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Page Box", nullptr));
        pageDetectDisableBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Disable", nullptr));
        pageDetectOptions->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Options", nullptr));
#if QT_CONFIG(tooltip)
        fineTuneBtn->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Shift with corners while they are in black. ", nullptr));
#endif // QT_CONFIG(tooltip)
        fineTuneBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Fine Tune Page Corners", nullptr));
        label->setText(QCoreApplication::translate("DefaultParamsDialog", "Width", nullptr));
        label_2->setText(QCoreApplication::translate("DefaultParamsDialog", "Height", nullptr));
        pageDetectAutoBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Auto", nullptr));
        pageDetectManualBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Manual", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Content Box", nullptr));
        contentDetectDisableBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Disable", nullptr));
        contentDetectAutoBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "Auto", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("DefaultParamsDialog", "Select Content", nullptr));
        marginsGroup->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Margins", nullptr));
        autoMargins->setText(QCoreApplication::translate("DefaultParamsDialog", "Auto Margins", nullptr));
        topBottomLink->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        label_3->setText(QCoreApplication::translate("DefaultParamsDialog", "Top", nullptr));
        label_4->setText(QCoreApplication::translate("DefaultParamsDialog", "Right", nullptr));
        label_5->setText(QCoreApplication::translate("DefaultParamsDialog", "Left", nullptr));
        leftRightLink->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        label_6->setText(QCoreApplication::translate("DefaultParamsDialog", "Bottom", nullptr));
        alignmentGroup->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Alignment", nullptr));
        alignWithOthersCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Match size with other pages", nullptr));
        hAlignmentModeLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Horizontal mode:", nullptr));
        vAlignmentModeLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Vertical mode:", nullptr));
        hAlignmentModeCB->setItemText(0, QCoreApplication::translate("DefaultParamsDialog", "Auto", "auto"));
        hAlignmentModeCB->setItemText(1, QCoreApplication::translate("DefaultParamsDialog", "Manual", "manual"));
        hAlignmentModeCB->setItemText(2, QCoreApplication::translate("DefaultParamsDialog", "Original", "original"));

        vAlignmentModeCB->setItemText(0, QCoreApplication::translate("DefaultParamsDialog", "Auto", "auto"));
        vAlignmentModeCB->setItemText(1, QCoreApplication::translate("DefaultParamsDialog", "Manual", "manual"));
        vAlignmentModeCB->setItemText(2, QCoreApplication::translate("DefaultParamsDialog", "Original", "original"));

        alignTopLeftBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignTopBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignTopRightBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignLeftBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignCenterBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignRightBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignBottomLeftBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignBottomBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        alignBottomRightBtn->setText(QCoreApplication::translate("DefaultParamsDialog", "...", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("DefaultParamsDialog", "Margins", nullptr));
        modePanel->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Mode", nullptr));
        commonOptions->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Options", nullptr));
        fillOffcutCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Fill offcut", nullptr));
        fillMarginsCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Fill margins", nullptr));
#if QT_CONFIG(tooltip)
        equalizeIlluminationCB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Normalize illumination before binarization.", nullptr));
#endif // QT_CONFIG(tooltip)
        equalizeIlluminationCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Equalize illumination (B&&W)", nullptr));
#if QT_CONFIG(tooltip)
        equalizeIlluminationColorCB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Normalize illumination in color mode / in picture zones in mixed mode.", nullptr));
#endif // QT_CONFIG(tooltip)
        equalizeIlluminationColorCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Equalize illumination (Color)", nullptr));
        savitzkyGolaySmoothingCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Savitzky-Golay smoothing", nullptr));
        morphologicalSmoothingCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Morphological smoothing", nullptr));
        fillingOptions->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Filling", nullptr));
        fillingColorLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Color: ", nullptr));
        colorOperationsOptions->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Color operations", nullptr));
#if QT_CONFIG(tooltip)
        colorSegmentationCB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Split the image into color segments and colorize b&w mask.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        colorSegmentationCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        colorSegmentationCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Color segmentation", nullptr));
        rLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "R", nullptr));
#if QT_CONFIG(tooltip)
        redAdjustmentSB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Red component adjustment. A negative value means the segmenter will be more sensitive to red and vice versa for a positive one.", nullptr));
#endif // QT_CONFIG(tooltip)
        gLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "G", nullptr));
#if QT_CONFIG(tooltip)
        greenAdjustmentSB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Green component adjustment. A negative value means the segmenter will be more sensitive to green and vice versa for a positive one.", nullptr));
#endif // QT_CONFIG(tooltip)
        bLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "B", nullptr));
#if QT_CONFIG(tooltip)
        blueAdjustmentSB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Blue component adjustment. A negative value means the segmenter will be more sensitive to blue and vice versa for a positive one.", nullptr));
#endif // QT_CONFIG(tooltip)
        reduceNoiseLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Reduce noise:", nullptr));
#if QT_CONFIG(tooltip)
        reduceNoiseSB->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        posterizeCB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Reduce the number of colors of the output image by grouping similar colors.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        posterizeCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        posterizeCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Posterize", nullptr));
        posterizeLevelLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Level:", nullptr));
#if QT_CONFIG(tooltip)
        posterizeLevelSB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Lower value means lower count of colors in the output image, values between 2 and 6 inclusive guarantee an indexed image.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        posterizeNormalizationCB->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        posterizeNormalizationCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        posterizeNormalizationCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Normalize", nullptr));
#if QT_CONFIG(tooltip)
        posterizeForceBwCB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Make dark and light gray gradients black and white respectively.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(statustip)
        posterizeForceBwCB->setStatusTip(QString());
#endif // QT_CONFIG(statustip)
        posterizeForceBwCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Force b&&w", nullptr));
        thresholdOptions->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Threshold", nullptr));
        thresholMethodLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Method:", nullptr));
        thresholLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "0", nullptr));
        lighterThresholdLink->setText(QCoreApplication::translate("DefaultParamsDialog", "Thinner", nullptr));
        neutralThresholdBtn->setText(QString());
        darkerThresholdLink->setText(QCoreApplication::translate("DefaultParamsDialog", "Thicker", nullptr));
        sauvolaCoefLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Coef:", nullptr));
#if QT_CONFIG(tooltip)
        sauvolaWindowSize->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "The dimensions of a pixel neighborhood to consider.", nullptr));
#endif // QT_CONFIG(tooltip)
        windowSizeLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Window size:", nullptr));
#if QT_CONFIG(tooltip)
        sauvolaCoef->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Default value is 0.34.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        wolfWindowSize->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "The dimensions of a pixel neighborhood to consider.", nullptr));
#endif // QT_CONFIG(tooltip)
        windowSizeLabel_2->setText(QCoreApplication::translate("DefaultParamsDialog", "Window size:", nullptr));
#if QT_CONFIG(tooltip)
        lowerBound->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "The minimum possible gray level that can be made white.", nullptr));
#endif // QT_CONFIG(tooltip)
        upperBoundLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Upper Bound: ", nullptr));
        lowerBoundLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Lower bound: ", nullptr));
#if QT_CONFIG(tooltip)
        upperBound->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "The maximum possible gray level that can be made black.", nullptr));
#endif // QT_CONFIG(tooltip)
        wolfCoefLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Coeff:", nullptr));
#if QT_CONFIG(tooltip)
        wolfCoef->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Default value is 0.3.", nullptr));
#endif // QT_CONFIG(tooltip)
        pictureShapeOptions->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Picture Shape", nullptr));
        pictureShapeSensitivityLabel->setText(QCoreApplication::translate("DefaultParamsDialog", "Sensitivity (%):", nullptr));
        higherSearchSensitivityCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Higher search sensivity", nullptr));
#if QT_CONFIG(tooltip)
        dpiPanel->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        dpiPanel->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Output Resolution (DPI)", nullptr));
        splittingOptions->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Splitting", nullptr));
        splittingCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Split output", nullptr));
        bwForegroundRB->setText(QCoreApplication::translate("DefaultParamsDialog", "B&&W foreground", nullptr));
#if QT_CONFIG(tooltip)
        originalBackgroundCB->setToolTip(QCoreApplication::translate("DefaultParamsDialog", "Save the original background of the foreground layer.", nullptr));
#endif // QT_CONFIG(tooltip)
        originalBackgroundCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Original background", nullptr));
        colorForegroundRB->setText(QCoreApplication::translate("DefaultParamsDialog", "Color foreground", nullptr));
        despecklePanel->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Despeckling", nullptr));
        despeckleCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Despeckle", nullptr));
        dewarpingPanel->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Dewarping", nullptr));
        dewarpingPostDeskewCB->setText(QCoreApplication::translate("DefaultParamsDialog", "Post deskew", nullptr));
        depthPerceptionPanel->setTitle(QCoreApplication::translate("DefaultParamsDialog", "Depth perception", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_6), QCoreApplication::translate("DefaultParamsDialog", "Output", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DefaultParamsDialog: public Ui_DefaultParamsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DEFAULTPARAMSDIALOG_H
